
package org.netbeans.modules.asm.core.editor;

import java.awt.Color;
import java.awt.Font;
import java.util.Map;
import org.netbeans.editor.BaseKit;
import org.netbeans.editor.Coloring;
import org.netbeans.editor.Settings;
import org.netbeans.editor.SettingsDefaults;
import org.netbeans.editor.SettingsNames;
import org.netbeans.editor.SettingsUtil;
import org.netbeans.editor.TokenCategory;
import org.netbeans.editor.TokenContext;
import org.netbeans.editor.TokenContextPath;

/**
 *
 * @author am204439
 */
public class AsmSettingsInitializer extends Settings.AbstractInitializer {
    
    public static final String NAME = "asm-settings-initializer"; // NOI18N
    
    public AsmSettingsInitializer() {
        super(NAME);
    }
    
    public void updateSettingsMap(Class kitClass, Map settingsMap) {
        if (kitClass == BaseKit.class) {
            new AsmTokenColoringInitializer().updateSettingsMap(kitClass, settingsMap);
        }
        
        if (kitClass == AsmEditorKit.class) {
            SettingsUtil.updateListSetting(
                    settingsMap,
                    SettingsNames.TOKEN_CONTEXT_LIST,
                    new TokenContext[] { AsmTokenContext.context }
            );
        }
    }
    
  
    private static class AsmTokenColoringInitializer extends SettingsUtil.TokenColoringInitializer {
        
        /** Bold font. */
        private static final Font boldFont = SettingsDefaults.defaultFont.deriveFont(Font.BOLD);
        /** Italic font. */
        private static final Font italicFont = SettingsDefaults.defaultFont.deriveFont(Font.ITALIC);
        
       
        private static final Coloring instrColoring = new Coloring(boldFont, Coloring.FONT_MODE_APPLY_STYLE, Color.red, null);
        
        private static final Coloring regsColoring = new Coloring(boldFont, Coloring.FONT_MODE_APPLY_STYLE, Color.blue, null);
        
        private static final Coloring unknwColoring = new Coloring(null, Color.magenta, null);
        
        private static final Coloring commentColoring = new Coloring(null, Color.DARK_GRAY, null);
       
        private static final Coloring emptyColoring = new Coloring(null, null, null);
        
                
        public AsmTokenColoringInitializer() {
            super(AsmTokenContext.context);
        }
        
        
        /** Gets token coloring. */
        public Object getTokenColoring(TokenContextPath tokenContextPath,
                TokenCategory tokenIDOrCategory, boolean printingSet) {
            
            if(!printingSet) {
                int tokenID = tokenIDOrCategory.getNumericID();
                
                if (tokenID == AsmTokenContext.INSTRUCTION_ID) {
                    return instrColoring;
                }   
                else if (tokenID == AsmTokenContext.REGISTER_ID) {
                    return regsColoring;
                }
                else if (tokenID == AsmTokenContext.COMMENT_ID){
                    return commentColoring;
                }
                else if (tokenID == AsmTokenContext.COMMENT_ID){
                    return commentColoring;
                }
                
                return emptyColoring;                    
            } 
            else { // printing set
                return SettingsUtil.defaultPrintColoringEvaluator;
            }
            
            //return null;
        }
        
    }    
}
