/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.coverage.actionperformers;

import java.util.Properties;
import org.netbeans.api.progress.ProgressHandle;
import org.netbeans.api.progress.ProgressHandleFactory;
import org.netbeans.api.project.Project;
import org.netbeans.modules.coverage.ProjectCoverageActivityOnAction;
import org.netbeans.modules.coverage.config.Config;
import org.netbeans.modules.coverage.exceptions.PluginNotConfiguredException;
import org.netbeans.modules.coverage.instrumentation.CoveragePreparers;
import org.netbeans.modules.coverage.instrumentation.ProjectCoveragePreparer;
import org.netbeans.modules.coverage.ui.CommonDialogsCollection;
import org.netbeans.modules.coverage.utils.ModuleActivityModes;
import org.openide.util.NbBundle;

/**
 *The class is used to turn coverage collection on
 **/
public final class ProjectCoverageActivityOn implements Runnable{
    
    private Project project = null;
    private String commandName = "";
            
    
    public ProjectCoverageActivityOn(Project project, String name){
        this.project = project;
        this.commandName = name;
    }
    
   
    /**
     *This method turns coverage plugin (and coverage collection) on and sets all necessary 
     * properties.
     **/
    public void run(){
        ProgressHandle progress = ProgressHandleFactory.createHandle(commandName); // NOI18N
        progress.start();
        
        try{
            //Set properties
            Properties props = null;
            try{
                Config config = Config.getInstanceForProject(project);
                props = config.getPluginProperties();
            }catch(PluginNotConfiguredException e){
                props = new Properties();
                props.put("coverage.coverageFile", NbBundle.getMessage(ProjectCoverageActivityOnAction.class, "PROP_coverage_coverageFile"));
                props.put("coverage.templateFile", NbBundle.getMessage(ProjectCoverageActivityOnAction.class, "PROP_coverage_templateFile"));
                
            }
            props.put("coverage.activity",ModuleActivityModes.ON);
            //set project type
            props.put("project.type", Config.getProjectType(project));
            
            Config.storeProjectProperties(project, props);
            
            //prepare the project for coverage metering
            ProjectCoveragePreparer preparer = CoveragePreparers.getPreparerForProject(project);
            if(preparer != null){
                preparer.prepare();
            } else{
                //show message
            }
            
            //coloring all open documents
            RefreshProjectOpenedEditors.refreshProject(project);
        
        }catch(Exception e){
            e.printStackTrace();
        }
        
        if (progress != null) progress.finish();       
    }
    
    
}

