/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.coverage.annotations;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Properties;
import org.netbeans.api.project.FileOwnerQuery;
import org.netbeans.api.project.Project;
import org.netbeans.modules.coverage.ProjectCoverageActivityOnAction;
import org.netbeans.modules.coverage.actionperformers.ClassMarkupCoverage;
import org.netbeans.modules.coverage.config.Config;
import org.netbeans.modules.coverage.exceptions.PluginNotConfiguredException;
import org.netbeans.modules.coverage.utils.ModuleActivityModes;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;

/**
 * The class is listening for all opened editors and adds markup to them if needed
 * We use CoverageModuleInstall class to subscribe to the events
 *
 *The class is a singleton as we want only one listener instance to exist and respond to events
 *
 * TODO: this class will be moved to another package later
 * @author mikk@netbeans.org
 */
public class OpenEditorListener implements PropertyChangeListener{
   
    private OpenEditorListener(){
    }
    
    private static OpenEditorListener instance = new OpenEditorListener();

        
    /**
     * Returns the listener's instance
     * @return Listener's instance
     */
    public static OpenEditorListener getInstance(){
        return instance;
    } 
    
    public void propertyChange(PropertyChangeEvent evt) {
        if (!evt.getPropertyName().equals(TopComponent.Registry.PROP_OPENED)){
            return;
        }
        
        Node[] nodes = TopComponent.getRegistry().getActivatedNodes();
        for(int i=0;i<nodes.length;i++){
            //check the dataobject for node is OK
            DataObject dataObj = (DataObject) nodes[i].getLookup().lookup(DataObject.class);
            if(dataObj==null){
                continue;
            }
            
            //check the file for dataobject is a java file
            FileObject fileObj = dataObj.getPrimaryFile();
            if (fileObj==null || !fileObj.getExt().equals("java")){
                continue;
            }
            
            //check the java file is in the project with activated coverage
            Project project = FileOwnerQuery.getOwner(fileObj);
            if (project == null){
                continue;
            }
            Properties props = null;
            try{
                Config config = Config.getInstanceForProject(project);
                props = config.getPluginProperties();
            }catch(PluginNotConfiguredException e){
                continue;                
            }
            if (!props.get("coverage.activity").equals(ModuleActivityModes.ON)){
                continue;
            }
            
            new Thread(new ClassMarkupCoverage(nodes[i], 
                    NbBundle.getMessage(ProjectCoverageActivityOnAction.class, "CTL_Class_MarkupCoverage"))).start();
            break;
        }
    }
    
    
    
}
