/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.coverage.data;

import java.io.File;
import org.netbeans.api.project.Project;
import org.netbeans.modules.coverage.config.Config;
import org.netbeans.modules.coverage.exceptions.PluginNotConfiguredException;
import org.netbeans.modules.coverage.exceptions.UnableToLoadCoverageDataException;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;

/**
 * This is the class to create concrete Coverage Data Factories
 * We plan to use different coverage storage repositories that may require specific factories
 *Vurrently we just read coverage data file stored on the disk
 *
 * @author Andrey Korostelev
 */
public class AbstractCoverageDataFactory {
  
    /**
     * returns ICoverageDataFactory object created basing on project config data.<br>
     * It will be local storage by default, DataBase or TCF according to config.
     * @param project Project to get coverage data for
     * @throws org.netbeans.modules.coverage.exceptions.PluginNotConfiguredException Exception thrown if plugin is not correctly configured
     * @throws org.netbeans.modules.coverage.exceptions.UnableToLoadCoverageDataException Exception is thrown if coverage data can't be obtained
     * @return Returns the concrete coverage data factory
     */
    public static ICoverageDataFactory getConcreteDataFactory(Project project) 
    throws PluginNotConfiguredException, UnableToLoadCoverageDataException{
        Config config = Config.getInstanceForProject(project);
        
        FileObject projectDir = project.getProjectDirectory();
        
        //System.err.println("coverage data factory: "+projectDir.getPath());

        FileObject covFileObj = projectDir.getFileObject(config.getPluginProperties().getProperty("coverage.coverageFile"));
        if(covFileObj==null){
            throw new UnableToLoadCoverageDataException();
        }
        File covFile = FileUtil.toFile(covFileObj);
        
        FileObject templFileObject = projectDir.getFileObject(config.getPluginProperties().getProperty("coverage.templateFile"));
        if(templFileObject==null){
            throw new UnableToLoadCoverageDataException();
        }        
        File templFile = FileUtil.toFile(templFileObject);

        if (!covFile.exists() || !templFile.exists())
            throw new UnableToLoadCoverageDataException();
        return new org.netbeans.modules.coverage.data.local.CoverageDataFactory(covFile, templFile);
    }
    
     
    
}
