/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * BPELBuilderTests.java
 *
 */

package org.netbeans.test.bpel.creator;
import java.io.File;
import org.netbeans.jellytools.JellyTestCase;
import org.netbeans.jellytools.ProjectsTabOperator;
import org.netbeans.jellytools.nodes.Node;
import org.netbeans.jellytools.nodes.ProjectRootNode;
import org.netbeans.jemmy.EventTool;
import org.netbeans.jemmy.JemmyProperties;
import org.netbeans.jemmy.operators.JButtonOperator;
import org.netbeans.jemmy.operators.JDialogOperator;
import org.netbeans.junit.NbTestSuite;
import org.netbeans.orchlib.core.operators.DesignViewOperator;
import org.netbeans.orchlib.core.operators.MultiviewEditorOperator;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.ext.Util;
import org.netbeans.orchlib.ext.creator.Creator;

/**
 *
 * @author mikk@netbeans.org
 */
public class BPELBuilderTests extends MultiTestSuite{
    
    private EventTool eventTool = new EventTool();
    private File[] bpelFiles;
    private int index = -1;
    private String folder = "";
    private MultiTestCase testCase = new CreatorTest();
    
    public static NbTestSuite suite() {
        NbTestSuite suite = new BPELBuilderTests();
        return suite;
    }
    
    private String PROJECT_PATH = System.getProperty("xtest.data")+
            File.separator+"data";
    
    protected MultiTestCase nextTestCase() {
        index++;
        if (index<bpelFiles.length){
            return testCase;
        }else{
            return null;
        }
    }
    
    
    public void prepare(){
        
        if (this.folder!=null){
            PROJECT_PATH = PROJECT_PATH+File.separator+folder;
        }
        System.err.println(PROJECT_PATH);
        File BPELs = new File(PROJECT_PATH);
        System.err.println("Files will be searched in"+PROJECT_PATH);
        bpelFiles = BPELs.listFiles();
    }
    
    
    
    public class CreatorTest extends MultiTestCase{
        
        public CreatorTest(){}
        
        public void cleanup(){
            //closeAllModal();
            closeSaveDlg();
        }
        
        
        public void execute(){
            String fileName = bpelFiles[index].getName();
            getLog().println(fileName);
            int dotPos = fileName.indexOf('.');
            fileName = fileName.substring(0, dotPos);
            
            String bpelFileName = bpelFiles[index].getPath();
            String projectName = fileName;
            
            try{
                Util.createOrchestrationProject(projectName, System.getProperty("xtest.workdir"));
                Util.createBPEL(projectName, projectName);
                MultiviewEditorOperator editor = openProjectBPEL(projectName, projectName+".bpel");
                
                DesignViewOperator view = editor.getDesignView();
                PatternOperator process = view.getRootPatternOperator();
                
                //clean the project
                PatternOperator sequence = new PatternOperator(process, PatternTypes.SEQUENCE, 0);
                sequence.select();
                sequence.getPopup().pushMenuNoBlock("Delete");
                eventTool.waitNoEvent(1500);
                
                new Creator().run(bpelFileName, process, "c:\\"+bpelFiles[index].getName());
                
            }catch(Exception e){
                e.printStackTrace(getLog());
                long timeoutValDlg = JemmyProperties.getCurrentTimeout("DialogWaiter.WaitDialogTimeout");
                try{
                    JemmyProperties.setCurrentTimeout("DialogWaiter.WaitDialogTimeout", 1000);
                    new JDialogOperator("Exception").close();
                    //fail(78244, "Unexpected Exception dialog was found");
                }catch(Exception excp){
                    fail("Test failed");
                }finally{
                    JellyTestCase.closeAllModal();                    
                    JemmyProperties.setCurrentTimeout("DialogWaiter.WaitDialogTimeout", timeoutValDlg);
                }
            }finally {
                closeSaveDlg();
            }
        }
        
    }
    
    public  MultiviewEditorOperator openProjectBPEL(String projectName, String bpelName){
        Node node = getNode("Process Files|"+projectName, projectName);
        node.tree().clickOnPath(node.getTreePath(),2);
        eventTool.waitNoEvent(2000);
        MultiviewEditorOperator editor = new MultiviewEditorOperator(bpelName);
        return editor;
    }
    
    
    private  Node getNode(String nodePath, String projectName){
        ProjectsTabOperator pto = ProjectsTabOperator.invoke();
        ProjectRootNode root = new ProjectRootNode(pto.tree(),projectName);
        Node node = new Node(root, nodePath);
        return node;
    }
    
    
    
    public void closeSaveDlg(){
        new Thread(new Runnable() {
            public void run() {
                long timeoutValDlg = JemmyProperties.getCurrentTimeout("DialogWaiter.WaitDialogTimeout");
                try{
                    JemmyProperties.setCurrentTimeout("DialogWaiter.WaitDialogTimeout", 10000);
                    JDialogOperator saveDlg = new JDialogOperator("Save");
                    new JButtonOperator(saveDlg, "Save All").pushNoBlock();
                }catch(Exception e){} finally{
                    JemmyProperties.setCurrentTimeout("DialogWaiter.WaitDialogTimeout", timeoutValDlg);
                }
            }
        }).start();
    }

    public void setFolder(String folder) {
        this.folder = folder;
    }
    
    
    
    
}
