/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.test.bpel.roundtrip;

import java.util.ArrayList;
import javax.swing.tree.TreePath;
import org.netbeans.orchlib.core.operators.DesignViewOperator;
import org.netbeans.orchlib.core.operators.MultiviewEditorOperator;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.core.operators.SourceViewOperator;
import org.netbeans.orchlib.exceptions.BadXMLSourceException;
import org.netbeans.orchlib.ext.operators.ExtJTreeOperator;
import org.netbeans.jellytools.ProjectsTabOperator;
import org.netbeans.jellytools.nodes.ProjectRootNode;
import org.netbeans.jemmy.EventTool;
import org.netbeans.jemmy.JemmyProperties;
import org.netbeans.jemmy.TimeoutExpiredException;
import org.netbeans.junit.NbTestSuite;
import org.w3c.dom.DOMException;
import org.w3c.dom.Node;
import org.netbeans.junit.MultiTestCase;
import org.netbeans.junit.MultiTestSuite;

/**
 *
 * @author ap153252
 */
public class ReverseTRS extends MultiTestSuite{

    private final static String projName = "TravelReservationService";
    private final static String bpelProjName = projName+".bpel";
    private EventTool eventTool = new EventTool();
    
    ProjectsTabOperator pto = ProjectsTabOperator.invoke();
    MultiviewEditorOperator editor = null;
    Node modelRoot = null;
    Node currentNode = null;
    DesignViewOperator view = null;
    ArrayList<Element> path = new ArrayList();
    int index = 0;
    MultiTestCase mainTest = new MainTest();
    
    public static NbTestSuite suite() {
        return new ReverseTRS(); 
    }
    
    public void prepare(){
        //open a diagram
        ExtJTreeOperator projTree = new ExtJTreeOperator(pto);
        TreePath projPath = new ProjectRootNode(projTree,projName).getTreePath();
        projTree.expandPath(projPath, "Process Files", bpelProjName);
        TreePath bpel = projTree.findPathEx(projPath, "Process Files", bpelProjName);
        projTree.clickOnPath(bpel ,2);
        eventTool.waitNoEvent(5000);
        editor = new MultiviewEditorOperator(bpelProjName);
        //get the source
        SourceViewOperator source = editor.getSourceView();
        eventTool.waitNoEvent(500);
        //get a source model (Node Main sequence) 
        String xPath = "/process/sequence";
        try{
            modelRoot = source.findNodeByXpath(xPath);
            currentNode = modelRoot;
        }
        catch(BadXMLSourceException e){
            throw new Error(e.toString());
        }
        //switch to design view
        view = editor.getDesignView();
    }
    public void cleanup(){
        //close the diagram
    }
        
    protected MultiTestCase nextTestCase() {
            //create a path to the element
            Node tempNode = null;
            if(currentNode.hasChildNodes()){
                System.out.println("Go to Child");
                tempNode = currentNode.getFirstChild();
                while(tempNode != null && getPatternType(tempNode) == PatternTypes.ANY){
                    tempNode = tempNode.getNextSibling();
                }
            }
            if(tempNode == null){
                shortenPath();//go to siblings
                System.out.println("Go to siblings");
            }
            while(tempNode == null){
                tempNode = currentNode.getNextSibling();
                System.out.println("Go to next sibling");
                if(tempNode == null){
                    currentNode = currentNode.getParentNode();//go to parent
                    System.out.println("Go to parent");
                    shortenPath();
                    if(currentNode == modelRoot)
                        return null;
                }
                else{
                    if(getPatternType(tempNode) == PatternTypes.ANY){
                        System.out.println("Sibling is ANY");
                        currentNode = tempNode;
                        tempNode = null;
                    }
                }
            }

            if(modelRoot != tempNode){
                currentNode = tempNode;
                path.add(createElement(currentNode));
                index++;
                String testName = "CheckElement"+getPatternType(currentNode).name()+index;
                mainTest.setName(testName);
                return mainTest;
            }
            //if there are no paths any more then
            return null;
    }
    
    
    public class MainTest extends MultiTestCase{

        public void execute(){
            //check that the element contains the same nodes as in the model
            //select all the path from the root till the element
            try{
//                DesignViewOperator view = editor.getDesignView();
                PatternOperator patRoot = view.getRootPatternOperator();
                PatternOperator patParent = new PatternOperator(patRoot, PatternTypes.SEQUENCE, 0);
                for(Element element: path){
                    PatternOperator elementPattern = null;
                    long waitPatternTime = JemmyProperties.getCurrentTimeouts().getTimeout("PatternOperator.WaitPatternTime");
                    JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", 1000);
                    try{
                        if(element.name != null)
                            elementPattern = new PatternOperator(patParent, element.type, element.name, 0);
                        else
                            elementPattern = new PatternOperator(patParent, element.type, 0);
                        elementPattern.select();
                        eventTool.waitNoEvent(200);
                        patParent = elementPattern;
                    }
                    catch(TimeoutExpiredException e){
                        fail("There is no element "+element.name+" of type "+element.type);
                    }
                    finally{
                        JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", waitPatternTime);
                    }
                }
            }
            catch(Exception e){
                e.printStackTrace(getLog());
                fail(e.getMessage());
            }
            
        }
        
    }
    
    
    Element createElement(Node node){
        PatternTypes type = getPatternType(node);
        String name = null;
        try{
            Node attrNode = node.getAttributes().getNamedItem("name");
            if(attrNode != null)
                name = attrNode.getNodeValue();
        }
        catch(DOMException e){
        }
        
        return new Element(type, name);
    }
    
    void shortenPath(){
        if(path.size()>0)
            path.remove(path.size()-1);

    }

    PatternTypes getPatternType(Node node){
        String stype = node.getNodeName().toUpperCase();
        //to do something extra
        if(stype.equals("CASE"))
            stype = "SWITCH_CASE";
        if(stype.equals("ONMESSAGE"))
            stype = "ON_MESSAGE";
        if(stype.equals("ONALARM"))
            stype = "ON_ALARM";
        if(stype.equals("THEN"))
            stype = "IF_THEN";
        
        PatternTypes type = PatternTypes.ANY;
        try{
            type = PatternTypes.valueOf(stype);
        }
        catch(IllegalArgumentException e){}
        return type;
    }

    class Element{
        PatternTypes type;
        String name;
        Element(PatternTypes _type, String _name){
            type = _type;
            name = _name;
        }
    }
}
