/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.test.bpel.roundtrip;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import javax.swing.tree.TreePath;
import org.netbeans.orchlib.core.operators.MultiviewEditorOperator;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.core.operators.SourceElementOperator;
import org.netbeans.orchlib.core.operators.SourceViewOperator;
import org.netbeans.orchlib.exceptions.ElementNotFoundException;
import org.netbeans.orchlib.ext.Util;
import org.netbeans.orchlib.ext.operators.ExtJTreeOperator;
import org.netbeans.orchlib.utils.ITitlesAndLabels;
import org.netbeans.orchlib.utils.TypeSpecificProperties;
import org.netbeans.jellytools.EditorOperator;
import org.netbeans.jellytools.ProjectsTabOperator;
import org.netbeans.jellytools.nodes.ProjectRootNode;
import org.netbeans.jellytools.properties.PropertySheetOperator;
import org.netbeans.jellytools.actions.Action;
import org.netbeans.jellytools.properties.Property;
import org.netbeans.jemmy.EventTool;
import org.netbeans.jemmy.JemmyProperties;
import org.netbeans.jemmy.TimeoutExpiredException;
import org.netbeans.jemmy.operators.Operator.StringComparator;
import org.w3c.dom.NodeList;
import org.netbeans.junit.ParametricTestCase;
import org.netbeans.junit.ParametricTestSuite;


/**
 *
 * @author ap153252
 */
public class Roundtrip extends ParametricTestSuite{
    
    private EventTool eventTool = new EventTool();
    private final String workDir = System.getProperty("xtest.workdir");
    private final static String projName = "OrchRoundTripProject1";
    private final static String bpelProjName = projName+".bpel";
    ProjectsTabOperator pto = ProjectsTabOperator.invoke();
    MultiviewEditorOperator editor = null;
    PropertySheetOperator pso = null;
    long waitPropertyTime;

    PatternTypes type = null;
    PatternHandler handler = null;
    
    int index = 0;
    
    public static ParametricTestSuite suite() {
        return new Roundtrip();
    }

    protected Object[] getParameters(){
        PatternTypes[] types = TypeSpecificProperties.getPalettePatternsTypes();
        ArrayList params = new ArrayList(types.length);
        //exclude some types from testing
        for(PatternTypes tp: types){
            switch (tp){
                case PARTNER_LINK:  break;
                default:            params.add(tp);
            }
        }
        return params.toArray();
//        return new Object[]{PatternTypes.FOR_EACH, PatternTypes.FOR_EACH, PatternTypes.FOR_EACH, PatternTypes.FOR_EACH,
//        PatternTypes.FOR_EACH, PatternTypes.FOR_EACH, PatternTypes.FOR_EACH, PatternTypes.FOR_EACH, PatternTypes.FOR_EACH,
//        PatternTypes.FOR_EACH};
//        return new Object[]{PatternTypes.FOR_EACH};
    }
    
    protected ParametricTestCase[] cases(Object parameter){
        
        index++;
        type = (PatternTypes)parameter;
//        String typeName = TypeSpecificProperties.getPatternPaletteName(type);
        handler = PatternHandler.createPatternHandler(type, editor);
        ParametricTestCase[] testCases = new ParametricTestCase[]{new CheckCreate(), new CheckDelete(), new CheckRestore(), new CheckDelete()};
        return testCases;
    }

    public void prepare(){
        //set timeouts
        System.out.println("########  "+getName()+"  #######");
        waitPropertyTime = JemmyProperties.getCurrentTimeout("Waiter.WaitingTime");
        //check if the project already exists
        ExtJTreeOperator projTree = new ExtJTreeOperator(pto);
        org.netbeans.jellytools.nodes.Node projnode = null;

        long waitNodeTime = JemmyProperties.getCurrentTimeout("JTreeOperator.WaitNextNodeTimeout");
        JemmyProperties.setCurrentTimeout("JTreeOperator.WaitNextNodeTimeout", 2000);
        try{
            projnode = new ProjectRootNode(projTree,projName);
        }
        catch(TimeoutExpiredException e){}
        finally{
            JemmyProperties.setCurrentTimeout("JTreeOperator.WaitNextNodeTimeout", waitNodeTime);
        }
        if(projnode == null){
            //create a new project
            Util.createOrchestrationProject(projName, workDir);
            eventTool.waitNoEvent(1000);
            Util.createBPEL(projName, projName);
        }

        //open the project for editing
        TreePath projPath = new ProjectRootNode(projTree,projName).getTreePath();
        projTree.expandPath(projPath, "Process Files", bpelProjName);
        TreePath bpel = projTree.findPathEx(projPath, "Process Files", bpelProjName);
        projTree.clickOnPath(bpel ,2);
        eventTool.waitNoEvent(5000);

        editor = new MultiviewEditorOperator(bpelProjName);

        new Action(ITitlesAndLabels.MAIN_MENU_WINDOW_PROPERTIES, null).performMenu();
        pso = PropertySheetOperator.invoke();
        pso.setComparator(new StringComparator(){
            public boolean equals(String caption, String match){
                if(caption == null || match == null)
                    return false;
                return caption.equals(match);
            }
        });
//            try{
//            SourceViewOperator src = editor.getSourceView();
//            eventTool.waitNoEvent(500);
//            SourceElementOperator.findSourceElementEx(src, "empty").deleteElement();
//            editor.getDesignView();
//            }catch(ElementNotFoundException e){}
    }
    public void cleanup(){
        Thread closer = Util.startQuestionCloser();
        EditorOperator.closeDiscardAll();
        Util.stopQuestionCloser(closer);
    }
    ////////////////////////////////////////////TEST CASES////////////////////////////////////
    //Create
    public class CheckCreate extends ParametricTestCase{

        public void parametrize(Object parameter){
            setName(getName()+handler.getPatternName()+index);
        }
        public void execute(){
            log("Hello! It's a roundtrip test.");
            NodeList result = null;
            try{
                handler.createElement();
                eventTool.waitNoEvent(1000);
                //find the element 
                PatternOperator elementPattern = handler.findElement();
                //test the element
                elementPattern.select();
                eventTool.waitNoEvent(500);
                //check for exception
                if(Util.closeException()){
                    fail("An Exception occured while processing the element of type "+handler.getPatternName()+".");
                }
                //get all properties
                Property[] props = Util.getProperties(pso);
                //check source
                SourceElementOperator element = elementPattern.goToSourceElement();
                eventTool.waitNoEvent(500);
                log("found element "+element.getTagName());
                log("Element "+elementPattern.getType().name()+" has following properties:");
                for(Property prop: props){
                    String propname = prop.getName();
                    String propval = prop.getValue();
                    boolean isAttribute = true;
                    log(propname+" = "+propval);
                    if(isAllowed(propname) && propval != null && !propval.equals("") && !propval.equals("N/A")){
                        String val = element.getAttribute(convertToAttr(propname));
                        if(val == null){
                            try{
                                val = element.getChild(convertToAttr(propname)).getContent();
                                isAttribute = false;
                            }catch(ElementNotFoundException e){
                            }
                            if(val == null){
                                fail("Element of type "+handler.getPatternName()+" has no attribute "+convertToAttr(propname)+
                                    " with value "+propval+" according Properties");
                            }
                        }
                        //attribute or child exist
                        //
                        val = removeNSPrefix(val);
                        if(!val.equals(propval)){
                            fail("Element of type "+handler.getPatternName()+" has attribute "+convertToAttr(propname)+"="+val+
                                    " when expected "+propval+" according Properties");
                        } 
                        handler.setProperty(propname, propval);
                        handler.setPropertyType(propname, isAttribute);
                    }
                    
                }
                log("FirstTest: Element "+elementPattern.getName()+" passed");
            }catch(Exception e){
                e.printStackTrace(getLog());
                fail(e.getMessage());
            }
        }
        
    }
    
    //Delete
    public class CheckDelete extends ParametricTestCase{

        public void parametrize(Object parameter){
            setName(getName()+handler.getPatternName()+index);
        }
        public void execute(){
            try{
                //find element
                PatternOperator elementPattern = handler.findElement();
                //go to source
                SourceElementOperator element = elementPattern.goToSourceElement();
                eventTool.waitNoEvent(500);
                //remove from source
                element.deleteElement();
                eventTool.waitNoEvent(500);
                //try to find in design view
                try{
                    elementPattern = handler.findElement();
                    if(elementPattern != null){
                        fail("Element of type "+handler.getType().toString()+" was not removed from the diagram");
                    }
                    
                }catch(ElementNotFoundException e){
                    log("Element of type "+handler.getType().toString()+" was successfully removed from the diagram");
                }
            }catch(Exception e){
                e.printStackTrace(getLog());
                fail(e.getMessage());
            }
        }
        
    }
    
    //Restore
    public class CheckRestore extends ParametricTestCase{

        public void parametrize(Object parameter){
            setName(getName()+handler.getPatternName()+index);
        }
        public void execute(){
            try{
                PatternOperator elementPattern = null;
                //go to source
                SourceViewOperator source = editor.getSourceView();
                SourceElementOperator seq_element = SourceElementOperator.findSourceElementEx(source, "sequence");
                System.out.println("found sequence");
//                Runtime.getRuntime().traceMethodCalls(true);
                SourceElementOperator element  = seq_element.createChild(SourceElementOperator.getTagNameByType(handler.getType()));
                System.out.println("created element");
                //restore all element's properties
                for(String key: handler.getPropNames()){
                    if(handler.getPropertyType(key).equals("attribute"))
                        element.setAttribute(convertToAttr(key), handler.getProperty(key));
                    else //not an attribute
                        element.createChild(convertToAttr(key)).setContent(handler.getProperty(key));
                    //eventTool.waitNoEvent(5000);
                }
//                System.out.println("set attributes");
                eventTool.waitNoEvent(1000);
//                //////////////////////////////////////////////////////////////////////////////////////////////
//                Map<Thread, StackTraceElement[]> map = Thread.getAllStackTraces();
//                Iterator<Thread> iter = map.keySet().iterator();
//                while(iter.hasNext()){
//                    Thread thread = iter.next();
//                    log("*******Thread "+thread.getName()+" id="+thread.getId()+" is alive="+thread.isAlive());
//                    StackTraceElement[] stack = map.get(thread);
//                    if(stack != null){
//                        for(StackTraceElement ste: stack){
//                            if(ste != null)
//                                log(ste.getFileName()+"-"+ste.getClassName()+":"+ste.getMethodName()+"("+ste.getLineNumber()+")");
//                        }
//                    }
//                }
//                log("******************************************************************************************");
                //eventTool.waitNoEvent(1000);
                //try to find in design view
                try{
                    elementPattern = handler.findElement();
                }catch(ElementNotFoundException e){
                    fail("Element of type "+handler.getType().toString()+" did not appear on the diagram");
                }
//                catch(Throwable th){
//                    log("find element: "+th.toString()+" "+th.getMessage());
//                    throw th;
//                }
                //test the element
                elementPattern.select();
                eventTool.waitNoEvent(500);
                //check element's properties
                Property[] props = Util.getProperties(pso);
                for(String key: handler.getPropNames()){
                    int i=0;
                    for(; i<props.length; i++){
                        if(props[i].getName().equals(key))
                            break;
                    }
                    if(i<props.length){
                        if(!props[i].getValue().equals(handler.getProperty(key))){
                            fail("Element of type "+handler.getType().toString()+" has property "+key+" with value "+props[i].getValue()+
                                    " when "+handler.getProperty(key)+" was expected");
                        }
                    }
                    else{
                            fail("Element of type "+handler.getType().toString()+" has not property "+key);
                    }
                }
            }catch(Exception e){
                //Thread.dumpStack();
//                Map<Thread, StackTraceElement[]> map = Thread.getAllStackTraces();
//                Iterator<Thread> iter = map.keySet().iterator();
//                while(iter.hasNext()){
//                    Thread thread = iter.next();
//                    log("*******Thread "+thread.getName()+" id="+thread.getId()+" is alive="+thread.isAlive());
//                    StackTraceElement[] stack = map.get(thread);
//                    if(stack != null){
//                        for(StackTraceElement ste: stack){
//                            if(ste != null)
//                                log(ste.getFileName()+"-"+ste.getClassName()+":"+ste.getMethodName()+"("+ste.getLineNumber()+")");
//                        }
//                    }
//                }
//                log("******************************************************************************************");
                StackTraceElement[] stack = e.getStackTrace();
                if(stack != null){
                    for(StackTraceElement ste: stack){
                        if(ste != null)
                            log(ste.getFileName()+"-"+ste.getClassName()+":"+ste.getMethodName()+"("+ste.getLineNumber()+")");
                    }
                }
                log("******************************************************************************************");
                try{
                    Thread.currentThread().dumpStack();
                }catch(Throwable ex){
                    ex.printStackTrace();
                }
//                e.fillInStackTrace().printStackTrace();
//                Runtime.getRuntime().traceMethodCalls(false);
//                e.printStackTrace(getLog());
                fail(e.getMessage());
            }
        
        }

        
    }
    
    ///////////////////////////UTILS//////////////////////////////////////////
    String convertToAttr(String propname){
        //remove spaces
        String attrname = propname.replaceAll(" ", "");
        //set first letter to low case
        attrname = attrname.substring(0, 1).toLowerCase() + attrname.substring(1);
        return attrname;
    }

    boolean isAllowed(String propname){
        boolean res = propname.equals("WSDL File") ||
                propname.equals("Assignments Count") ||
                propname.equals("Alarm Type") ||
                propname.equals("Create Instance") ||
                propname.equals("Count Completed Branches Only") ||
//                propname.equals("Date/Duration") ||
                propname.equals("Property Editor");
        return !res; 
        
    }
    
    String removeNSPrefix(String qName){
        int index = qName.indexOf(':');
        return index<0?
            qName:
            qName.substring(index+1);
    }
    
    
}
