/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */
package org.netbeans.test.bpel.verification.lib;

import org.netbeans.jellytools.NewFileWizardOperator;
import org.netbeans.jellytools.OutputTabOperator;
import org.netbeans.jellytools.ProjectsTabOperator;
import org.netbeans.jellytools.nodes.Node;
import org.netbeans.jellytools.nodes.ProjectRootNode;
import org.netbeans.jemmy.EventTool;
import org.netbeans.jemmy.JemmyProperties;
import org.netbeans.jemmy.operators.JButtonOperator;
import org.netbeans.jemmy.operators.JDialogOperator;
import org.netbeans.jemmy.operators.JRadioButtonOperator;
import org.netbeans.jemmy.operators.JTextFieldOperator;
import org.netbeans.jemmy.operators.Operator;
import org.netbeans.orchlib.core.operators.MultiviewEditorOperator;

/**
 *
 * @author mikk@netbeans.org
 */
public class ProjectManager {
    
    /** Creates a new instance of ProjectManager */
    public ProjectManager() {
    }
    
    private static EventTool eventTool = new EventTool();
    
    private final static String XML_CATEGORY="XML";
    private final static String EXTERNAL_WSDL="External WSDL Document(s)";
    private final static String NEW_WSDL_TTL = "New File";
    private final static String LOCAL_WSDL_OPT="From Local File System";
    private final static String FINISH_BTN = "Finish";
    private final static String OPEN_BPEL_MENU = "Open";
    private final static String CLEAN_AND_BUILD_MENU = "Clean and Build";
    private final static String OUTPUT_TTL="build.xml";
    private final static String SUCCESS_TXT="BUILD SUCCESSFUL";
    
    
    
    
    public static void createProject(String projectName, String projectPath){
        org.netbeans.orchlib.ext.Util.createOrchestrationProject(projectName, projectPath);
    }
    
    public static void verifyNewBPELModule(String projectName, String projectPath) throws VerificationException{
        //verifying visual part
        try{
            ProjectsTabOperator pto = ProjectsTabOperator.invoke();
            ProjectRootNode root = new ProjectRootNode(pto.tree(),projectName);
            root.tree().setVerification(true);
            root.select();
            
            Util.verifyProjectPath(projectName, Util.PROCESS_SOURCES_PATH);
            
        }catch(Exception e){
            VerificationException ex = new VerificationException(e, "Some of the project nodes was not found");
            throw ex;
        }
        //TODO: add verification of files on disk
    }
    
    public static void addBPELToProject(String projectName, String bpelName){
        org.netbeans.orchlib.ext.Util.createBPEL(projectName, bpelName);
    }
    
    
    public static void addExternalWSDL(String projectName, String wsdlLocation){
        ProjectsTabOperator pto = ProjectsTabOperator.invoke();
        ProjectRootNode root = new ProjectRootNode(pto.tree(),projectName);
        root.tree().setVerification(true);
        root.select();
        
        NewFileWizardOperator wiz = NewFileWizardOperator.invoke();
        eventTool.waitNoEvent(500);
        wiz.selectCategory(XML_CATEGORY);
        wiz.selectFileType(EXTERNAL_WSDL);
        wiz.next();
        
        JDialogOperator newFileDlg = new JDialogOperator(NEW_WSDL_TTL);
        JRadioButtonOperator localWSDLBtn = new JRadioButtonOperator(newFileDlg, LOCAL_WSDL_OPT);
        localWSDLBtn.clickMouse();
        localWSDLBtn.waitSelected(true);
        
        JTextFieldOperator txtLocation = new JTextFieldOperator(newFileDlg, 2);
        try{
            txtLocation.waitComponentEnabled();
        }catch(Exception e){}
        txtLocation.typeText(wsdlLocation);
        
        JButtonOperator btnFinish = new JButtonOperator(newFileDlg, FINISH_BTN);
        try{
            btnFinish.waitComponentEnabled();
        }catch(Exception e){}
        btnFinish.pushNoBlock();
        
        newFileDlg.waitClosed();
        
        //TODO: there must be a way to write some waiter
        eventTool.waitNoEvent(5000);
    }
    
    
    public static MultiviewEditorOperator openBPEL(String projectName, String bpelName){
        ProjectsTabOperator pto = ProjectsTabOperator.invoke();
        ProjectRootNode root = new ProjectRootNode(pto.tree(),projectName);
        
        Node node = new Node(root, Util.PROCESS_SOURCES_PATH+"|"+bpelName);
        node.tree().setVerification(true);
        node.select();
        
        node.performPopupActionNoBlock(OPEN_BPEL_MENU);
        
        MultiviewEditorOperator editor = new MultiviewEditorOperator(bpelName, new Operator.DefaultStringComparator(false, false));
        return editor;
    }
    
    public static void cleanAndBuild(String projectName){
        ProjectsTabOperator pto = ProjectsTabOperator.invoke();
        ProjectRootNode root = new ProjectRootNode(pto.tree(),projectName);
        root.performPopupActionNoBlock(CLEAN_AND_BUILD_MENU);
        eventTool.waitNoEvent(2000);
    }
    
    public static void verifyBuildSuccessfull(String projectName) throws VerificationException{
        try{
        OutputTabOperator outTab = new OutputTabOperator(OUTPUT_TTL);
        eventTool.waitNoEvent(2000);
        JemmyProperties.setCurrentTimeout("ComponentOperator.WaitStateTimeout", 60000);
        outTab.waitText(SUCCESS_TXT);        
        }catch(Exception e){
            VerificationException ex = new VerificationException(e, "Build failed");
            throw ex;
        }
    }
    
}
