/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */


/*
 * MultiviewEditorOperator.java
 */

package org.netbeans.orchlib.core.operators;

import java.awt.Component;
import javax.swing.JComponent;
import org.netbeans.jellytools.MainWindowOperator;
import org.netbeans.jellytools.TopComponentOperator;
import org.netbeans.jemmy.ComponentChooser;
import org.netbeans.jemmy.JemmyProperties;
import org.netbeans.jemmy.Timeouts;
import org.netbeans.jemmy.Waitable;
import org.netbeans.jemmy.Waiter;
import org.netbeans.jemmy.operators.JToggleButtonOperator;
import org.netbeans.jemmy.operators.Operator;
import org.netbeans.modules.bpel.editors.multiview.DesignerMultiViewElement;

/**
 * @author Mikhail.Kondratyev@sun.com
 */
public class MultiviewEditorOperator extends TopComponentOperator{
    
    public final static String BTN_DESIGN = "Design";
    public final static String BTN_SOURCE = "Source";
    
    private DesignViewOperator designView;
    private SourceViewOperator sourceView;
    public MultiviewEditorOperator(String bpelFileName){
        super((JComponent)waitForDiagram(bpelFileName, new Operator.DefaultStringComparator(false,true)) );
//        Component comp = findSubComponent((ComponentChooser)new MultiviewElementChooser());
//        designView = new DesignViewOperator((DesignerMultiViewElement)comp);
    }
    
    
     public MultiviewEditorOperator(String bpelFileName, Operator.StringComparator comp){
        super((JComponent)waitForDiagram(bpelFileName, comp));
    }
    
    
   public static Component waitForDiagram(final String name, final Operator.StringComparator comp) {
        try{
            Waiter w = new Waiter(new Waitable() {
                public Object actionProduced(Object obj) {
                    return findDiagram(name,comp);
                }
                public String getDescription() {
                    return("Wait for a diagram");
                }
            });
            
            Timeouts t = JemmyProperties.getCurrentTimeouts();
            t.setTimeout("Waiter.WaitingTime", t.getTimeout("DiagramOperator.WaitDiagramOperator"));
            
            return (Component)w.waitAction(null);            
        }catch(InterruptedException ie) {
            return null;
        }
    }
    
    public static Component findDiagram(String name,Operator.StringComparator comp){
        return MainWindowOperator.getDefault().findSubComponent(new DiagramComponentChooser(name,comp));
    }
    
    static class  DiagramComponentChooser implements ComponentChooser
    {        
        String nm=null;
        Operator.StringComparator cmp=null;
        DiagramComponentChooser(String name,Operator.StringComparator comp)
        {
            nm=name;
            cmp=comp;
        }
        
       public boolean checkComponent( java.awt.Component arg0 ) {
//           System.err.println("arg0 - " + arg0);
//           System.err.println("arg0 - " + arg0.getName());
            if(arg0 instanceof org.netbeans.core.multiview.MultiViewCloneableTopComponent) {
                return cmp.equals(arg0.getName(),nm);
            }
                else return false;
        }
        public String getDescription( ) {
            return "Chooser for BPEL";
        }
    }

    
    
    public JToggleButtonOperator btnDesign(){
        return new JToggleButtonOperator(this, BTN_DESIGN);
    }
    
    public JToggleButtonOperator btnSource(){
        return new JToggleButtonOperator(this, BTN_SOURCE);
    }
    
    public EditorViewStatus getCurrentViewStatus(){
        JToggleButtonOperator btn = btnDesign();
        if (btn.isSelected()){
            return EditorViewStatus.Design;
        }else{
            return EditorViewStatus.Source;
        }
    }   
    
    public DesignViewOperator getDesignView(){
        // switch to design view
        btnDesign().push();
        Component comp = findSubComponent((ComponentChooser)new MultiviewElementChooser());
        designView = new DesignViewOperator((DesignerMultiViewElement)comp);
        return designView;
    }
    
    public SourceViewOperator getSourceView(){
        // switch to XML view
        btnSource().push();
        if(sourceView == null)
            sourceView = new SourceViewOperator(getName());
        return sourceView;
    }
    
    
    
    
    public static enum EditorViewStatus{Design, Source};
    
     class MultiviewElementChooser implements ComponentChooser {        
        
        public boolean checkComponent( java.awt.Component arg0 ) {
            if(arg0 instanceof org.netbeans.modules.bpel.editors.multiview.DesignerMultiViewElement) {
                return true;
            }
            return false;
        }
        
        public String getDescription( ) {
            return "Chooser for BPEL diagram Designer Component";
        }
    }
       
     
    
}
