/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * SourceElementOperator.java
 */

package org.netbeans.orchlib.core.operators;

import java.awt.Point;
import org.netbeans.jemmy.JemmyProperties;
import org.netbeans.orchlib.exceptions.ElementNotFoundException;
import org.netbeans.jemmy.JemmyException;

/**
 *
 * @author Alexander Pepin
 */
public class SourceElementOperator {
    private SourceViewOperator editor = null;
    private PatternTypes type = null;
    private int startLine = -1;
    private int startColumn = -1;
    private int endLine = -1;
    private int endColumn = -1;
    private String element = null;
    
    /** Creates a new instance of SourceElementOperator */
    public SourceElementOperator(SourceViewOperator editor) throws ElementNotFoundException{
        this(editor, "");
    }
    
    public SourceElementOperator(SourceViewOperator editor, PatternTypes pType) throws ElementNotFoundException{
        this(editor, getTagNameByType(pType));
        type = pType;
    }
    
    public SourceElementOperator(SourceViewOperator editor, String tagName)throws ElementNotFoundException{
        this.editor = editor;
        this.element = tagName;
        //find beginning of the element
        startLine = editor.getLineNumber();
        String lineText = editor.getText(startLine);
        //TODO: <workaround>
        //startColumn = lineText.indexOf("<"+element);//assume that element starts at the same line
        startColumn = lineText.indexOf(element);//assume that element starts at the same line
        //TODO: </workaround>
        if(startColumn<0){
            throw new ElementNotFoundException("No element with name "+element+" exists in the source.");
        }
        //TODO: <workaround>
        //the following may fail in some cases
        startColumn = lineText.indexOf("<");
        //TODO: </workaround>
        //find end of the element
        calcEndPosition();
        //get tag name assuming it's following <
        if(element == null || element.equals("")){
            char[] chars = new char[]{' ', '/', '>', '\n'};
            int delim = -1;
            for(int i=0; i<chars.length; i++){
                int occur = lineText.indexOf(chars[i], startColumn+1);
                if(occur>=0 && (occur<delim || delim<0))
                    delim = occur;
            }
            if(delim != -1)
                element = lineText.substring(startColumn+1, delim);
                
        }
        
//        JemmyProperties.getCurrentOutput().printLine("==========element "+ element+"\n"+"=========startLine "+startLine +" startColumn "+ startColumn+"\n=========endLine "+endLine +" endColumn "+ endColumn);
        
    }
    private void calcEndPosition(){
        //find end of the element
        endLine = startLine;
        endColumn = editor.getText(startLine).indexOf('>',startColumn);
        while(endColumn<0){
            endLine++;
            endColumn = editor.getText(endLine).indexOf('>');
        }
        
    }
    
    public String getTagName(){
        return element;
    }
    
    public static String getTagNameByType(PatternTypes t){
        String tagName = null;
        switch(t){
            case PARTNER_LINK:  tagName = "partnerLink";
            break;
            case ON_ALARM:  tagName = "onAlarm";
            break;
            case ON_MESSAGE:    tagName = "onMessage";
            break;
            case EVENT_HANDLER: tagName = "eventHandlers";
            break;
            case FAULT_HANDLER: tagName = "faultHandlers";
            break;
            case CATCH_ALL: tagName = "catchAll";
            break;
            case IF_THEN:   tagName = "then";
            break;
            case IF_ELSE:   tagName = "elseif";
            break;
            case REPEAT_UNTIL:  tagName = "repeatUntil";
            break;
            case FOR_EACH:  tagName = "forEach";
            break;
            case ANY:       tagName = "";
            break;
            default: tagName = t.name().toLowerCase();
        }
        return tagName;
    }
    
    public String getElementText(){
        String text = "";
        if(startLine == endLine)
            text = editor.getText(startLine).substring(startColumn, endColumn+1);
        else{
            text = editor.getText(startLine).substring(startColumn);
            for(int index=startLine+1; index<endLine; index++){
                text += editor.getText(index);
            }
            text += editor.getText(endLine).substring(0, endColumn+1);
        }
        return text;
    }
    
    public SourceViewOperator getSourceView(){
        return editor;
    }
    
    public PatternTypes getPatternType(){
        return type;
    }
    
    //
    public void setAttribute(String attrName, String attrVal){
        Point pos = findAttribute(attrName);
        if(pos.x<0){
            //create an attribute at the end of element
            int insertCol =
                    editor.getText(endLine).indexOf('/')<0? endColumn: endColumn-1;
            editor.insert(" "+attrName+"=\""+attrVal+"\"", endLine, insertCol+1);
        } else{
            //assume that value is in the same line
            String prop = editor.getText(pos.y).substring(pos.x);
            int valStart = prop.indexOf('"');
            int valEnd = prop.indexOf('"', valStart+1);
            JemmyProperties.getCurrentOutput().printLine("==="+prop+"======valStart "+valStart +" valEnd "+ valEnd);
            
            editor.delete(pos.y, pos.x+valStart+2, pos.x+valEnd);
            editor.insert(attrVal, pos.y, pos.x+valStart+2);
        }
        calcEndPosition();
    }
    
    public void removeAttribute(String attrName){
        Point pos = findAttribute(attrName);
        if(pos.x<0)
            return;
        //assume that value is in the same line
        String prop = editor.getText(pos.y).substring(pos.x);
        int valStart = prop.indexOf('"');
        int valEnd = prop.indexOf('"', valStart+1);
        editor.delete(pos.y, pos.x, pos.x+valEnd+1);
        calcEndPosition();
    }
    public String getAttribute(String attrName){
        Point pos = findAttribute(attrName);
        if(pos.x<0)
            return null;
        //assume that value is in the same line
        String prop = editor.getText(pos.y).substring(pos.x);
        int valStart = prop.indexOf('"');
        int valEnd = prop.indexOf('"', valStart+1);
//       JemmyProperties.getCurrentOutput().printLine("==="+prop+"======valStart "+valStart +" valEnd "+ valEnd);
        return prop.substring(valStart+1, valEnd);
    }
    
    
    
    protected Point findAttribute(String attrName){
        int line = startLine;
        int col = -1;
        if(startLine == endLine)
            col = editor.getText(startLine).indexOf(attrName, startColumn);
        else{
            for(;line<endLine; line++){
                if((col = editor.getText(line).indexOf(attrName)) >= 0)
                    break;
            }
            if(col<0){
                //searching in the last line
                col = editor.getText(line).substring(0, endColumn).indexOf(attrName);
            }
        }
        return new Point(col, line);
    }
    
    protected Point findEndOfContent(){
        int line = endLine;
        int pos = endColumn;
//        int endPos = endColumn;
        if(hasContent()){
            //if element has content
            // then find end of closing tag
            //discarding all inner elements of the same type
            //assume that the document is well formatted
//            line = endLine;
//            endPos = endColumn;
            int innerCount = 0;
            int inStart = -1;
            int inEnd = -1;
            pos = editor.getText(line).indexOf("</"+element,endColumn);
            while(pos<0 || innerCount != 0){
                line++;
                inStart = editor.getText(line).indexOf("<"+element);
                pos = editor.getText(line).indexOf("</"+element);
                
                if(inStart>=0){ //exist one more element
                    inEnd = editor.getText(line).indexOf(">", inStart+1); //find end of inner element
                    if(inEnd>=0 && editor.getText(line).charAt(inEnd-1) != '/' ){   //element has body
                        innerCount++;
                    }
                }
                if(pos>=0 && innerCount>0){
                    innerCount--;   //close inner element
                    pos = -1;       //cancel closing tag
                }
            }
            pos = editor.getText(line).indexOf('>', pos);
        }
        return new Point(pos, line);
    }
    
    public void deleteElement(){
        removeContent();
        int line = endLine;
        
        
        //element is in one line
        if(startLine == line){
            editor.delete(startLine, startColumn+1, endColumn+1);
        } else{
            //delete from the last line
            editor.delete(line--, 1, endColumn+1);
            //delete intermediate lines
            for(;line > startLine; line--){
                editor.deleteLine(line);
            }
            //delete from the start line
            editor.delete(startLine, startColumn+1, editor.getText(startLine).length());
        }
        //element is removed
        endLine = -1;
        endColumn = -1;
    }
    
    public void restoreElement(){
        if(endLine == -1 || endColumn == -1){
            editor.insert("<"+element+" />", startLine, startColumn+1);
            calcEndPosition();
        }
    }
    
    public SourceElementOperator createChild(String elType) throws ElementNotFoundException{
        String offset = "";
        for(int i=0; i<startColumn; i++)
            offset += " ";
        
        if(!hasContent()){
            //create content
            editor.delete(endLine, endColumn,  endColumn);
            calcEndPosition();
            editor.insert("\n"+offset+"</"+element+">", endLine, endColumn+2);
        }
        //create child
        editor.insert("\n    "+offset+"<"+elType+"/>", endLine, endColumn+2);
        editor.setCaretPositionToLine(endLine+1);
        
        return new SourceElementOperator(editor);
    }
    /**
     * Creates a new element of elType after existing
     */
    public SourceElementOperator createElementAfter(String elType) throws ElementNotFoundException{
        String offset = "";
        for(int i=0; i<startColumn; i++)
            offset += " ";
        
        Point eoe = findEndOfContent();
        //create an new element after existing
        editor.insert("\n"+offset+"<"+elType+"/>", eoe.y, eoe.x+2);
        editor.setCaretPositionToLine(eoe.y+1);
        
        return new SourceElementOperator(editor, elType);
    }
    
    
    public boolean hasContent(){
        return editor.getText(endLine).charAt(endColumn-1) != '/';
    }
    
    public void removeContent(){
        if(hasContent()){
            Point endOfContent = findEndOfContent();
            int endPos = endOfContent.x;
            int line = endOfContent.y;
            
            //content is in one line
            if(endLine == line){
                editor.delete(endLine, endColumn+1, endPos+1);
            } else{
                //delete from the last line
                editor.delete(line--, 1, endPos+1);
                //delete intermediate lines
                for(;line > endLine; line--){
                    editor.deleteLine(line);
                }
                //delete from the start line
                editor.delete(endLine, endColumn+1, editor.getText(endLine).length());
            }
            //insert closing />
            editor.insert("/>", endLine, endColumn+1);
            calcEndPosition();
        }
    }
    
    public void setContent(String text){
        //remove old content
        removeContent();
        if(text != null){
            //create content
            editor.delete(endLine, endColumn,  endColumn);
            calcEndPosition();
            editor.insert(text+"</"+element+">", endLine, endColumn+2);
        }
    }
    
    public String getContent(){
        //find content
        String content = null;
        if(hasContent()){
            Point endOfContent = findEndOfContent();
            int endPos = endOfContent.x;
            int line = endOfContent.y;
            if(endLine == line){
                content = editor.getText(endLine).substring(endColumn+1, endPos);
                int length = content.lastIndexOf('<');
                content = (length == 0) ?
                    "":
                    content.substring(0, length);
            } else{
                content = editor.getText(endLine).substring(endColumn+1);
                for(int index=endLine+1; index<line; index++){
                    content += editor.getText(index);
                }
                //last line
                String lastLine = editor.getText(line).substring(0, endPos);
                int length = lastLine.lastIndexOf('<');
                if(length>0)
                    content += lastLine.substring(0, length);
            }
        }
        
        return content;
    }
    
    public SourceElementOperator getChild(String elType) throws ElementNotFoundException{
        if(!hasContent()){
            throw new ElementNotFoundException("Element has no children.");
        }
        Point eoe = findEndOfContent();
        
        SourceElementOperator child = findNextElement(elType);
        Point eoc = child.findEndOfContent();
        if(eoc.y > eoe.y || ( eoc.y == eoe.y && eoc.x > eoe.x)){
            throw new ElementNotFoundException("Element has no children of type "+elType);
        }
        
        return child;
    }
    
    public SourceElementOperator findNextElement() throws ElementNotFoundException{
        return findNextElement(null);
    }
    
    public SourceElementOperator findNextElement(String elTagName) throws ElementNotFoundException{
        //assume next element is not placed in the same line
        return findSourceElement(editor, elTagName, endLine+1);
    }
    
    public SourceElementOperator findNextSiblingElement() throws ElementNotFoundException{
        return findNextSiblingElement(null);
    }
    
    public SourceElementOperator findNextSiblingElement(String elTagName) throws ElementNotFoundException{
        //assume next element is not placed in the same line in which this element ends
        Point eoe = findEndOfContent();
        
        return findSourceElement(editor, elTagName, eoe.y+1);
    }
    
    public static SourceElementOperator findSourceElement(SourceViewOperator editor, String elTagName, int fromLine) throws ElementNotFoundException{
        //assume next element is not placed in the same line
        if(elTagName == null)
            elTagName = "";
        SourceElementOperator nextElement = null;
        for(int line = fromLine;;line++){
            try{
                editor.setCaretPositionToLine(line);
                nextElement = new SourceElementOperator(editor, elTagName);
                break;
            }catch(ElementNotFoundException e){
                
            }catch(IllegalArgumentException e){
                throw new ElementNotFoundException("No more elements");
            }catch(JemmyException e){
                throw new ElementNotFoundException("No more elements");
            }
        }
        return nextElement;
    }
    
    
    public static SourceElementOperator findSourceElementEx(SourceViewOperator editor, String elTagName) throws ElementNotFoundException{
        return findSourceElementEx(editor, elTagName, 0);
    }
    
    public static SourceElementOperator findSourceElementEx(SourceViewOperator editor, String elTagName, int index) throws ElementNotFoundException{
        if(elTagName == null){
            return findSourceElementEx(editor, "sequence");
        }
        editor.setCaretPosition("<"+elTagName, index, true);
        return new SourceElementOperator(editor, elTagName);
    }
}
