/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * SourceViewOperator.java
 */

package org.netbeans.orchlib.core.operators;
import java.io.File;
import java.io.StringReader;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPathExpressionException;
import org.netbeans.orchlib.exceptions.BadXMLSourceException;
import org.netbeans.jellytools.EditorOperator;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import javax.xml.validation.SchemaFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;
import org.w3c.dom.Node;
import org.xml.sax.helpers.DefaultHandler;
/**
 *
 * @author Alexander Pepin
 */
public class SourceViewOperator extends EditorOperator{
    
    private static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";
    private static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
    private static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";
    
    //temporary scema paths
    private static String schema_path_url = "http://schemas.xmlsoap.org/ws/2003/03/business-process/";
    private static String schema = "bpel.xsd";
    private static String schema_path = "/lib/";
    private static String default_schema_full_path;
    private static final String BASE_PATH = System.getProperty("xtest.home");//"D:/SunProjects/cokeTB/ext/lib/";
    
    /**
     * Creates a new instance of SourceViewOperator
     */
    {
        default_schema_full_path = BASE_PATH.substring(0, BASE_PATH.lastIndexOf(File.separatorChar))+schema_path+schema;
    }
    public SourceViewOperator(String bpelFileName) {
        super(bpelFileName);
        
//        Enumeration props = System.getProperties().propertyNames();
//        String propNames = "System properties:\n";
//        while(props.hasMoreElements()){
//            String propName = (String)props.nextElement();
//            propNames += propName+" = "+ System.getProperty(propName)+"\n";
//        }
//        getOutput().printTrace(propNames+"end of System properties");
    }

    public boolean validateSource(){
        return validateSource(default_schema_full_path);
    }
    
    
    public boolean validateSource(String schema_full_path){
        boolean bres = false;
        
        SchemaFactory sf = SchemaFactory.newInstance(W3C_XML_SCHEMA);
        
        SAXParserFactory spf = SAXParserFactory.newInstance();
        spf.setNamespaceAware(true);
        spf.setValidating(true);
        try{
            SAXParser parser = spf.newSAXParser();
            getOutput().printTrace("path to scema: "+schema_full_path);
            parser.setProperty(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
            parser.setProperty(JAXP_SCHEMA_SOURCE, schema_full_path);
            StringReader source = new StringReader(getText());
            parser.parse(new InputSource(source), new XMLErrorHandler());
            bres = true;
        }catch(SAXParseException e){
            getOutput().printError("SAXParseException! "+e.toString());
            getOutput().printError("in line: "+e.getLineNumber()+" column : "+e.getColumnNumber());
        }catch(SAXException e){
            getOutput().printError("SAXException! "+e.toString());
        }catch(Throwable e){
            getOutput().printError("Exception! "+e.toString());
            
        }
        
        return bres;
    }
    
    
    public String evaluateExpression(String xpathExpression) throws BadXMLSourceException{
        try{
            Document doc = getDocument();
            XPath xpath = XPathFactory.newInstance().newXPath();
            return xpath.evaluate(xpathExpression, doc);
        }catch(XPathExpressionException e){
            e.printStackTrace();
            return null;
        }
    }
    
    public NodeList findNodesByXpath(String xpathExpression) throws BadXMLSourceException{
        try{
            Document doc = getDocument();
            XPath xpath = XPathFactory.newInstance().newXPath();
            return (NodeList)xpath.evaluate(xpathExpression, doc, XPathConstants.NODESET);
        }catch(XPathExpressionException e){
            e.printStackTrace();
            return null;
        }        
    }
    
    public Node findNodeByXpath(String xpathExpression) throws BadXMLSourceException{
        try{
            Document doc = getDocument();
            XPath xpath = XPathFactory.newInstance().newXPath();
            return (Node)xpath.evaluate(xpathExpression, doc, XPathConstants.NODE);
        }catch(XPathExpressionException e){
            e.printStackTrace();
            return null;
        }        
    }
    
    
    
    
    
    public Document getDocument() throws BadXMLSourceException{
        try{
            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            StringReader source = new StringReader(getText());
            return builder.parse(new InputSource(source));
        }catch(SAXException saxE){
            saxE.printStackTrace();
            throw new BadXMLSourceException();
        }catch(Exception e){
            e.printStackTrace(System.err);
            return null;
        }
    }
    
}

class XMLErrorHandler extends DefaultHandler{
    
    public void warning(SAXParseException e)
    throws SAXException {
        // no op
//        System.out.println("Warning! "+e.toString());
//        System.out.println("in line: "+e.getLineNumber()+" column : "+e.getColumnNumber());
        throw e;
    }
    
    public void error(SAXParseException e)
    throws SAXException {
        // no op
//        System.out.println("Error! "+e.toString());
//        System.out.println("in line: "+e.getLineNumber()+" column : "+e.getColumnNumber());
        throw e;
    }
    
    public void fatalError(SAXParseException e)
    throws SAXException {
//        System.out.println("Fatal Error! "+e.toString());
//        System.out.println("in line: "+e.getLineNumber()+" column : "+e.getColumnNumber());
        throw e;
    }
    
}


