/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.orchlib.core.verifiers;

import java.awt.Component;
import java.awt.Point;
import java.awt.Robot;
import java.awt.event.KeyEvent;
import java.io.PrintStream;
import java.util.HashMap;
import javax.swing.MenuElement;
import org.netbeans.orchlib.core.operators.ConnectionOperator;
import org.netbeans.orchlib.core.operators.DesignViewOperator;
import org.netbeans.orchlib.core.operators.ElementTypes;
import org.netbeans.orchlib.core.operators.MultiviewEditorOperator;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.core.operators.VisualElementOperator;
import org.netbeans.orchlib.exceptions.ElementNotFoundException;
import org.netbeans.orchlib.utils.TypeSpecificProperties;
import org.netbeans.jellytools.properties.Property;
import org.netbeans.jellytools.properties.PropertySheetOperator;
import org.netbeans.jemmy.ComponentChooser;
import org.netbeans.jemmy.EventTool;
import org.netbeans.jemmy.JemmyProperties;
import org.netbeans.jemmy.operators.JButtonOperator;
import org.netbeans.jemmy.operators.JDialogOperator;
import org.netbeans.jemmy.operators.JMenuItemOperator;
import org.netbeans.jemmy.operators.JPopupMenuOperator;
import org.netbeans.jemmy.operators.JTextComponentOperator;
import org.netbeans.jemmy.operators.JTextFieldOperator;
import org.netbeans.jemmy.operators.Operator;
import org.netbeans.jemmy.operators.TextFieldOperator;
import org.netbeans.modules.bpel.design.decoration.components.DiagramButton;

/**
 * @author Andrey.Yamkovoy@sun.com
 */
public class PatternVerifier {
    public static final String PATTERN_CONTEXT_PROPERTY_EDITOR = "Edit ...";
    private static HashMap<Enum, String> tools = null;
    
    protected MultiviewEditorOperator editor;
    protected PatternTypes patternType;
    protected String patternTypeName;
    protected String namePrefix;
    protected PrintStream log;

    protected DesignViewOperator designView;
    protected EventTool eventTool = new EventTool();
    protected Robot robot = null;
    
    public PatternVerifier(MultiviewEditorOperator editor, PatternTypes patternType) {
        this(editor, patternType, "");
    }

    public PatternVerifier(MultiviewEditorOperator editor, PatternTypes patternType, String namePrefix) {
        this(editor, patternType, namePrefix, null);
    }

    public PatternVerifier(MultiviewEditorOperator editor, PatternTypes patternType, PrintStream log) {
        this(editor, patternType, "", log);
    }
    
    public PatternVerifier(MultiviewEditorOperator editor, PatternTypes patternType, String namePrefix, PrintStream log) {
        this.editor = editor;
	this.designView = editor.getDesignView();
        log(this.designView.toString());
        this.patternType = patternType;
	this.patternTypeName = getPatternTypeName(patternType);
	this.namePrefix = namePrefix;
        this.log = log;
	
	try{
	    robot = new Robot();
	}catch(Exception e){
	}
    }

    public static String getPatternTypeName(Enum element) {
	if (tools == null){
	    tools  = new HashMap<Enum, String>();
	    
	    tools.put(PatternTypes.INVOKE, "Invoke");
	    tools.put(PatternTypes.PARTNER_LINK, "Partner Link");
	    tools.put(PatternTypes.RECEIVE, "Receive");
	    tools.put(PatternTypes.REPLY, "Reply");
	    tools.put(PatternTypes.EMPTY, "Empty");
	    tools.put(PatternTypes.ASSIGN, "Assign");
	    tools.put(PatternTypes.WAIT, "Wait");
	    tools.put(PatternTypes.EXIT, "Terminate");
	    tools.put(PatternTypes.THROW, "Throw");
	    tools.put(PatternTypes.COMPENSATE, "Compensate");
	    tools.put(PatternTypes.FLOW, "Flow");
	    tools.put(PatternTypes.PICK, "Pick");
	    tools.put(PatternTypes.SEQUENCE, "Sequence");
	    tools.put(PatternTypes.SCOPE, "Scope");
	    tools.put(PatternTypes.SWITCH, "Switch");
	    tools.put(PatternTypes.SWITCH_CASE, "Case");
	    tools.put(PatternTypes.IF, "If");
	    tools.put(PatternTypes.WHILE, "While");
	    tools.put(PatternTypes.ON_MESSAGE, "onMessage");
	    tools.put(PatternTypes.ON_ALARM, "onAlarm");
	}
	
        return (String)tools.get(element);
    }
    
    public PatternOperator createPattern() throws ElementNotFoundException{
        DesignViewOperator view = editor.getDesignView();
        //create an element
        PatternOperator patRoot = view.getRootPatternOperator();
        PatternOperator patTarget = new PatternOperator(patRoot, PatternTypes.SEQUENCE, 0);
        VisualElementOperator el1 = new VisualElementOperator(patRoot, ElementTypes.CONTENT_ELEMENT, 0);
        VisualElementOperator el2 = new VisualElementOperator(patTarget, ElementTypes.ANY, 0);
        if(el2.getTypeString().equals(ElementTypes.PLACEHOLDER_ELEMENT.toString())){
            view.createPatternFromPalette(patternType, el2);
        }else{
            ConnectionOperator conn = new ConnectionOperator(el1, el2);            
            //workaround for a bug
            patTarget.select();
            eventTool.waitNoEvent(500);
            //workaround end
            view.createPatternFromPalette(patternType, conn, patTarget);            
        }     
	
        PatternOperator pattern = new PatternOperator(patTarget, patternType, 0);
        pattern.select();
        
        return pattern;
    }
    
    private void namePatternByCustomEditor(PatternOperator pattern, String newName){
        pattern.select();
        String patternName = TypeSpecificProperties.getCurrentPatternPaletteName(patternType);
        PropertySheetOperator props = PropertySheetOperator.invoke();
	props.setComparator(new Operator.DefaultStringComparator(true, true));
        Property nameProp = new Property(props, "Name");
        nameProp.openEditor();
        eventTool.waitNoEvent(500);
        JDialogOperator editor = new JDialogOperator(patternName);
        JTextComponentOperator txtArea = new JTextComponentOperator(editor);
        txtArea.clearText();
        txtArea.typeText(newName);
        eventTool.waitNoEvent(500);
        new JButtonOperator(editor, "OK").push();
        editor.waitClosed();
    }

    private void namePatternByPropField(PatternOperator pattern, String newName){
        pattern.select();
        PropertySheetOperator props = PropertySheetOperator.invoke();
	props.setComparator(new Operator.DefaultStringComparator(true, true));
        Property nameProp = new Property(props, "Name");
	nameProp.setValue(newName);
        eventTool.waitNoEvent(500);
    }

    private void namePatternByDiagram(PatternOperator pattern, String newName){
        pattern.select();
        Point point = pattern.getDesignView().convertPointToScreen(pattern.getNameCenterPoint());
        designView.clickMouse(point.x, point.y, 2);
        eventTool.waitNoEvent(500);
        
        JTextFieldOperator text = new JTextFieldOperator(designView);
        text.setText(newName);
        text.pressKey(KeyEvent.VK_ENTER);
        eventTool.waitNoEvent(500);
    }
    
    private void namePatternByPropEditor(PatternOperator pattern, String newName){
	pattern.clickActionPoint(2);
        String patternName = TypeSpecificProperties.getCurrentPatternPaletteName(patternType);
        JDialogOperator editor = new JDialogOperator(patternName);
	JTextFieldOperator txtName = new JTextFieldOperator(editor, 0);
	txtName.setText(newName);
        eventTool.waitNoEvent(500);
        new JButtonOperator(editor, "OK").push();
        editor.waitClosed();
    }

    protected boolean removePatternByCrossButton(PatternOperator pattern) throws ElementNotFoundException{
        pattern.select();
        eventTool.waitNoEvent(500);
        JButtonOperator btn = new JButtonOperator(pattern.getDesignView(), new DiagramButtonChooser("Delete"));
        btn.clickMouse();
        eventTool.waitNoEvent(500);
        
        return isPatternRemoved(pattern);
    }
    
    protected boolean removePatternByPopup(PatternOperator pattern) throws ElementNotFoundException{
        pattern.select();
        eventTool.waitNoEvent(500);
	pattern.getPopup().pushMenu("Delete");
        eventTool.waitNoEvent(500);
        
        return isPatternRemoved(pattern);
    }
    
    protected boolean removePatternByShortcut(PatternOperator pattern) throws ElementNotFoundException{
	pattern.select();
	robot.keyPress(KeyEvent.VK_DELETE);
	robot.keyRelease(KeyEvent.VK_DELETE);
        eventTool.waitNoEvent(500);
        
        return isPatternRemoved(pattern);
    }

    protected boolean isPatternRemoved(PatternOperator pattern) throws ElementNotFoundException{
        PatternOperator root = designView.getRootPatternOperator();
        int childsCount = root.getAllChildPatternsCount(patternType);
        for(int i=0; i<childsCount; i++){
            PatternOperator pat = new PatternOperator(root, patternType, i);
            if (pat.equals(pattern)){
                return false;
            }
        }
        return true;
/*        
	long waitPatTime = 0;
	try{
            waitPatTime = JemmyProperties.getCurrentTimeouts().getTimeout("PatternOperator.WaitPatternTime");
	    JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", 3000);
//	    PatternOperator pat = new PatternOperator(designView, patName, patternType, new Operator.DefaultStringComparator(true, true));
            pattern.select();
	    log("Pattern '" + pattern.getName() + "' was found but should not be");
	    return false;
	}catch(Exception e){
	    return true;
	}finally{
            JemmyProperties.setCurrentTimeout("PatternOperator.WaitPatternTime", waitPatTime);
	}
 */
    }
    
    private boolean compareNames(PatternOperator element){
	try{
            PropertySheetOperator pso = PropertySheetOperator.invoke();
	    pso.setComparator(new Operator.DefaultStringComparator(true, true));
	    if (!new Property(pso, "Name").getValue().equals(element.getName())){
		log("From diagram = '" + element.getName() + "'");
		log("From properties = '" + new Property(pso, "Name").getValue() + "'");
		log("Names from diagram and from properties not equals");
		return false;
	    }
	    return true;
	}catch(Exception e){
	    if (log != null){
	        log("Exception while check element name:");
		e.printStackTrace(log);
	    }
	    return false;
	}
    }
    
    protected void log(String message){
	if (log != null){
	    log.println(message);
	}
    }
    
// ------- Check methods ---------------------------    
    
    public boolean checkCreate(){
	final String patName = namePrefix + "1";
	PatternOperator element = null;
	
	try{
	    element = createPattern();
	    element.select();
	}catch(Exception e){
	    if (log != null){
		e.printStackTrace(log);
	    }
	    return false;
	}
	
	return compareNames(element);
    }
    
    public boolean checkRenameByCustomEditor(){
	final String patName = namePrefix + "2";
	PatternOperator element = null;
	
	try{
            log("before CreatePattern" + editor.btnDesign().isSelected());
	    element = createPattern();
            log("after CreatePattern" + editor.btnDesign().isSelected());
	    namePatternByCustomEditor(element, patName);
	}catch(Exception e){
	    if (log != null){
		e.printStackTrace(log);
	    }
	    return false;
	}
	
	return compareNames(element);
    }
    
    public boolean checkRenameByPropField(){
	final String patName = namePrefix + "3";
	PatternOperator element = null;
	
	try{
	    element = createPattern();
	    namePatternByPropField(element, patName);
	}catch(Exception e){
	    if (log != null){
		e.printStackTrace(log);
	    }
	    return false;
	}
	
	return compareNames(element);
    }

    public boolean checkRenameByPropEditor(){
	final String patName = namePrefix + "4";
	PatternOperator element = null;
	
	try{
	    element = createPattern();
	    namePatternByPropEditor(element, patName);
	}catch(Exception e){
	    if (log != null){
		e.printStackTrace(log);
	    }
	    return false;
	}
	
	return compareNames(element);
    }

    public boolean checkRenameByDiagram(){
	final String patName = namePrefix + "5";
	PatternOperator element = null;
	
	try{
	    element = createPattern();
	    namePatternByDiagram(element, patName);
	}catch(Exception e){
	    if (log != null){
		e.printStackTrace(log);
	    }
	    return false;
	}
	
	return compareNames(element);
    }
    
    public boolean checkContextMenu(String[] expectedItems){
        return checkContextMenu(null, expectedItems);
    }
    
    public boolean checkContextMenu(String root, String[] expectedItems){
	PatternOperator element = null;
	
	try{
            element = createPattern();
            JPopupMenuOperator popup = element.getPopup();
            MenuElement[] subElements;
            
	    for(int i = 0; i < expectedItems.length; i++){
                if (root == null){
                    popup.showMenuItem(expectedItems[i]);
                }else{
                    popup.showMenuItem(root + "|" + expectedItems[i]);
                }
	    }

            if (root == null){
                subElements = popup.getSubElements();
            }else{
                JMenuItemOperator rootItem = popup.showMenuItem(root);
                eventTool.waitNoEvent(500);
                subElements = rootItem.getSubElements();
            }
            
            log("Root node: " + root);
            log("Subelements:");
            for(int j=0; j < subElements.length; j++){
                MenuElement elem = subElements[j];
                log(j + " - '" + elem.getComponent().getName() + "'");
            }
/*            
            try{
                Robot robot = new Robot();
                robot.keyPress(KeyEvent.VK_ESCAPE);
                robot.keyRelease(KeyEvent.VK_ESCAPE);
            }catch(Exception e){}
*/
            
/* Not working ... Need to find the problem ...            
            if (subElements.length != expectedItems.length){
                log("Should be " + expectedItems.length + " elements in popup menu but " + subElements.length + " found");
                return false;
            }
*/            
	    return true;
	}catch(Exception e){
	    if (log != null){
		e.printStackTrace(log);
	    }
	    return false;
	}
    }

    public boolean checkRemoveByCrossButton(){
	PatternOperator element = null;
	
	try{
	    element = createPattern();
	    return removePatternByCrossButton(element);
	}catch(Exception e){
	    if (log != null){
		e.printStackTrace(log);
	    }
	    return false;
	}
    }
    
    public boolean checkRemoveByPopup(){
	PatternOperator element = null;
	
	try{
	    element = createPattern();
	    return removePatternByPopup(element);
	}catch(Exception e){
	    if (log != null){
		e.printStackTrace(log);
	    }
	    return false;
	}
    }

    public boolean checkRemoveByShortcut(){
	PatternOperator element = null;
	
	try{
	    element = createPattern();
	    return removePatternByShortcut(element);
	}catch(Exception e){
	    if (log != null){
		e.printStackTrace(log);
	    }
	    return false;
	}
    }
    
    class DiagramButtonChooser implements ComponentChooser{
        private String tooltip;
        
        public DiagramButtonChooser(String tooltip){
            this.tooltip = tooltip;
        }
        
        public String getDescription(){
            return "Chooser for diagram buttons";
        }
        
        public boolean checkComponent(Component component){
            if ((component instanceof DiagramButton) && (tooltip.equals(((DiagramButton)component).getToolTipText()))){
                return true;
            } else {
                return false;
            }
        }
    }
}
