/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * Logger.java
 */

package org.netbeans.orchlib.ext.creator;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;

/**
 * The Logger class is an attempt to automate the test specification creation
 * based on the actions performed in Creator. The Logger with help of LogActions
 * allows to generate a test spec in our predefined XML-like format
 *
 * The logger is not used in most of the cases and this is controlled by the noLog private field
 * To stress the fact that the logger is not used in most of the cases this field can't be set through
 * any API so changing behavior requires recompilation
 *
 * @author Mikhail.Kondratyev@sun.com
 */
public class Logger {
    
    private static File outFile;
    private static boolean noLog = true;
    private PrintStream stream;
    
    /** Creates a new instance of Logger */
    private Logger() {
        try{
            stream = new PrintStream(new FileOutputStream(outFile));
        }catch(Exception e){
            e.printStackTrace();
        }
    }
    
    private static Logger instance = null;
    
    public static Logger instance(){
        if (instance==null){
            instance = new Logger();
        }
        return instance;
    }
    
    
    public void writeAction(String text){
        if (noLog) return;
        stream.println("<action>"+text+"</action>");
        stream.flush();
    }
    
    public void writeResult(String text){
        if (noLog) return;
        stream.println("<result>"+text+"</result>");
        stream.flush();
    }
    
    public void writeCase(String name){
        if (noLog) return;
        stream.println("<usecase name=\""+name+"\">");
        stream.flush();
    }
    
    public void writeCaseEnd(){
        if (noLog) return;
        stream.println("</usecase>");
        stream.flush();
    }
    
    public static File getOutFile() {
        return outFile;
    }
    
    public static void setOutFile(File aOutFile) {
        outFile = aOutFile;
    }
    
    
}
