/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * CreateActionFactory.java
 */

package org.netbeans.orchlib.ext.creator.actions;

import java.util.Hashtable;
import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.w3c.dom.Element;

/**
 * The factory used to create new CreateAction based on the parent pattern 
 * and new tag found in the BPEL
 * @author Mikhail.Kondratyev@sun.com
 */
public class CreateActionFactory {
    
    public static CreateActionFactory getInstance(){
        return instance;
    }
    
    public CreateAction createAction(PatternOperator parentPattern, Element element){
        Hashtable<String, CreateAction> hash = actionsByPattern.get(parentPattern.getTypeString());
        if (hash!=null){
            return hash.get(element.getTagName());
        }else{
            return null;
        }
    }
    
    
    /** Creates a new instance of CreateActionFactory */
    private CreateActionFactory() {
        Hashtable<String, CreateAction> actionsByTag = new Hashtable<String, CreateAction>();
        actionsByPattern.put(PatternTypes.PROCESS.toString(), actionsByTag);
        addAllActivitiesToActions(actionsByTag, new BasePatternCreateAction());
        actionsByTag.put("faultHandlers", new AnyFaultHandlerCreateAction());
        actionsByTag.put("compensationHandler", new AnyCompensateHandlerCreateAction());
        actionsByTag.put("eventHandlers", new AnyEventHandlerCreateAction());
        
        //Sequence create actions
        actionsByTag = new Hashtable<String, CreateAction>();
        actionsByPattern.put(PatternTypes.SEQUENCE.toString(), actionsByTag);
        addAllActivitiesToActions(actionsByTag, new SequenceCreateAction());
        
        //if create actions
        actionsByTag = new Hashtable<String, CreateAction>();
        actionsByPattern.put(PatternTypes.IF.toString(), actionsByTag);
        //now the elements from then section go directly to If
        addAllActivitiesToActions(actionsByTag, new BasePatternCreateAction());
        
        actionsByTag.put("else", new SwitchElseCreateAction());
        actionsByTag.put("elseif", new IfElseIfCreateAction());
        
        //switch create actions
        actionsByTag = new Hashtable<String, CreateAction>();
        actionsByPattern.put(PatternTypes.ELSEIF.toString(), actionsByTag);
        addAllActivitiesToActions(actionsByTag, new BasePatternCreateAction());
        
        actionsByTag.put("invoke", new BasePatternCreateAction());
        
        //pick create actions
        actionsByTag = new Hashtable<String, CreateAction>();
        actionsByPattern.put(PatternTypes.PICK.toString(), actionsByTag);
        actionsByTag.put("onMessage", new PickOnMessageCreateAction());
        actionsByTag.put("onAlarm", new PickOnAlarmCreateAction());
        
        
        //onAlarm create actions
        actionsByTag = new Hashtable<String, CreateAction>();
        actionsByPattern.put(PatternTypes.ON_ALARM.toString(), actionsByTag);
        addAllActivitiesToActions(actionsByTag, new BasePatternCreateAction());
        
        //onMessage create actions
        actionsByTag = new Hashtable<String, CreateAction>();
        actionsByPattern.put(PatternTypes.ON_MESSAGE.toString(), actionsByTag);
        addAllActivitiesToActions(actionsByTag, new BasePatternCreateAction());
        
        //Scope create actions
        actionsByTag = new Hashtable<String, CreateAction>();
        actionsByPattern.put(PatternTypes.SCOPE.toString(), actionsByTag);
        addAllActivitiesToActions(actionsByTag, new BasePatternCreateAction());
        
        actionsByTag.put("faultHandlers", new AnyFaultHandlerCreateAction());
        actionsByTag.put("compensationHandler", new AnyCompensateHandlerCreateAction());
        actionsByTag.put("eventHandlers", new AnyEventHandlerCreateAction());
        
        //Flow create actions
        actionsByTag = new Hashtable<String, CreateAction>();
        actionsByPattern.put(PatternTypes.FLOW.toString(), actionsByTag);
        addAllActivitiesToActions(actionsByTag, new FlowCreateAction());
        
        //While create actions
        actionsByTag = new Hashtable<String, CreateAction>();
        actionsByPattern.put(PatternTypes.WHILE.toString(), actionsByTag);
        addAllActivitiesToActions(actionsByTag, new BasePatternCreateAction());
        
        //CompensationHandler create actions
        actionsByTag = new Hashtable<String, CreateAction>();
        actionsByPattern.put(PatternTypes.COMPENSATION_HANDLER.toString(), actionsByTag);
        addAllActivitiesToActions(actionsByTag, new BasePatternCreateAction());
        
        //fault  create actions
        actionsByTag = new Hashtable<String, CreateAction>();
        actionsByPattern.put(PatternTypes.FAULT_HANDLER.toString(), actionsByTag);
        actionsByTag.put("catch", new FaultHandlerCatchCreateAction());
        actionsByTag.put("catchAll", new FaultHandlerCatchAllCreateAction());
        
        //event handler  create actions
        actionsByTag = new Hashtable<String, CreateAction>();
        actionsByPattern.put(PatternTypes.EVENT_HANDLER.toString(), actionsByTag);
        actionsByTag.put("onMessage", new EventHandlersOnMessageCreateAction());
        actionsByTag.put("onAlarm", new EventHandlerOnAlarmCreateAction());
        
        //catch  create actions
        actionsByTag = new Hashtable<String, CreateAction>();
        actionsByPattern.put(PatternTypes.CATCH.toString(), actionsByTag);
        addAllActivitiesToActions(actionsByTag, new BasePatternCreateAction());
        actionsByTag.put("compensate", new BasePatternCreateAction());
        
        //catch All  create actions
        actionsByTag = new Hashtable<String, CreateAction>();
        actionsByPattern.put(PatternTypes.CATCH_ALL.toString(), actionsByTag);
        addAllActivitiesToActions(actionsByTag, new BasePatternCreateAction());
        actionsByTag.put("compensate", new BasePatternCreateAction());
        
        //TODO:add invoke handlers actions
    }
    
    
    private void addAllActivitiesToActions(Hashtable<String, CreateAction> actionsByTag, CreateAction action){
        actionsByTag.put("invoke", action);
        actionsByTag.put("receive", action);
        actionsByTag.put("reply", action);
        actionsByTag.put("if", action);
        actionsByTag.put("pick", action);
        actionsByTag.put("flow", action);
        actionsByTag.put("scope", action);
        actionsByTag.put("while", action);
        actionsByTag.put("sequence", action);
        actionsByTag.put("empty", action);
        actionsByTag.put("wait", action);
        actionsByTag.put("throw", action);
        actionsByTag.put("assign", action);
        actionsByTag.put("terminate", action);
    }
    
    private static CreateActionFactory instance = new CreateActionFactory();
    
    private Hashtable<String, Hashtable<String, CreateAction>> actionsByPattern = new Hashtable<String, Hashtable<String, CreateAction>>();
    private Hashtable<String, CreateAction> actionsByTag = new Hashtable<String, CreateAction>();
    
}
