/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * NameAction.java
 *
 */

package org.netbeans.orchlib.ext.creator.actions;

import org.netbeans.orchlib.core.operators.PatternOperator;
import org.netbeans.orchlib.core.operators.PatternTypes;
import org.netbeans.orchlib.exceptions.ElementNotFoundException;
import org.netbeans.orchlib.ext.creator.exceptions.NamingVerificationException;
import org.netbeans.jellytools.properties.Property;
import org.netbeans.jellytools.properties.PropertySheetOperator;
import org.netbeans.jemmy.EventTool;
import org.netbeans.jemmy.operators.Operator;

/**
 * This is the base class for naming the activities being created by the creator 
 * Some specific namer can be inherited from this class
 * @author Mikhail.Kondratyev@sun.com
 */
public class NameAction {
    
    /** Creates a new instance of NameAction */
    public NameAction() {
    }
    
    EventTool eventTool = new EventTool();
    
    /**
     * performs default naming of an activity through activity's properties
     * @param patternToName Activity to be named
     * @param name Name to be assigned
     * @throws orchlib.exceptions.ElementNotFoundException 
     * @throws orchlib.ext.creator.exceptions.NamingVerificationException 
     */
    public void perform(PatternOperator patternToName, String name) throws ElementNotFoundException, NamingVerificationException{
        if (name==null || name.length()==0){
            return;
        }
        
        patternToName.select();
        eventTool.waitNoEvent(1000);
        
        PropertySheetOperator props = PropertySheetOperator.invoke();
        
        Property p = new Property(props, "Name");
        p.setValue(name);
        eventTool.waitNoEvent(500);

        //verification
        PatternOperator parent = patternToName.getParent();
        int sameNamedPatterns = parent.getChildPatternsCount(new PatternOperator.PatternByTypeAndNameChooser(PatternTypes.ANY, name, new Operator.DefaultStringComparator(true, true)));
        boolean verified = false;
        for(int i=0;i<sameNamedPatterns;i++){
            if (parent.getDirectChildPattern(new PatternOperator.PatternByTypeAndNameChooser(PatternTypes.ANY, name, new Operator.DefaultStringComparator(true, true)), i).equals(patternToName)){
                verified = true;
                break;
            }
        }
        if (!verified){
            throw new NamingVerificationException("Naming of pattern named "+patternToName.getName()+" failed");
        }
        
    }
    
    
}
