/*
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at http://www.netbeans.org/cddl.html
 * or http://www.netbeans.org/cddl.txt.
 * 
 * When distributing Covered Code, include this CDDL Header Notice in each file
 * and include the License file at http://www.netbeans.org/cddl.txt.
 * If applicable, add the following below the CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

/*
 * LibProperties.java
 */

package org.netbeans.orchlib.utils;

import java.util.Collection;
import java.util.HashMap;
import java.util.Set;
import org.netbeans.orchlib.core.operators.PatternTypes;

/**
 * The class is used to map the activities types to some additional data.
 * Currently it maps activities types to corresponding palette elements names
 * @author Mikhail.Kondratyev@sun.com
 */
public class TypeSpecificProperties {
    private static TypeSpecificProperties properties = null;;
    private HashMap<Enum, String> tools = null;
    
    
    /** Creates a new instance of LibProperties */
    protected TypeSpecificProperties() {
        tools  = new HashMap<Enum, String>();
        initDefaultPatternPaletteNames();
    }
    
    
    public static TypeSpecificProperties getProperties() {
        if(properties == null) {
            properties = new TypeSpecificProperties();
        }
        return properties;
    }
    
    public static String getCurrentPatternPaletteName(Enum element) {
        return getProperties().getPatternPaletteName(element);
    }
    
    public static String[] getPalettePatternsNames(){
        Collection<String> values = getProperties().tools.values();
        return values.toArray(new String[values.size()]);
    }
    
    public static PatternTypes[] getPalettePatternsTypes(){
        Set<Enum> keys = getProperties().tools.keySet();
        return keys.toArray(new PatternTypes[keys.size()]);
    }    
    
    /**
     * Mapping activity types and palette names is done here
     */
    protected void initDefaultPatternPaletteNames()  {
        setPatternPaletteName(PatternTypes.INVOKE, "Invoke");
        setPatternPaletteName(PatternTypes.PARTNER_LINK, "Partner Link");
        setPatternPaletteName(PatternTypes.RECEIVE, "Receive");
        setPatternPaletteName(PatternTypes.REPLY, "Reply");
        setPatternPaletteName(PatternTypes.EMPTY, "Empty");
        setPatternPaletteName(PatternTypes.ASSIGN, "Assign");
        setPatternPaletteName(PatternTypes.WAIT, "Wait");
        setPatternPaletteName(PatternTypes.THROW, "Throw");
        setPatternPaletteName(PatternTypes.FLOW, "Flow");
        setPatternPaletteName(PatternTypes.PICK, "Pick");
        setPatternPaletteName(PatternTypes.SEQUENCE, "Sequence");
        setPatternPaletteName(PatternTypes.SCOPE, "Scope");
        setPatternPaletteName(PatternTypes.WHILE, "While");
        setPatternPaletteName(PatternTypes.IF, "If");
        setPatternPaletteName(PatternTypes.FOR_EACH, "ForEach");
        setPatternPaletteName(PatternTypes.REPEAT_UNTIL, "RepeatUntil");
    }
        
    public String setPatternPaletteName(Enum element, String toolTip) {
        return (String)tools.put(element, toolTip);
    }
    
    public static String getPatternPaletteName(Enum element) {
        return (String)getProperties().tools.get(element);
    }
}
