/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.jasm.parser;

import java.io.*;
import java.util.ArrayList;

public class AttrCode extends Attribute {

    int maxStack;
    int maxLocals;
    int codeLen;
    ArrayList code;
    int countExceptions;
    AttrCodeExc[] exceptions;
    Attributes attrs;

    public AttrCode (Attributes p, ConstantPool cp, int name, int l) {
        super (p, cp, name, l);
    }

    public void fromClass (DataInputStream dis) throws Exception {
        maxStack = dis.readUnsignedShort ();
        maxLocals = dis.readUnsignedShort ();
        codeLen = dis.readInt ();
        code = new ArrayList ();
        boolean wide = false;
        for (int a = 0; a < codeLen;) {
            int c = dis.readUnsignedByte ();
            Opcode o;
            if (wide) {
                if (c == OpcodeTable.OPCODE_IINC)
                    o = new OpcodeWideIInc ();
                else if (c >= OpcodeTable.OPCODE_WIDE_LOAD_LOW  &&  c <= OpcodeTable.OPCODE_WIDE_LOAD_HIGH)
                    o = new OpcodeWideUnsignedOneArg ();
                else if (c >= OpcodeTable.OPCODE_WIDE_STORE_LOW  &&  c <= OpcodeTable.OPCODE_WIDE_STORE_HIGH)
                    o = new OpcodeWideUnsignedOneArg ();
                else
                    throw new ParserException ("Bad wide opcode " + c + " " + a);
                wide = false;
            } else {
                Class cl = OpcodeTable.getClassByOpcode (c);
                if (cl == null)
                    throw new ParserException ("Bad opcode " + c + " " + a);
                o = (Opcode) cl.newInstance ();
            }
            o.setParent (this);
            o.setPos (a);
            a ++;
            o.setOpcode (c);
            a += o.fromClass (dis);
            if (a > codeLen)
                throw new ParserException ("Code out of range");
            code.add (o);
            if (c == OpcodeTable.OPCODE_WIDE)
                wide = true;
        }
        if (wide)
            throw new ParserException ();
        countExceptions = dis.readUnsignedShort ();
        exceptions = new AttrCodeExc[countExceptions];
        for (int a = 0; a < countExceptions; a ++) {
            AttrCodeExc e = new AttrCodeExc (this);
            e.fromClass (dis);
            exceptions[a] = e;
        }
        attrs = new Attributes (this, constPool);
        attrs.fromClass (dis);
    }
    
    public void toClass (DataOutputStream dos) throws Exception {
        dos.writeShort (maxStack);
        dos.writeShort (maxLocals);
        dos.writeInt (codeLen);
        for (int a = 0; a < code.size (); a ++)
            ((Opcode) code.get (a)).toClass (dos);
        dos.writeShort (countExceptions);
        for (int a = 0; a < countExceptions; a ++)
            exceptions[a].toClass (dos);
        attrs.toClass (dos);
    }
    
    public void fromJRAW (Lexan lex) throws Exception {
        lex.cmpLexel ("MaxStack");
        maxStack = (int) lex.getLong ();
        lex.cmpLexel ();
        lex.cmpLexel ("MaxLocals");
        maxLocals = (int) lex.getLong ();
        lex.cmpLexel ();
        lex.cmpLexel ("Code");
        lex.cmpLexel ("{");
        boolean wide = false;
        codeLen = 0;
        code = new ArrayList ();
        for (;;) {
            String s = lex.getLexel ();
            if ("}".equals (s))
                break;
            int c = OpcodeTable.getOpcodeByName (s);
            if (c < 0)
                throw new ParserException ("Bad opcode " + s + " " + codeLen);
            Opcode o;
            if (wide) {
                if (c == OpcodeTable.OPCODE_IINC)
                    o = new OpcodeWideIInc ();
                else if (c >= OpcodeTable.OPCODE_WIDE_LOAD_LOW  &&  c <= OpcodeTable.OPCODE_WIDE_LOAD_HIGH)
                    o = new OpcodeWideUnsignedOneArg ();
                else if (c >= OpcodeTable.OPCODE_WIDE_STORE_LOW  &&  c <= OpcodeTable.OPCODE_WIDE_STORE_HIGH)
                    o = new OpcodeWideUnsignedOneArg ();
                else
                    throw new ParserException ("Bad wide opcode " + s + " " + codeLen);
                wide = false;
            } else {
                Class cl = OpcodeTable.getClassByOpcode (c);
                if (cl == null)
                    throw new ParserException ("Bad opcode " + s + " " + codeLen);
                o = (Opcode) cl.newInstance ();
            }
            o.setParent (this);
            o.setPos (codeLen);
            codeLen ++;
            o.setOpcode (c);
            codeLen += o.fromJRAW (lex);
            code.add (o);
            if (c == OpcodeTable.OPCODE_WIDE)
                wide = true;
        }
        lex.cmpLexel ();
        lex.cmpLexel ("Exceptions");
        lex.cmpLexel ("{");
        ArrayList al = new ArrayList ();
        for (;;) {
            String s = lex.getLexel ();
            if ("}".equals (s))
                break;
            AttrCodeExc ace = new AttrCodeExc (this);
            ace.fromJRAW (lex);
            al.add (ace);
        }
        lex.cmpLexel ();
        countExceptions = al.size ();
        exceptions = new AttrCodeExc[countExceptions];
        for (int a = 0; a < countExceptions; a ++)
            exceptions[a] = (AttrCodeExc) al.get (a);
        attrs = new Attributes (this, constPool);
        attrs.fromJRAW (lex);
    }
    
    public void toJRAW (ExtStringBuffer esb) {
        esb.println ("MaxStack " + maxStack);
        esb.println ("MaxLocals " + maxLocals);
        esb.println ("Code { // " + codeLen);
        esb.incIndent ();
        ExtStringArray esa = constPool.esa;
        AttrLineNumberTable alnt = attrs.getAttrLineNumberTable ();
        if (esa != null  &&  alnt != null) {
            for (int a = 0; a < code.size (); a ++) {
                Opcode op = ((Opcode) code.get (a));
                String s = esa.readLine (alnt.getLineNumber(op.pos) - 1);
                if (s != null) {
                    esb.decIndent();
                    esb.println ("//" + s);
                    esb.incIndent ();
                }
                op.toJRAW (esb);
            }
        } else {
            for (int a = 0; a < code.size (); a ++) {
                ((Opcode) code.get (a)).toJRAW (esb);
            }
        }
        esb.decIndent ();
        esb.println ("}");
        esb.println ("Exceptions {");
        esb.incIndent ();
        for (int a = 0; a < countExceptions; a ++)
            exceptions[a].toJRAW (esb);
        esb.decIndent ();
        esb.println ("}");
        attrs.toJRAW (esb);
    }
    
}
