/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.jasm.parser;

public class StringUtil {

    public static String convertToProg (String str) {
        StringBuffer out = new StringBuffer ();
        for (int a = 0; a < str.length (); a ++) {
            int c = str.charAt (a);
            if (c == '\\')
                out.append ("\\\\"); // NOI18N
            else if (c == '"')
                out.append ("\\\""); // NOI18N
            else if (c == 10)
                out.append ("\\n"); // NOI18N
            else if (c == 9)
                out.append ("\\t"); // NOI18N
            else if (c < 32)
                out.append (("\\0" + (c >> 3)) + (c & 7)); // NOI18N
            else
                out.append ((char) c);
        }
        return out.toString ();
    }
    
    public static String convertFromProg (String str) {
        StringBuffer out = new StringBuffer ();
        for (int a = 0; a < str.length (); a ++) {
            int c = str.charAt (a);
            if (c == '\\') {
                a ++;
                if (a >= str.length ())
                    return null;
                c = str.charAt (a);
                switch (c) {
                    case '"':
                    case '\\':
                        out.append ((char) c);
                        break;
                    case 'n':
                        out.append ('\n');
                        break;
                    case 't':
                        out.append ('\t');
                        break;
                    case '0':
                        int b, d;
                        a ++;
                        if (a >= str.length ())
                            return null;
                        d = str.charAt (a);
                        if (d < '0'  ||  d > '9')
                            return null;
                        b = d - '0';
                        a ++;
                        if (a >= str.length ())
                            return null;
                        d = str.charAt (a);
                        if (d < '0'  ||  d > '9')
                            return null;
                        b = (b * 8) + (d - '0');
                        out.append ((char) b);
                        break;
                        default:
                            return null;
                }
            } else
                out.append ((char) str.charAt (a));
        }
        return out.toString ();
    }
    
    public static int decodeArg (String args, int pos) {
        int start = pos;
        while (pos < args.length ()  &&  args.charAt (pos) == '[') {
            pos ++;
        }
        if (pos < args.length ()) {
            switch (args.charAt (pos)) {
                case 'V':
                case 'B':
                case 'C':
                case 'D':
                case 'F':
                case 'I':
                case 'J':
                case 'S':
                case 'Z':
                    return pos + 1;
                case 'L':
                    pos ++;
                    while (pos < args.length ()) {
                        if (args.charAt (pos) == ';') {
                            return pos + 1;
                        }
                        pos ++;
                    }
                    return start;
            }
        }
        return start;
    }
    
    public static String decodeArg (String args, int start, int end) {
        int array = 0;
        while (start < end  &&  args.charAt (start) == '[') {
            start ++;
            array ++;
        }
        char[] chararray = new char[array * 2];
        for (int a = 2 * array - 2; a >= 0; a -= 2) {
            chararray[a] = '[';
            chararray[a + 1] = ']';
        }
        switch (args.charAt (start)) {
            case 'V':
                return "void" + new String (chararray); // NOI18N
            case 'B':
                return "byte" + new String (chararray); // NOI18N
            case 'C':
                return "char" + new String (chararray); // NOI18N
            case 'D':
                return "double" + new String (chararray); // NOI18N
            case 'F':
                return "float" + new String (chararray); // NOI18N
            case 'I':
                return "int" + new String (chararray); // NOI18N
            case 'J':
                return "long" + new String (chararray); // NOI18N
            case 'S':
                return "short" + new String (chararray); // NOI18N
            case 'Z':
                return "boolean" + new String (chararray); // NOI18N
            case 'L':
                start ++;
                return args.substring(start, end - 1).replace ('/', '.') + new String (chararray);
        }
        return args;
    }
    
    public static String decodeArgs (String args) {
        if (args == null  ||  args.length () <= 0) {
            return ""; // NOI18N
        }
        if (args.charAt (0) == '(') {
            String out = "("; // NOI18N
            int start = 1, end;
            for (;;) {
                end = decodeArg (args, start);
                if (start >= end)
                    break;
                if (start != 1) {
                    out += ", "; // NOI18N
                }
                out += decodeArg (args, start, end);
                start = end;
            }
            if (args.charAt (start) == ')') {
                start ++;
                out += ") -> "; // NOI18N
                end = decodeArg (args, start);
                out += decodeArg (args, start, end);
                if (end >= args.length ())
                    return out;
            }
            out += " DECODE_ERROR"; // NOI18N
            return out;
        } else {
            int end = decodeArg (args, 0);
            return (end < args.length ()) ? "DECODE_ERROR" : decodeArg (args, 0, end); // NOI18N
        }
    }
    
    public static String cpName (int cp) {
        if (cp > 0)
            return "#cp" + cp;
        else
            return Integer.toString (cp);
    }
    
}
