/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.jasm.syntax;

import java.util.HashMap;
import org.netbeans.editor.BaseTokenCategory;
import org.netbeans.editor.BaseTokenID;
import org.netbeans.editor.TokenID;
import org.netbeans.editor.BaseImageTokenID;
import org.netbeans.editor.TokenContext;
import org.netbeans.editor.TokenContextPath;


public class JRAWTokenContext extends TokenContext {

    public static final int WHITESPACE_ID = 1;
    public static final int INVALID_CHAR_ID = WHITESPACE_ID + 1;
    public static final int KEYWORDS_ID = INVALID_CHAR_ID + 1;
    public static final int INT_LITERAL_ID = KEYWORDS_ID + 1;
    public static final int FLOAT_LITERAL_ID = INT_LITERAL_ID + 1;
    public static final int STRING_LITERAL_ID = FLOAT_LITERAL_ID + 1;
    public static final int OPERATORS_ID = STRING_LITERAL_ID + 1;
    public static final int IDENTIFIER_ID = OPERATORS_ID + 1;
    public static final int LABEL_ID = IDENTIFIER_ID + 1;
    public static final int LINE_COMMENT_ID = LABEL_ID + 1;
    public static final int BLOCK_COMMENT_ID = LINE_COMMENT_ID + 1;
    public static final int LBRACE_ID = BLOCK_COMMENT_ID + 1;
    public static final int RBRACE_ID = LBRACE_ID + 1;
    public static final int COMMA_ID = RBRACE_ID + 1;
    public static final int MAJOR_VERSION_ID = COMMA_ID + 1;
    public static final int MINOR_VERSION_ID = MAJOR_VERSION_ID + 1;
    public static final int CONSTANT_POOL_ID = MINOR_VERSION_ID + 1;
    public static final int ACCESS_FLAG_ID = CONSTANT_POOL_ID + 1;
    public static final int THIS_ID = ACCESS_FLAG_ID + 1;
    public static final int SUPER_ID = THIS_ID + 1;
    public static final int INTERFACES_ID = SUPER_ID + 1;
    public static final int FIELDS_ID = INTERFACES_ID + 1;
    public static final int METHODS_ID = FIELDS_ID + 1;
    public static final int ATTRIBUTES_ID = METHODS_ID + 1;

    public static final BaseTokenID WHITESPACE
    = new BaseTokenID("whitespace", WHITESPACE_ID);
    
    public static final BaseTokenID INVALID_CHAR
    = new BaseTokenID("invalid-char", INVALID_CHAR_ID);
    
    public static final BaseTokenCategory KEYWORDS
    = new BaseTokenCategory("keywords", KEYWORDS_ID);
    
    public static final BaseTokenCategory OPERATORS
    = new BaseTokenCategory("operators", OPERATORS_ID);
    
    public static final BaseTokenID IDENTIFIER
    = new BaseTokenID("identifier", IDENTIFIER_ID);
    
    public static final BaseTokenID LABEL
    = new BaseTokenID("label", LABEL_ID);
    
    public static final BaseTokenID LINE_COMMENT
    = new BaseTokenID("line-comment", LINE_COMMENT_ID);
    
    public static final BaseTokenID BLOCK_COMMENT
    = new BaseTokenID("block-comment", BLOCK_COMMENT_ID);
    
    public static final BaseTokenID STRING_LITERAL
    = new BaseTokenID("string-literal", STRING_LITERAL_ID);
    
    public static final BaseTokenID INT_LITERAL
    = new BaseTokenID("int-literals", INT_LITERAL_ID);
    
    public static final BaseTokenID FLOAT_LITERAL
    = new BaseTokenID("float-literals", FLOAT_LITERAL_ID);
    
    public static final BaseImageTokenID LBRACE
    = new BaseImageTokenID("lbrace", LBRACE_ID, OPERATORS, "{");
    
    public static final BaseImageTokenID RBRACE
    = new BaseImageTokenID("rbrace", RBRACE_ID, OPERATORS, "}");
    
    public static final BaseImageTokenID COMMA
    = new BaseImageTokenID("comma", COMMA_ID, OPERATORS, ",");
    
    public static final BaseImageTokenID MAJOR_VERSION
    = new BaseImageTokenID("MajorVersion", MAJOR_VERSION_ID, KEYWORDS);

    public static final BaseImageTokenID MINOR_VERSION
    = new BaseImageTokenID("MinorVersion", MINOR_VERSION_ID, KEYWORDS);

    public static final BaseImageTokenID CONSTANT_POOL
    = new BaseImageTokenID("ConstantPool", CONSTANT_POOL_ID, KEYWORDS);
    
    public static final BaseImageTokenID ACCESS_FLAG
    = new BaseImageTokenID("AccessFlag", ACCESS_FLAG_ID, KEYWORDS);
    
    public static final BaseImageTokenID THIS
    = new BaseImageTokenID("This", THIS_ID, KEYWORDS);
    
    public static final BaseImageTokenID SUPER
    = new BaseImageTokenID("Super", SUPER_ID, KEYWORDS);
    
    public static final BaseImageTokenID INTERFACES
    = new BaseImageTokenID("Interfaces", INTERFACES_ID, KEYWORDS);
    
    public static final BaseImageTokenID FIELDS
    = new BaseImageTokenID("Fields", FIELDS_ID, KEYWORDS);
    
    public static final BaseImageTokenID METHODS
    = new BaseImageTokenID("Methods", METHODS_ID, KEYWORDS);
    
    public static final BaseImageTokenID ATTRIBUTES
    = new BaseImageTokenID("Attributes", ATTRIBUTES_ID, KEYWORDS);
    
    // Context instance declaration
    public static final JRAWTokenContext context = new JRAWTokenContext();
    
    public static final TokenContextPath contextPath = context.getContextPath();
    
    private static final HashMap str2kwd = new HashMap();
    private static final HashMap str2oper = new HashMap();
    
    static {
        BaseImageTokenID[] kwds = new BaseImageTokenID[] {
            MAJOR_VERSION, MINOR_VERSION, CONSTANT_POOL, ACCESS_FLAG,
            THIS, SUPER, INTERFACES, FIELDS, METHODS, ATTRIBUTES
        };
        BaseImageTokenID[] opers = new BaseImageTokenID[] {
            LBRACE, RBRACE, COMMA
        };
        for (int i = kwds.length - 1; i >= 0; i--) {
            str2kwd.put(kwds[i].getImage(), kwds[i]);
        }
        for (int i = opers.length - 1; i >= 0; i--) {
            str2oper.put(opers[i].getImage(), opers[i]);
        }
    }
    
    /** Get the keyword token-id from string */
    public static TokenID getKeyword(String s) {
        return (TokenID)str2kwd.get(s);
    }
    
    public static TokenID getOperator(String s) {
        return (TokenID)str2oper.get(s);
    }
    
    private JRAWTokenContext() {
        super("jraw-");
        
        try {
            addDeclaredTokenIDs();
        } catch (Exception e) {
            if (Boolean.getBoolean("netbeans.debug.exceptions")) { // NOI18N
                e.printStackTrace();
            }
        }
        
    }
    
}
