/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.server.uihandler;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import javax.servlet.jsp.PageContext;
import org.netbeans.junit.NbTestCase;
import org.netbeans.modules.exceptions.bugs.BugReporterFactory;
import org.netbeans.modules.exceptions.utils.HttpUtils;
import org.netbeans.modules.exceptions.utils.PersistenceUtils;
import org.netbeans.server.componentsmatch.Matcher;
import org.netbeans.server.uihandler.StatisticsTest.PgCnt;

/**
 *
 * @author Jindrich Sedek
 */
public class DatabaseTestCase extends NbTestCase{
    private final static String[] TEST_COMPONENTS = {
        "openide=util,bbb,ccc,ddd",
        "core=code,222,333,444", 
        "subversion=code,444,555",
        "web=jspsyntax"};
    private final static String TEST_FILE = "components.properties";

    private static Logger LOG;
    protected PersistenceUtils perUtils;
    protected File data;
    protected LogsManager logsManager;
    protected PageContext page;
    
    
    public static final void initPersitenceUtils(File workDir) {
        PersistenceUtils.initTestPersistanceUtils(new File(workDir, "db"));
    }
    
    public DatabaseTestCase(String str) {
        super(str);
    }
    
    @Override
    protected void setUp() throws Exception {
        LOG = Logger.getLogger("test." + getName());

        clearWorkDir();
        File directory = getWorkDir().getAbsoluteFile();
        
        File testFile = new File(directory + File.separator + TEST_FILE);
        PrintWriter out = new PrintWriter(new FileOutputStream(testFile));
        for (int i = TEST_COMPONENTS.length - 1; i >= 0; i--) {
            out.println(TEST_COMPONENTS[i]);
        }
        out.close();
        PersistenceUtils.setSettingsDirectory(directory.toString());
        
        InputStream inputStream = DatabaseTestCase.class.getResourceAsStream("testPrivKey");
        DbInsertion.loadPrivateKey(inputStream);
        DbInsertion.setDuplicatesNo(100l);
        Matcher.read(new InputStreamReader(Matcher.class.getResourceAsStream("componentmapping.txt")));
        initPersitenceUtils(getWorkDir());
        BugReporterFactory.setDefault(BugReporterFactory.Reporters.LogReporter);
        perUtils = PersistenceUtils.getInstance();
        data = new File(getWorkDir(), "data");
        data.mkdirs();
        logsManager = LogsManager.createManager(data);
        page = PgCnt.create();
        HttpUtils.setTestMode(true);
    }
    
    @Override
    protected Level logLevel() {
        return Level.INFO;
    }
    
    @SuppressWarnings("unchecked")
    protected Map<Integer, LogRecord> addLog(PageContext page, File log, String ip, String userId) throws Exception{
        logsManager.addLog(log, ip);
        waitLogsParsed();
        LOG.fine("Feeding page for user " + userId);
        logsManager.preparePageContext(page, userId, null);
        return (Map<Integer, LogRecord>) page.getAttribute("lastExceptions");
    }
    
    protected final void waitLogsParsed() throws Exception {
        LOG.fine("waitLogsParsed");
        logsManager.preparePageContext(LogsManagerTest.createPageContext(), "noID", null);
        LOG.fine("waitLogsParsed - after preparePageContext");
        DbInsertion.waitLogFileInsertFinished();
        DbInsertion.waitIssueInsertFinished();
        LOG.fine("waitLogsParsed - after DbInsertion");
    }
    
    protected Map<Integer, LogRecord> addLog(PageContext page, File log, String userId) throws Exception{
        return addLog(page, log, "127.0.0.1", userId);
    }

    protected Map<Integer, LogRecord> addLog(File log, String userId) throws Exception{
        return addLog(page, log, "127.0.0.1", userId);
    }

    protected void waitLog(File log, String ip) throws Exception{
        logsManager.addLog(log, ip);
        waitLogsParsed();
    }

}
