/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.server.uihandler;

import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import org.junit.Test;
import org.netbeans.modules.exceptions.entity.Comment;
import org.netbeans.modules.exceptions.entity.Jarfile;
import org.netbeans.modules.exceptions.entity.Line;
import org.netbeans.modules.exceptions.entity.Logfile;
import org.netbeans.modules.exceptions.entity.Nbuser;
import org.netbeans.server.componentsmatch.Component;

/**
 *
 * @author Jindrich Sedek
 */
public class DbInsertionTest extends DatabaseTestCase{
    private static int cnt = 0;
    
    public DbInsertionTest(String testName) {
        super(testName);
    }
    
    
    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    @Override
    protected Level logLevel() {
        return Level.FINE;
    }
    
    public void testDoBothTests() throws Exception {
        doRun();
        doIssueZillaId();
    }
    
    
    private void doRun() throws Exception {
        
        
        List<Logfile> logFiles = perUtils.getAll(Logfile.class);
        assertEquals("Empty at start", 0, logFiles.size());
        
        LogRecord rec= new LogRecord(Level.SEVERE, "TESTING LOG RECORD");
        String[] users = {"GUEST", "GUEST"};
        String[] comments = {"COMMENT", "[subversion/code] \n test"};
        String[] productVersion = {"NetBeans IDE Dev (Build 200702121900)", "NetBeans IDE Dev (Build 20070416-1344)"};
        String[] params = new String[7];
        String[] fileNames = {"file.2", "file.3"};
        params[0] = "Linux, 2.6.15-1.2054_FC5smp, i386";
        params[1] = "Java HotSpot(TM) Client VM, 1.6.0-b105";
        params[2] = productVersion[0];
        params[3] = users[0];
        params[4] = "UnknownError : Summary Message";
        params[5] = comments[0];
        params[6] = "test";
        String userId = "file";
        int session = 3;
        
        rec.setParameters(params);
        Throwable thrown = new NullPointerException("TESTING NULL POINTER");
        
        
        DbInsertion insertion = new DbInsertion(rec, thrown, userId, session);
        Object[] insertionResult = insertion.start();
        assertFalse("THIS ISSUE IS NOT ANY DUPLICATE", (Boolean) insertionResult[1]);
        assertEquals("NO ISSUEZILLA ID", 0, insertionResult[2]);
        waitLogsParsed();
        
        List<Nbuser> usersList = perUtils.getAll(Nbuser.class);
        Nbuser user = usersList.get(0);
        assertEquals("THERE MUST BE ONE USER", 1, usersList.size());
        assertEquals("USER NAME CHECK", user.getName(), params[3]);
        
        List<org.netbeans.modules.exceptions.entity.Exceptions> exceptionList = perUtils.getAll(org.netbeans.modules.exceptions.entity.Exceptions.class);
        org.netbeans.modules.exceptions.entity.Exceptions exception = exceptionList.get(0);
        assertEquals("THERE MUST BE ONE ISSUE", 1, exceptionList.size());
        assertEquals("THIS ISSUE WAS ADDED WITH ID", insertionResult[0], exception.getId());
        assertEquals("SUMMARY CHECK", exception.getSummary(), params[4]);
        assertEquals(fileNames[0], exception.getLoggername());
        assertEquals("VM MACHINE CHECK", exception.getVm(), params[1]);
        assertEquals("NB PRODUCT VERSION CHECK", exception.getProductversion(), params[2]);
        assertEquals("OPERATING SYSTEM NAME CHECK", exception.getOperatingsystem(), params[0]);
        assertEquals("PRODUCT BUILD NUMBER", exception.getBuild(), Long.valueOf("070212"));
        assertNull("THIS SHOULD NOT BE A DUPLICATE", exception.getDuplicateof());// NULL
        assertEquals("STACKTRACE LINES NUMBER", thrown.getStackTrace().length, exception.getStacktrace().getLineCollection().size());

        List<Line> linesList = perUtils.getAll(Line.class);
        Iterator<Line> lineIterator = linesList.iterator();
        assertTrue("THERE MUST BE FEW LINES", linesList.size() > 0);
        List<StackTraceElement> list = Arrays.asList(thrown.getStackTrace());
        do{
            Line line = lineIterator.next();
            assertNotNull("JarFile is not inserted", line.getJarfileId());
            int lineOrder = line.getLinePK().getLineOrder();
            int lineNumber = line.getLinePK().getLinenumber();
            int stackLineNumber = list.get(lineOrder).getLineNumber();
            if (stackLineNumber < 0) stackLineNumber = 0;
            assertEquals(lineNumber, stackLineNumber);
        }while (lineIterator.hasNext());
        assertEquals(linesList.size(), list.size());
        
        Integer count = perUtils.count(Jarfile.class);
        assertTrue("THE NUMBER MUST BE NOT 0", count > 0);
        
        logFiles = AddLogFileTest.logFiles(perUtils);
        assertEquals("JUST ONE logfile: " + logFiles,1, logFiles.size());
        assertEquals("Build Number", logFiles.get(0).getBuildnumber(), Long.valueOf("070212"));
        assertEquals("Upload Number", logFiles.get(0).getUploadNumber(), new Integer(session));
        assertEquals("userDir", logFiles.get(0).getUserdir(), userId);
        assertEquals("File Name", logFiles.get(0).getFileName(), userId+"."+String.valueOf(session-1));
        assertNull("IP is null - it was not set", logFiles.get(0).getIp());
        
        //********* ADD DUPLICATE *********/
        params[3] = users[1];
        params[5] = comments[1];
        params[2] = productVersion[1];
        rec.setParameters(params);
        insertion = new DbInsertion(rec, thrown, userId, session+1);
        insertionResult = insertion.start();
        assertTrue("THIS ISSUE IS A DUPLICATE", (Boolean) insertionResult[1]);
        assertEquals("DUPLICATE INSERTION", 1, insertionResult[0]);
        assertEquals("NO ISSUEZILLA ID", 0, insertionResult[2]);
        waitLogsParsed();
        
        Integer nextCount = perUtils.count(Jarfile.class);
        assertEquals("THE NUMBER OF JAR FILES MUST BE THE SAME", count, nextCount);
        
        usersList = perUtils.getAll(Nbuser.class);
        user = usersList.get(0);
        assertEquals("THERE MUST BE USER", 1, usersList.size());
        assertEquals(users[0], user.getName());
        
        
        
        List<Comment> commentList = perUtils.getAll(Comment.class);
        assertEquals("THERE SHOULD BE TWO COMMENTS", 2, commentList.size());
        //first record
        Comment comment = commentList.get(0);
        assertEquals(comments[0],comment.getComment());
        
        comment = commentList.get(1);
        //second record
        assertEquals(comments[1],comment.getComment());
        
        exceptionList = perUtils.getAll(org.netbeans.modules.exceptions.entity.Exceptions.class);
        exception = exceptionList.get(0);
        assertEquals("THERE MUST BE ONE ISSUE", 2, exceptionList.size());
        //check the first one
        assertEquals("THIS ISSUE WAS ADDED WITH ID", new Integer(1), exception.getId());
        assertEquals("SUMMARY CHECK", exception.getSummary(), params[4]);
        assertEquals(fileNames[0], exception.getLoggername());
        assertEquals("VM MACHINE CHECK", exception.getVm(), params[1]);
        assertEquals("NB PRODUCT VERSION CHECK", exception.getProductversion(), productVersion[0]);
        assertEquals("PRODUCT BUILD NUMBER", exception.getBuild(),Long.valueOf("070212"));
        assertEquals("OPERATING SYSTEM NAME CHECK", exception.getOperatingsystem(), params[0]);
        assertNull("THIS SHOULD NOT BE A DUPLICATE", exception.getDuplicateof());// NULL
        //check the second one
        exception = exceptionList.get(1);
        assertEquals("THIS ISSUE WAS ADDED WITH ID", new Integer(2), exception.getId());
        assertEquals("SUMMARY CHECK", exception.getSummary(), params[4]);
        assertEquals(fileNames[1], exception.getLoggername());
        assertEquals("VM MACHINE CHECK", exception.getVm(), params[1]);
        assertEquals("NB PRODUCT VERSION CHECK", exception.getProductversion(), productVersion[1]);
        assertEquals("PRODUCT BUILD NUMBER", exception.getBuild(), Long.valueOf("070416"));
        assertEquals("OPERATING SYSTEM NAME CHECK", exception.getOperatingsystem(), params[0]);
        assertNotNull("THIS SHOULD BE A DUPLICATE", exception.getDuplicateof());
        assertEquals("THIS SHOULD BE A DUPLICATE OF ISSUE 1", exception.getDuplicateof(), exceptionList.get(0));
        assertEquals("COMPONENT FROM COMMENT", exception.getComponent(), "subversion");
        assertEquals("SUBCOMPONENT FROM COMMENT", exception.getSubcomponent(), "code");
        
        
        logFiles = AddLogFileTest.logFiles(perUtils);
        assertEquals("TWO logfile",2, logFiles.size());
        //--- FIRST --//
        Logfile logFile = logFiles.get(0);
        assertEquals("Build Number", logFile.getBuildnumber(), Long.valueOf("070212"));
        assertEquals("Upload Number", logFile.getUploadNumber(), new Integer(session));
        assertEquals("userDir", logFile.getUserdir(), userId);
        assertEquals("File Name", logFile.getFileName(), userId+"."+String.valueOf(session-1));
        assertNull("IP is null - it was not set", logFile.getIp());
        assertNotNull(logFile.getProductVersionId());
        assertEquals("PRODUCT VERSION CHECK",  productVersion[0], logFile.getProductVersionId().getProductVersion());
        //---SECOND--//
        logFile = logFiles.get(1);
        assertEquals("Build Number", logFile.getBuildnumber(), Long.valueOf("070416"));
        assertEquals("Upload Number", logFile.getUploadNumber(), new Integer(session+1));
        assertEquals("userDir", logFile.getUserdir(), userId);
        assertEquals("File Name", logFile.getFileName(), userId+"."+String.valueOf(session));
        assertNull("IP is null - it was not set", logFile.getIp());
        assertNotNull(logFile.getProductVersionId());
        assertEquals("PRODUCT VERSION CHECK",  productVersion[1], logFile.getProductVersionId().getProductVersion());
    }
    
    private void doIssueZillaId() throws Exception {
        LogRecord rec= new LogRecord(Level.SEVERE, "TESTING LOG RECORD");
        String[] users = {"userName", "SECOND USER"};
        String[] comments = {"COMMENT", "SECOND COMMENT"};
        String[] productVersion = {"NetBeans IDE Dev (Build 200702121900)", "NetBeans IDE Dev (Build 20070416-1344)"};
        String[] params = new String[6];
        String[] fileNames = {"file.2", "file.3"};
        params[0] = "Linux, 2.6.15-1.2054_FC5smp, i386";
        params[1] = "Java HotSpot(TM) Client VM, 1.6.0-b105";
        params[2] = productVersion[0];
        params[3] = users[0];
        params[4] = "UnknownError : Summary Message";
        params[5] = comments[0];
        String userId = "file";
        int issueZillaId = 10;
        int session = 3;
        
        rec.setParameters(params);
        Throwable thrown = new NullPointerException("TESTING NULL POINTER");
        
        
        DbInsertion insertion = new DbInsertion(rec, thrown, userId, session);
        Object[] insertionResult = insertion.start();
        //called from different class => different stackTrace
        assertFalse("THIS ISSUE IS NOT ANY DUPLICATE", (Boolean) insertionResult[1]);
        assertEquals("NEW INSERTION NEW ID", 3, insertionResult[0]);
        assertEquals("NO ISSUEZILLA ID", 0, insertionResult[2]);
        waitLogsParsed();
        
        List<org.netbeans.modules.exceptions.entity.Exceptions> exceptionList = perUtils.getAll(org.netbeans.modules.exceptions.entity.Exceptions.class);
        assertEquals("LIST SIZE", 3, exceptionList.size());
        org.netbeans.modules.exceptions.entity.Exceptions exception = exceptionList.get(2);
        exception.setIssuezillaid(issueZillaId);
        perUtils.merge(exception);

        // ADD DUPLICATE
        insertion = new DbInsertion(rec, thrown, userId, session);
        insertionResult = insertion.start();
        assertTrue("THIS ISSUE A DUPLICATE", (Boolean) insertionResult[1]);
        assertEquals("DUPLICATE INSERTION", 3, insertionResult[0]);
        waitLogsParsed();

        insertion = new DbInsertion(rec, thrown, userId, session);
        insertionResult = insertion.start();
        assertTrue("THIS ISSUE A DUPLICATE", (Boolean) insertionResult[1]);
        assertEquals("DUPLICATE INSERTION", 3, insertionResult[0]);
        assertEquals("ISSUEZILLA ID", 10, insertionResult[2]);
        waitLogsParsed();
        
        
    }

    
    @Test
    public void testGetComponentFromComment() throws Exception {
        String comment = "[web/jspsyntax] \n this is my comment";
        Component comp = DbInsertion.getComponentFromComment(comment);
        assertEquals(comp.getComponent(), "web");
        assertEquals(comp.getSubComponent(), "jspsyntax");
        
        comment = "unknown/nnnnnnnnnnnnn";
        comp = DbInsertion.getComponentFromComment(comment);
        assertNull("NO BRACKETS", comp);

        comment = "[unknown/nnnnnnnnnnnnn]";
        comp = DbInsertion.getComponentFromComment(comment);
        assertNull("NON EXISTING COMPONENT", comp);
        
    }   
}