/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.server.uihandler;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import org.openide.util.NbBundle;

/**
 *
 * @author Jindrich Sedek
 */
public class ExceptionsTest extends DatabaseTestCase {
    
    public ExceptionsTest(String testName) throws IOException {
        super(testName);
    }

    @Override
    protected Level logLevel() {
        return Level.FINE;
    }
    
    
    
    public void testProcess() {
        Exceptions exceptions = new Exceptions();
        LogRecord log = getDataLog();
        LogRecord logThrown = getThrownLog();
        Map<Integer, LogRecord> mapa = exceptions.process(log);
        assertEquals("PROCESS DATA", log, mapa.get(0));
        mapa = exceptions.process(logThrown);
        assertEquals("PROCESS EXCEPTION", logThrown, mapa.get(1));
        mapa = exceptions.process(getInfoLog());
        assertNull("PROCESS INFO", mapa.get(1));
    }
    
    public void testJoin() {
        Exceptions exceptions = new Exceptions();
        Map<Integer, LogRecord> empty = exceptions.newData();
        Map<Integer, LogRecord> data = exceptions.process(getDataLog());
        Map<Integer, LogRecord> thr = exceptions.process(getThrownLog());
        LogRecord log2 = getThrownLog();
        log2.setThrown(new UnknownError("ERRRROR"));
        Map<Integer, LogRecord> thr2 = exceptions.process(log2);
        Map<Integer, LogRecord> result;
        result = exceptions.join(empty, data);
        assertEquals("JOIN DATA", data, result);
        result = exceptions.join(empty, thr);
        assertEquals("JOIN THROWN", thr, result);
        result = exceptions.join(data, thr);
        assertEquals("JOIN BOTH", data.get(0), result.get(0));
        assertEquals("JOIN BOTH", thr.get(1), result.get(1));
        Map<Integer, LogRecord> result2;
        result2 = exceptions.join(result, empty);
        assertEquals("HOLD DATA", result, result2);
        result2 = exceptions.join(thr2, result);
        assertEquals("GET LAST EXCEPTION", result, result2);
        result2 = exceptions.join(result, thr2);
        if (result.equals(result2)) throw new AssertionError("EXCEPTION SHOULD BE REVRITEN BE THE LAST ONE");
        assertEquals("DATA SHOULD STAY THE SAME", result.get(0), result2.get(0));
        assertNotNull(result.get(0));
        assertEquals("EXCEPTION SHOULD BE REVRITEN BE THE LAST ONE", thr2.get(1), result2.get(1));
        assertNotNull(result.get(1));
    }
    
    public void testFinishSessionUpload() throws Exception {
        String userId = "log4445";
        Exceptions exceptions = new Exceptions();
        Map<Integer, LogRecord> data = exceptions.process(getDataLog());
        Map<Integer, LogRecord> thr = exceptions.process(getThrownLog());
        Map<Integer, LogRecord> both = exceptions.newData();
        both = exceptions.join(both, thr);
        both = exceptions.join(data, both);
        Map<Integer, LogRecord> result = exceptions.finishSessionUpload(userId, 1, false, data);
        checkResult("NOT INITIAL UPLOAD",result, -2, true, false);//not INITIAL
        result = exceptions.finishSessionUpload(userId, 1, true, thr);
        checkResult("NO METADATA IN REPORT", result, -2, true, false);//not META DATA
        result = exceptions.finishSessionUpload(userId, 1, true, data);
        checkResult("NO THROWN IN REPORT", result, -2, true, false);//not THROWN
        waitLogsParsed();
        result = exceptions.finishSessionUpload(userId, 1, true, both);
        
        checkResult("THERE SHOULD BE NO DUPLICATE OF THIS ISSUE", result, 1, false, false);//not DUPLICATE
        waitLogsParsed();
        
        result = exceptions.finishSessionUpload(userId, 1, true, both);// INSERT ONCE MORE
        checkResult("THIS ISSUE SHOULD BE A DUPLICATE OF #1", result, 1, false, true);//duplicate of issue no 1
    }
    
    private void checkResult(String str, Map<Integer, LogRecord> result, Integer issueId, boolean isNull, Boolean duplicate){
        assertNotNull(result);
        LogRecord rec = result.get(2);
        assertNotNull("RECORD SHOULD BE NEVER NULL", rec);
        Object[] params = rec.getParameters();
        if (isNull) assertNull("RECORD SHOULD BE NULL", params);
        else{
            assertNotNull("RECORD SHOULD BE NOT NULL", params);
            assertEquals(str, issueId, params[0]);
            assertEquals(str, duplicate, params[1]);
        }
    }
    
    private LogRecord getDataLog(){
        LogRecord rec = new LogRecord(Level.CONFIG, Exceptions.USER_CONFIGURATION);
        ArrayList<String> params = new ArrayList<String>(6);
        params.add("Linux");
        params.add("HOT SPOT");
        params.add("IDE DEV");
        params.add("USER NAME");
        params.add("SUMMARY");
        params.add("COMMENT");
        rec.setResourceBundle(NbBundle.getBundle(ExceptionsTest.class));
        rec.setParameters(params.toArray());
        rec.setResourceBundleName("MUJ BUNDLE");
        return rec;
    }
    
    private LogRecord getThrownLog(){
        Throwable thrown = new NullPointerException("TESTING EXCEPTION");
        LogRecord logThrown = new LogRecord(Level.SEVERE, "CHYBA");
        logThrown.setThrown(thrown);
        return logThrown;
    }
    
    private LogRecord getInfoLog(){
        Throwable thrown = new NullPointerException("TESTING EXCEPTION");
        LogRecord logThrown = new LogRecord(Level.INFO, "WARNING");
        logThrown.setThrown(thrown);
        return logThrown;
    }
    
}
