/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.server.uihandler;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import org.netbeans.modules.exceptions.bugs.BugReporterFactory;
import org.netbeans.modules.exceptions.entity.Directuser;
import org.netbeans.modules.exceptions.entity.Issue;
import org.netbeans.modules.exceptions.entity.Nbuser;

/**
 *
 * @author Jindrich Sedek
 */
public class IssuezillaInsertionTest extends DatabaseTestCase{
    
    public IssuezillaInsertionTest(String str)  {
        super(str);
    }
    
    public void testIssuezillaInsertion() throws Exception{
        TestHandler handler = new TestHandler();
        Logger.getLogger(BugReporterFactory.class.getName()).addHandler(handler);
        DbInsertion.setDuplicatesNo(new Long(1));
        File log = LogsManagerTest.extractResourceAs(data, "dup3", "NB2040811605.1");
        waitLog(log, "127.0.0.1");
        assertNull("NO DUPLICATE", handler.getResult());
        waitLog(log, "127.0.0.1");
        assertNull("ONE DUPLICATE", handler.getResult());
        log = LogsManagerTest.extractResourceAs(data, "dup1_1", "NB2040811605.0");
        waitLog(log, "127.0.0.1");
        assertNull("TWO DUPLICATES", handler.getResult());
        waitLog(log, "127.0.0.1");
        log = LogsManagerTest.extractResourceAs(data, "dup1", "NB2040811605.0");
        assertNull("THREE DUPLICATES", handler.getResult());
        waitLog(log, "127.0.0.1");
        assertNull("UNKNOWN USER", handler.getResult());
        log = LogsManagerTest.extractResourceAs(data, "dup3", "NB2040811605.1");
        waitLog(log, "127.0.0.1");
        LogRecord result = handler.getResult();
        assertNotNull(result);
        Object[] params = result.getParameters();
        assertEquals("NullPointerException", params[0]);
        assertEquals("exceptions", params[1]);
        assertEquals("petrzajac", params[2]);
        handler.flush();
        log = LogsManagerTest.extractResourceAs(data, "dup1", "NB2040811605.0");
        waitLog(log, "127.0.0.1");
        assertNull("UNKNOWN USER", handler.getResult());
        log = LogsManagerTest.extractResourceAs(data, "dup3", "NB2040811605.1");
        waitLog(log, "127.0.0.1");
        assertNull("MANY DUPLICATES FROM ONE USER", handler.getResult());
        
        //since issue 114597, 114598 OUT OF MEMORY WAS CHANGED TO NPE
        log = LogsManagerTest.extractResourceAs(data, "outOfMem", "NB2040811605.10");
        waitLog(log, "112.15.14.13");
        result = handler.getResult();
        assertNotNull(result);
        params = result.getParameters();
        assertEquals("OutOfMemoryError", params[0]);
        assertEquals("exceptions", params[1]);
        assertEquals("petrzajac", params[2]);
    }
    
    public void testQEUserFilter() throws Exception{
        TestHandler handler = new TestHandler();
        Logger.getLogger(BugReporterFactory.class.getName()).addHandler(handler);
        DbInsertion.setDuplicatesNo(new Long(2));

        String directUserName = "exceptions";
        LogRecord rec= new LogRecord(Level.SEVERE, "TESTING LOG RECORD");
        ArrayList<String> params = new ArrayList<String>(7);
        params.add("Linux, 2.6.15-1.2054_FC5smp, i386");
        params.add("Java HotSpot(TM) Client VM, 1.6.0-b105");
        params.add("NetBeans IDE Dev (Build 200702121900)");
        params.add(directUserName);
        String summary = "UnknownError : Summary Message";
        params.add(summary);
        params.add("COMMENT");
        params.add("petrzajac");
        rec.setParameters(params.toArray());
        Throwable thrown = new NullPointerException("TESTING NULL POINTER");
        new DbInsertion(rec, thrown, "file", 1).start();
        waitLogsParsed();

        LogRecord result = handler.getResult();
        assertNull("NOT A DIRECT USER", result);
        
        perUtils.persist(new Directuser(directUserName));

        Object[] dbInsertionResult =  new DbInsertion(rec, thrown, "file", 1).start();
        Integer issueId = (Integer) dbInsertionResult[0];
        waitLogsParsed();

        result = handler.getResult();
        assertNotNull("DIRECT USER", result);
        Object[] parameters = result.getParameters();
        assertEquals(summary, parameters[0]);
        assertEquals("exceptions", parameters[1]);
        assertEquals("petrzajac", parameters[2]);
        handler.flush();
        
        int issuezillaId = 10;
        org.netbeans.modules.exceptions.entity.Exceptions exc =
                 perUtils.getEntity(org.netbeans.modules.exceptions.entity.Exceptions.class, issueId);
        exc.setIssuezillaid(issuezillaId);
        perUtils.merge(exc);

        String undirectUser = "tester";
        params.set(3, undirectUser);
        params.set(6, undirectUser);
        rec.setParameters(params.toArray());
        dbInsertionResult =  new DbInsertion(rec, thrown, "file", 1).start();
        waitLogsParsed();
        result = handler.getResult();
        assertNotNull("MANY DUPLICATES FILTER - ADD COMMENT", result);
        parameters = result.getParameters();
        assertEquals(Integer.toString(issuezillaId), parameters[0].toString());
        assertEquals(undirectUser, parameters[1]);
        assertEquals(undirectUser, parameters[2]);
        String str = (String) parameters[3];
        assertNotNull("GENERATED MESSAGE", str);
        assertTrue(str.contains("MORE"));
        handler.flush();
        
        Issue issue = new Issue();
        issue.setIssueId(issuezillaId);
        issue.setIssueStatus("RESOLVED");
        perUtils.persist(issue);
        
        assertTrue(perUtils.issueIsFixed(issuezillaId));
        
        
        new DbInsertion(rec, thrown, "file", 1).start();
        waitLogsParsed();
        
        assertNull("CLOSED ISSUE", handler.getResult());
    }
    
    private Integer rootId;
    
    public void testGetDuplicates() throws Exception{
        LogRecord rec= new LogRecord(Level.SEVERE, "TESTING LOG RECORD");
        String userName = "GUEST";
        ArrayList<String> params = new ArrayList<String>(7);
        params.add("Linux, 2.6.15-1.2054_FC5smp, i386");
        params.add("Java HotSpot(TM) Client VM, 1.6.0-b105");
        params.add("NetBeans IDE Dev (Build 200702121900)");
        params.add(userName);
        params.add("UnknownError : Summary Message");
        params.add("COMMENT");
        params.add("test");
        rec.setParameters(params.toArray());
        Throwable thrown = new NullPointerException("TESTING NULL POINTER");

        new DbInsertion(rec, thrown, "file", 1).start();
        waitLogsParsed();
        List<org.netbeans.modules.exceptions.entity.Exceptions> all = 
                perUtils.getAll(org.netbeans.modules.exceptions.entity.Exceptions.class);
        org.netbeans.modules.exceptions.entity.Exceptions rootException = all.get(all.size()-1);
        rootId = rootException.getId();
        List<Nbuser> users = perUtils.getAll(Nbuser.class);
        Nbuser user=null;
        for (Nbuser us : users) {
            if (userName.equals(us.getName())) user = us;
        }
        assertNotNull(rootException);
        assertEquals(0, rootException.getDuplicates());
        assertEquals(0, rootException.getDuplicatesFromDistinctUsers());
        assertEquals(0, rootException.getDuplicatesFromOneUser(user));
        
        rootException = insert(new DbInsertion(rec, thrown, "file", 2));
        assertEquals(1, rootException.getDuplicates());
        assertEquals(1, rootException.getDuplicatesFromDistinctUsers());
        assertEquals(1, rootException.getDuplicatesFromOneUser(user));
        
        
        rootException = insert(new DbInsertion(rec, thrown, "file", 3));
        assertEquals(2, rootException.getDuplicates());
        assertEquals(1, rootException.getDuplicatesFromDistinctUsers());
        assertEquals(2, rootException.getDuplicatesFromOneUser(user));
        userName = "exceptions";
        params.set(3, userName);
        params.set(6, "petrzajac");
        rec.setParameters(params.toArray());

        rootException = insert(new DbInsertion(rec, thrown, "file", 4));
        users = perUtils.getAll(Nbuser.class);
        Iterator<Nbuser> userIt = users.iterator();
        Nbuser user2 = null;
        while (userIt.hasNext()){
            user2 = userIt.next();
            if (user2.getName().equals(userName)) break;
        }
        assertEquals(3, rootException.getDuplicates());
        assertEquals(2, rootException.getDuplicatesFromDistinctUsers());
        assertEquals(2, rootException.getDuplicatesFromOneUser(user));
        assertEquals(1, rootException.getDuplicatesFromOneUser(user2));
        
        rootException = insert(new DbInsertion(rec, thrown, "file", 5));
        assertEquals(4, rootException.getDuplicates());
        assertEquals(2, rootException.getDuplicatesFromDistinctUsers());
        assertEquals(2, rootException.getDuplicatesFromOneUser(user));
        assertEquals(2, rootException.getDuplicatesFromOneUser(user2));
    }

    private org.netbeans.modules.exceptions.entity.Exceptions insert(DbInsertion dbInsertion) throws Exception{
        Object[] result = dbInsertion.start();
        assertTrue((Boolean)result[1]);
        assertEquals(rootId, (Integer)result[0]);
        waitLogsParsed();
        return perUtils.getEntity(org.netbeans.modules.exceptions.entity.Exceptions.class, rootId);
    }
    
    private class TestHandler extends Handler{
        LogRecord result;
        public void publish(LogRecord record) {
            if ("BUG REPORTER".equals(record.getMessage())) result = record;
        }

        public void flush() {
            result = null;
        }

        public void close() throws SecurityException {
            //
        }
        public LogRecord getResult(){
            return result;
        }
        
    }
       
}