/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.persistence.jdbcimpl;

import org.netbeans.mdr.persistence.*;
import java.sql.*;
import java.io.*;

/**
 * This is thrown to wrap an SQLException as a StorageException.
 *
 * @author John V. Sichi
 * @version $Id$
 *
 * @see StorageIOException
 */
public class JdbcStorageException extends StorageBadRequestException
{
    /**
     * The original SQLException that we are converting to a StorageException.
     */
    private SQLException sqlException;

    /** this constructs a JdbcStorageException from an SQLException
    * @param err the SQLException
    */
    public JdbcStorageException(SQLException err)
    {
        super(err.getMessage());
        sqlException = err;
    }

    /** return the original SQLException's message */
    public String getMessage()
    {
        return sqlException.getMessage();
    }
	
    /** return the original SQLException's localized message */
    public String getLocalizedMessage()
    {
        return sqlException.getLocalizedMessage();
    }

    /** print the original SQLException's stack trace */
    public void printStackTrace()
    {
        sqlException.printStackTrace();
    }

    /** print the original SQLException's stack trace */
    public void printStackTrace(PrintStream ps)
    {
        sqlException.printStackTrace(ps);
    }

    /** print the original SQLException's stack trace */
    public void printStackTrace(PrintWriter pw)
    {
        sqlException.printStackTrace(pw);
    }

    /** get the original SQLException's cause */
    public Throwable getCause()
    {
        return sqlException.getNextException();
    }
}

// End JdbcStorageException.java
