/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.lib.jmi.query.mof;

import java.util.Comparator;

import javax.jmi.model.*;
import javax.jmi.reflect.RefFeatured;

import org.netbeans.lib.jmi.query.*;

/** Abstract class for creating filter queries.
 *
 * @author Petr Hrebejk
 */
public class FeatureFilter extends ValueFilter {

    /** Name of the feature to filter by */
    protected String featureName;

    /** Cache for the meta class and feature */
    private javax.jmi.model.Feature feature = null;
    private javax.jmi.model.MofClass metaClazz = null;

    public FeatureFilter(Query query, String featureName, Object value) {
        super( query, value );
        this.featureName = featureName;
    }
     
    public FeatureFilter(Query query, String featureName, Object value, Comparator comparator ) {
        super( query, value, comparator );
        this.featureName = featureName;
    }
    
    protected boolean accept( Object object ) {
        
        if ( object instanceof RefFeatured ) {
            resolveFeature( (RefFeatured)object );

            if ( feature == null ) {
                return false;
            }
            Object featureValue = ((RefFeatured)object).refGetValue( (javax.jmi.model.StructuralFeature) feature );


            return super.accept( featureValue );
        }
        else {
            return false;
        }
    }
    
    
    /** Finds the right feature for filtering, caches the information 
    * to perform quicker.
    */
    private void resolveFeature( RefFeatured refFeatured ) {
        javax.jmi.model.MofClass clazz = (javax.jmi.model.MofClass)refFeatured.refMetaObject();
        
        
        
        if ( metaClazz == null || !metaClazz.equals( clazz ) || feature != null) {
            metaClazz = null;
            feature = null;
            
            try {
                metaClazz = clazz;
               
                ModelElement me = clazz.lookupElementExtended( featureName ); 
                if ( me instanceof Feature ) {
                    feature = (Feature) me;
                }
            }
            catch( NameNotFoundException e ) {
                // Let feature be null
            }
        }
    }
    
}
