/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdrshell;

import java.util.*;
import java.io.*;

import koala.dynamicjava.interpreter.*;
import koala.dynamicjava.parser.wrapper.*;

import org.netbeans.api.mdr.MDRManager;
import org.netbeans.mdr.handlers.*;
import org.netbeans.mdr.persistence.StorageException;
import org.netbeans.mdr.persistence.memoryimpl.StorageFactoryImpl;
import org.openide.ErrorManager;

/**
 *
 * @author Petr Hrebejk
 * @version
 */
public class DJava extends Object {

    /** Creates new DJava */
    public DJava() {
    }

    static koala.dynamicjava.interpreter.Interpreter interpreter;
    /**
     * Initialize the engine.
     */
    public static void initialize ( ) {
        // create an interpreter
        interpreter = new TreeInterpreter(new JavaCCParserFactory());
        BaseObjectHandler.setClassLoaderProvider(new CLProviderImpl());

        // this is here just to make MDR boot when starting up the shell (not lazily)
        MDRManager.getDefault().getDefaultRepository().getExtentNames(); 
        
        String storageName = System.getProperty("MDRStorageProperty." + StorageFactoryImpl.STORAGE_ID);
        if (storageName != null) {
            try {
                StorageFactoryImpl.serialize(storageName);
            } catch (StorageException e) {
            }
        }
    }

    public static Object eval ( String line ) throws Exception {
        try {
            Reader r = new StringReader(line + ";");
            return interpreter.interpret(r, line );
        } catch (Exception e) {
            ErrorManager.getDefault().notify(e);
            throw e;
        }
    }

    /**
     * Declare a bean
     */
    public static void declareVariable ( String name, Object object )  {
        interpreter.defineVariable( name, object );
    }

    private static class CLProviderImpl implements ClassLoaderProvider {
        public ClassLoader getClassLoader() {
            return interpreter.getClassLoader();
        }
        
        /** Implementation of this method can define a given class.
         * The defined class should be then visible from the ClassLoader returned
         * from {@link #getClassLoader} method.
         * @param className name of the class to define.
         * @param
         * @return Defined class or null (if null is return, MDR will define the
         * class in its own classloader. This class will then not be accessible from outside
         * of MDR
         */
        public Class defineClass(String className, byte[] classFile) {
            return interpreter.defineClass(className, classFile);
        }
    }
}
