/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.jmiimpl.mof.model;

import java.util.Collection;
import javax.jmi.model.AssociationEnd;
import javax.jmi.model.Classifier;
import javax.jmi.model.MultiplicityType;
import javax.jmi.model.Reference;
import javax.jmi.model.RefersTo;
import org.netbeans.mdr.handlers.AssociationHandler;
import org.netbeans.mdr.storagemodel.StorableAssociation;
import org.netbeans.mdr.util.Logger;

/**
 *
 * @author  mm109185
 */
public abstract class RefersToImpl extends AssociationHandler implements RefersTo {

    /** Creates a new instance of RefersToImpl */
    public RefersToImpl(StorableAssociation s) {
        super(s);
    }
    
    public boolean add(Reference reference, AssociationEnd associationEnd) {
        boolean result = super_add(reference, associationEnd);
        if (result) {
            boolean isChangeable = associationEnd.isChangeable();
            MultiplicityType multiplicity = associationEnd.getMultiplicity();
            
            if (reference.isChangeable() != isChangeable) {
                Logger.getDefault().log(Logger.WARNING, "Reference<->AssociationEnd inconsistency: \n    " +
                    reference.getName() + "[Reference].isChangeable = " + reference.isChangeable() +
                    "\n    " + associationEnd.getName() + "[AssociationEnd].isChangeable = " + isChangeable);
                reference.setChangeable(isChangeable);
                Logger.getDefault().log(Logger.WARNING, "Value of isChangeable attribute of the reference was updated to " + isChangeable);
            } else if (!reference.getMultiplicity().equals(multiplicity)) {
                Logger.getDefault().log(Logger.WARNING, "Reference<->AssociationEnd inconsistency: \n    " +
                    reference.getName() + "[Reference].multiplicity = " + reference.getMultiplicity() +
                    "\n    " + associationEnd.getName() + "[AssociationEnd].multiplicity = " + multiplicity);
                reference.setMultiplicity(multiplicity);
                Logger.getDefault().log(Logger.WARNING, "Value of multiplicity attribute of the reference was updated to " + multiplicity);
            }
        }
        return result;
    }
        
    protected abstract boolean super_add(Reference reference, AssociationEnd associationEnd);
}
