/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.persistence.btreeimpl.btreestorage;

import java.io.*;

import org.netbeans.mdr.persistence.*;

/**
* This represents a page fetched from the cache.
*/
public class CachedPage extends IntrusiveList.Member {

    /** Description of which page this is */
    PageID key;
    /** How many times this page has been pinned and not unpinned */
    private int pinCount;
    /** true if this page has been modified */
    boolean isDirty;
    /** true if the log file must be flushed before this page can be written */
    boolean heldForLog;

    /** the contents of the page */
    public byte contents[];

    /* cache we were created by */
    private FileCache owner;

    /** create a page of the specified size 
    * @param size the page size for the cache
    */
    CachedPage(int size) {
        key = null;
        pinCount = 0;
        isDirty = false;
        heldForLog = false;
        owner = null;
        contents = new byte[size];
    }
    
    public FileCache getOwner() {
        return owner;
    }

    /** Make this page writable.  If it was not writable previously,
    * this causes it to be logged.  This must be called before the page
    * is modified.  If the cache is not currently in a transaction,
    * this implicitly begins one.
    * @exception StorageException I/O error logging the page
    */
    public synchronized void setWritable() throws StorageException{
        owner.setWritable(this);
    }

    /** reinitialize thie object to point to a different file page 
    * @param id the file and page number this will become
    */
    void reInit(FileCache owner, PageID id) {
        this.owner = owner;
        key = id;
        pinCount = 0;
        isDirty = false;
        heldForLog = false;
    }
    
    public int pin(FileCache owner) {
        assert pinCount == 0 || this.owner == owner;
        this.owner = owner;
        return pinCount++;
    }
    
    public int getPinCount() {
        return pinCount;
    }
    
    public int innerUnpin() {
        pinCount--;
        return pinCount;
    }
        
    /** client calls this when it is done with the page */
    public void unpin() throws StorageException {
        owner.unpin(this);
    }

    /** format debugging info */
    public String toString() {
        StringBuffer debug = new StringBuffer("" + key);
        if (pinCount > 0)
            debug.append(" pin count: " + pinCount);
        if (isDirty)
            debug.append(" dirty");
        if (heldForLog)
            debug.append(" held");
        debug.append("\n");

        int j = 0;
        for (int i = 0; i < contents.length; i++, j++) {
            if (j >= 16) {
                debug.append("\n");
                j = 0;
            }
            int data = (int)(contents[i]) & 0xFF;
            debug.append(Integer.toHexString(data));
            debug.append(" ");
        }
        debug.append("\n");

        return debug.toString();
    }
                

}
