/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.util;

import java.util.HashMap;
import java.util.HashSet;
import org.netbeans.api.mdr.events.TransactionEvent;
import org.netbeans.mdr.NBMDRepositoryImpl;
import org.netbeans.mdr.storagemodel.MdrStorage;
import org.netbeans.mdr.persistence.StorageException;

/** This class implements mutex that controls storage transactions as well as
 * MDR events firing.
 *
 * @author  Martin Matula
 * @version 0.1
 */
public abstract class TransactionMutex {

    // storage
    private final MdrStorage storage;

    // events notifier
    private final EventNotifier notifier;

    // the repository (used as event source for transaction events)
    private final NBMDRepositoryImpl repository;
    
    protected TransactionMutex(Object storage, Object notifier, Object repository) {
        this.storage = (MdrStorage) storage;
        this.notifier = (EventNotifier) notifier;
        this.repository = (NBMDRepositoryImpl) repository;
    }
    
    public abstract boolean willFail();
    
    public abstract boolean pendingChanges();
 
    public abstract void enter(boolean writeAccess);
    public abstract boolean leave(boolean fail);
    
    public final void leave() {
        leave(false);
    }
    
    protected final void start() {
        TransactionEvent event = new TransactionEvent(
            repository,
            TransactionEvent.EVENT_TRANSACTION_START
        );
        notifier.REPOSITORY.firePlannedChange(storage, event);
    }
    
    protected final void end(boolean fail) {
        try {
            TransactionEvent event = new TransactionEvent(
                repository,
                TransactionEvent.EVENT_TRANSACTION_END
            );
            notifier.REPOSITORY.firePlannedChange(storage, event);
            if (fail) {
                notifier.fireCancelled();
                storage.rollback();
//                            Logger.getDefault().notify(Logger.INFORMATIONAL, new DebugException("rollback"));
            } else {
                notifier.fireChanged();
                storage.commit();
                //Logger.getDefault().log("commited");
//                            Logger.getDefault().notify(Logger.INFORMATIONAL, new DebugException("commit"));
            }
        } catch (StorageException e) {
            try { 
                Logger.getDefault().notify(Logger.INFORMATIONAL, e);
                storage.rollback();
            } catch (StorageException fatal) {
                throw new DebugException("Fatal I/O error: " + fatal.toString());
            }
        }
    }
}
