/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.examples.modules.audioloader;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import org.openide.ErrorManager;
import org.openide.filesystems.FileAttributeEvent;
import org.openide.filesystems.FileChangeListener;
import org.openide.filesystems.FileEvent;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileRenameEvent;
import org.openide.filesystems.FileUtil;
import org.openide.nodes.PropertySupport;
import org.openide.util.NbBundle;

public class ClipLengthProperty extends PropertySupport.ReadOnly implements FileChangeListener {
    private AudioDataNode n;
    private FileObject fo;
    private Float value = null;
    public ClipLengthProperty(AudioDataNode n, FileObject fo) {
        super("clipLength", Float.TYPE,
                NbBundle.getMessage(ClipLengthProperty.class, "PROP_clipLength"),
                NbBundle.getMessage(ClipLengthProperty.class, "HINT_clipLength"));
        this.n = n;
        this.fo = fo;
        fo.addFileChangeListener(FileUtil.weakFileChangeListener(this, fo));
    }
    public synchronized Object getValue() throws IllegalAccessException, InvocationTargetException {
        if (value == null) {
            try {
                // Nope, does not support mark/reset:
                //InputStream is = fo.getInputStream ();
                File f = FileUtil.toFile(fo);
                AudioInputStream ais = AudioSystem.getAudioInputStream(f);
                try {
                    value = new Float(ais.getFrameLength() / ais.getFormat().getFrameRate());
                } finally {
                    ais.close();
                }
            } catch (Exception e) {
                ErrorManager.getDefault().notify(ErrorManager.INFORMATIONAL, e);
                throw new InvocationTargetException(e);
            }
        }
        return value;
    }
    public synchronized void fileChanged(FileEvent p1) {
        value = null;
        n.fireClipLengthChange();
    }
    // Ignore others:
    public void fileDeleted(FileEvent p1) {
    }
    public void fileDataCreated(FileEvent p1) {
    }
    public void fileFolderCreated(FileEvent p1) {
    }
    public void fileRenamed(FileRenameEvent p1) {
    }
    public void fileAttributeChanged(FileAttributeEvent p1) {
    }
}
