/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.examples.modules.audioloader;
import java.io.*;
import java.util.ResourceBundle;
import javax.sound.sampled.*;
import javax.swing.JPanel;
import org.openide.*;
import org.openide.util.NbBundle;
public class RecordPanel extends JPanel implements Runnable {
    private static final AudioFormat simpleFormat = new AudioFormat (44100, 16, 2, true, true);
    private MicrophoneTemplateIterator mti;
    private boolean running;
    private TargetDataLine line;
    public RecordPanel (MicrophoneTemplateIterator mti) {
        this.mti = mti;
        running = false;
        initComponents();
        setName (bundle.getString ("RecordPanel.name"));
        recordButton.setEnabled (true);
        stopButton.setEnabled (false);
    }
    private void initComponents () {//GEN-BEGIN:initComponents
        recordButton = new javax.swing.JButton ();
        stopButton = new javax.swing.JButton ();
        setLayout (new java.awt.GridBagLayout ());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        recordButton.setText (bundle.getString ("RecordPanel.recordButton.text"));
        recordButton.addActionListener (new java.awt.event.ActionListener () {
            public void actionPerformed (java.awt.event.ActionEvent evt) {
                recordButtonActionPerformed (evt);
            }
        }
        );
        
        gridBagConstraints1 = new java.awt.GridBagConstraints ();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.SOUTH;
        add (recordButton, gridBagConstraints1);
        
        
        stopButton.setText (bundle.getString ("RecordPanel.stopButton.text"));
        stopButton.addActionListener (new java.awt.event.ActionListener () {
            public void actionPerformed (java.awt.event.ActionEvent evt) {
                stopButtonActionPerformed (evt);
            }
        }
        );
        
        gridBagConstraints1 = new java.awt.GridBagConstraints ();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.SOUTH;
        add (stopButton, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void stopButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopButtonActionPerformed
        if (! running) return;
        running = false;
        stopButton.setEnabled (false);
    }//GEN-LAST:event_stopButtonActionPerformed

    private void recordButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_recordButtonActionPerformed
        if (running) return;
        try {
            line = (TargetDataLine) AudioSystem.getLine (new DataLine.Info (TargetDataLine.class, simpleFormat));
            line.open (simpleFormat);
            line.start ();
            new Thread (this).start ();
            running = true;
            recordButton.setEnabled (false);
            stopButton.setEnabled (true);
        } catch (Exception e) {
            ErrorManager.getDefault ().notify (e);
        }
    }//GEN-LAST:event_recordButtonActionPerformed

    public void run () {
        ByteArrayOutputStream baos = new ByteArrayOutputStream ();
        byte[] buf = new byte[1024];
        int read;
        try {
            while (running && ((read = line.read (buf, 0, 1024)) > 0)) {
                baos.write (buf, 0, read);
            }
            line.stop ();
        } finally {
            line.close ();
            line = null;
            running = false;
        }
        mti.setRecorded (baos.toByteArray (), simpleFormat);
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton recordButton;
    private javax.swing.JButton stopButton;
    // End of variables declaration//GEN-END:variables

    private static final ResourceBundle bundle = NbBundle.getBundle ("org.netbeans.examples.modules.audioloader.Bundle");    

}
