/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.clock;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.Timer;
import org.openide.awt.StatusLineElementProvider;
/**
 * Implements StatusLineElementProvider to add a component to the status bar.
 * The file in META-INF/services installs this class in the default lookup,
 * so when the statusbar is constructed, it finds the instance in
 * META-INF/services, creates and instance and uses it.
 *
 * @author Timothy Boudreau
 */
public final class ClockStatusLineElement implements StatusLineElementProvider {
    private JComponent comp = new Clock();
    
    public ClockStatusLineElement() {
    }
    
    public Component getStatusLineElement() {
        return comp;
    }
    
    private static final class Clock extends JLabel implements ActionListener {
        private Timer timer = null;
        public Clock() {
            //use some text so the initial preferred size is not 0,0
            super ("I am a clock"); 
        }
        
        public void addNotify() {
            timer = new Timer (1000, this);
            timer.start();
            super.addNotify();
        }
        
        public void removeNotify() {
            super.removeNotify();
            timer.stop();
            timer = null;
        }
        
        public void actionPerformed(ActionEvent e) {
            Date d = new Date();
            String time = twoChar (d.getHours()) 
                    + ":" + twoChar (d.getMinutes()) + ":" 
                    + twoChar (d.getSeconds());
            
            setText (time);
        }
        
        private String twoChar (int i) {
            String result = Integer.toString(i);
            if (result.length() == 1) {
                result = "0" + result;
            }
            return result;
        }
        
    }
}
