/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.examples.modules.minicomposer;

import java.util.MissingResourceException;
import org.netbeans.api.convertor.ConvertorException;
import org.netbeans.spi.convertor.Convertor;
import org.openide.execution.NbProcessDescriptor;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

/**
 * Storage for {@link ExternalPlayer}s.
 * @author Jesse Glick
 */
public class ExternalPlayerConvertor implements Convertor {
    
    private static final String NS = "http://openide.netbeans.org/examples/minicomposer";
    
    public ExternalPlayerConvertor() {}
    
    public Object read(Element element) throws ConvertorException {
        String label;
        NodeList l = element.getElementsByTagNameNS(NS, "label");
        if (l.getLength() == 1) {
            Element lab = (Element)l.item(0);
            String bund = text((Element)lab.getElementsByTagNameNS(NS, "bundle").item(0));
            String key = text((Element)lab.getElementsByTagNameNS(NS, "key").item(0));
            try {
                label = NbBundle.getBundle(bund).getString(key);
            } catch (MissingResourceException e) {
                throw (ConvertorException)new ConvertorException(e.toString()).initCause(e);
            }
        } else {
            label = null;
        }
        l = element.getElementsByTagNameNS(NS, "command");
        assert l.getLength() == 1;
        String command = text((Element)l.item(0));
        l = element.getElementsByTagNameNS(NS, "arguments");
        assert l.getLength() == 1;
        l = ((Element)l.item(0)).getElementsByTagNameNS(NS, "*");
        String[] params = new String[l.getLength()];
        for (int i = 0; i < l.getLength(); i++) {
            Element arg = (Element)l.item(i);
            if (arg.getLocalName().equals("aufile")) {
                params[i] = ExternalPlayer.TAG_AUFILE;
            } else {
                assert arg.getLocalName().equals("literal");
                params[i] = text(arg);
            }
        }
        NbProcessDescriptor pc = new NbProcessDescriptor(
            command,
            Utilities.escapeParameters(params),
            NbBundle.getMessage(ExternalPlayerConvertor.class, "MSG_format_hint"));
        ExternalPlayer p = new ExternalPlayer(pc, label);
        return p;
    }
    
    public Element write(Document doc, Object inst) {
        ExternalPlayer p = (ExternalPlayer)inst;
        NbProcessDescriptor pc = p.getPlayerCommand();
        Element ep = doc.createElementNS(NS, "external-player");
        // ignore label - we never write it, it is only for registration
        {
            Element comm = doc.createElementNS(NS, "command");
            comm.appendChild(doc.createTextNode(pc.getProcessName()));
            ep.appendChild(comm);
        }
        {
            Element args = doc.createElementNS(NS, "arguments");
            String[] ss = Utilities.parseParameters(pc.getArguments());
            for (int i = 0; i < ss.length; i++) {
                if (ss[i].equals(ExternalPlayer.TAG_AUFILE)) {
                    args.appendChild(doc.createElementNS(NS, "aufile"));
                } else {
                    Element lit = doc.createElementNS(NS, "literal");
                    lit.appendChild(doc.createTextNode(ss[i]));
                    args.appendChild(lit);
                }
            }
            ep.appendChild(args);
        }
        assert ep.getNamespaceURI().equals(NS);
        return ep;
    }
    
    private static String text(Element el) {
        assert el != null;
        NodeList l = el.getChildNodes();
        assert l.getLength() == 1;
        return ((Text)l.item(0)).getNodeValue();
    }    
}
