/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.examples.modules.minicomposer;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.io.IOException;
import org.netbeans.api.registry.Context;
import org.netbeans.api.registry.ObjectRef;
import org.netbeans.spi.looks.DefaultLook;
import org.netbeans.spi.looks.Look;
import org.openide.execution.NbProcessDescriptor;
import org.openide.nodes.Node;
import org.openide.nodes.PropertySupport;
import org.openide.nodes.Sheet;
import org.openide.util.HelpCtx;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.datatransfer.ExTransferable;

/**
 * An appearance for an {@link ExternalPlayer}.
 * @author Jesse Glick
 */
public class ExternalPlayerLook extends DefaultLook  {
    
    public ExternalPlayerLook() {
        super("ExternalPlayerLook");
    }
    
    public String getDisplayName() {
        return NbBundle.getMessage(ExternalPlayerLook.class, "LBL_ExternalPlayer");
    }
        
    public String getName(Object o, Lookup l) {
        // XXX could e.g. check for it in Context.default and return path...
        return getDisplayName(o, l);
    }
    
    public String getDisplayName(Object o, Lookup l) {
        ExternalPlayer p = (ExternalPlayer) ((ObjectRef) o).getObject();
        String lab = p.getLabel();
        if (lab != null) {
            return lab;
        } else {
            NbProcessDescriptor pc = p.getPlayerCommand();
            return pc.getProcessName() + " " + pc.getArguments();
        }
    }
    
    public String getShortDescription(Object o, Lookup l) {
        return NbBundle.getMessage(ExternalPlayerLook.class, "HINT_ExternalPlayer");
    }
    
    protected String iconBase(Object o, Lookup l) {
        return "org/netbeans/examples/modules/minicomposer/resources/ExternalPlayerIcon";
    }
    
    public HelpCtx getHelpCtx(Object o, Lookup l) {
        return new HelpCtx("org.netbeans.examples.modules.minicomposer.playing");
    }
    
    public boolean isLeaf(Object o, Lookup l) {
        // XXX this should be the default
        return true;
    }
    
    public Node.PropertySet[] getPropertySets(Object o, Lookup l) {
        Sheet.Set ss = Sheet.createPropertiesSet();
        ss.put(new PlayerCommandProperty((ObjectRef)o));
        return new Node.PropertySet[] {ss};
    }
    
    public boolean canCopy(Object o, Lookup l) {
        return true;
    }
    
    public Transferable clipboardCopy(final Object o, Lookup l) {
        String mime = DataFlavor.javaJVMLocalObjectMimeType + ";class=" + ObjectRef.class.getName();
        try {
            return new ExTransferable.Single(new DataFlavor(mime)) {
                protected Object getData() {
                    return o;
                }
            };
        } catch (ClassNotFoundException e) {
            assert false : e;
            return null;
        }
    }
    
    public boolean canDestroy(Object o, Lookup l) {
        ExternalPlayer p = (ExternalPlayer) ((ObjectRef) o).getObject();
        return p != null && p.getLabel() == null;
    }
    
    public void destroy(Object o, Lookup l) throws IOException {
        Context ctx = ((ObjectRef) o).getContext();
        ctx.putObject(((ObjectRef) o).getBindingName(), null);
    }
        
    private final class PlayerCommandProperty extends PropertySupport.ReadWrite {
        
        private final ObjectRef pRef;
        
        public PlayerCommandProperty(ObjectRef pRef) {
            super("playerCommand", NbProcessDescriptor.class,
                // XXX I18N
                "Player Command",
                "What command to use to play the score");
            this.pRef = pRef;
        }
        
        public Object getValue() {
            ExternalPlayer p = (ExternalPlayer) pRef.getObject();
            return p.getPlayerCommand();
        }
        
        public void setValue(Object val) {
            ExternalPlayer p = (ExternalPlayer) pRef.getObject();            
            p.setPlayerCommand((NbProcessDescriptor) val);
            Context ctx = pRef.getContext();
            ctx.putObject(pRef.getBindingName(), pRef.getObject());
            fireChange(pRef, Look.GET_NAME | Look.GET_DISPLAY_NAME);
        }
        
    }
    
}
