/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.examples.modules.minicomposer.registryutils.backcompatible;

import org.netbeans.api.registry.Context;
import org.netbeans.api.registry.ObjectRef;
import org.netbeans.examples.modules.minicomposer.registryutils.DefaultContextLook;
import org.openide.actions.PasteAction;
import org.openide.util.Lookup;
import org.openide.util.actions.SystemAction;
import org.openide.util.datatransfer.PasteType;

import javax.swing.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

public class ObjectRefContextLook extends DefaultContextLook {
    private Class refType;

    public ObjectRefContextLook(String name, Class refType) {
        super("ObjectRefContextLook_" + name, ObjectRef.class);
        this.refType = refType;
    }

    public String getDisplayName(Object o, Lookup l) {
        // XXX I18N?
        return "Choosing " + refType.getName() + "'s";
    }

    public List getChildObjects(Object o, Lookup l) {
        List items = new ArrayList();
        Context ctx = (Context) o;
        Iterator it = ctx.getOrderedNames().iterator();
        while (it.hasNext()) {
            String name = (String) it.next();
            if (name.endsWith("/")) {
                // ignore subfolders
                continue;
            }
            Object obj = ctx.getObject(name, null);
            assert obj != null;
            if (refType.isInstance(obj)) {
                ObjectRef ref = new ObjectRef(ctx, name);
                items.add(ref);
            }
        }
        return items;
    }

    public PasteType[] getPasteTypes(Object o, Transferable t, Lookup l) {
        DataFlavor[] flavors = t.getTransferDataFlavors();
        for (int i = 0; i < flavors.length; i++) {
            DataFlavor f = flavors[i];
            if (f.isMimeTypeEqual(DataFlavor.javaJVMLocalObjectMimeType) &&
                    f.getRepresentationClass().equals(ObjectRef.class)) {
                try {
                    ObjectRef ref = (ObjectRef) t.getTransferData(f);
                    if (ref.isValid() && refType.isInstance(ref.getObject())) {
                        return new PasteType[]{new CopyObjectRefPasteType((Context) o, t, f)};
                    }
                } catch (IOException e) {
                    // ignore
                } catch (UnsupportedFlavorException ex) {
                    // ignore
                }
            }
        }
        return null;
    }

    protected Object cloneBinding(Object object) {
        return null;
    }

    public Action[] getActions(Object o, Lookup l) {
        return new Action[]{
            SystemAction.get(PasteAction.class),
        };
    }


    private class CopyObjectRefPasteType extends PasteType {

        private final Context c;
        private final Transferable t;
        private final DataFlavor f;

        public CopyObjectRefPasteType(Context c, Transferable t, DataFlavor f) {
            this.c = c;
            this.t = t;
            this.f = f;
        }

        public String getName() {
            return "Duplicate"; // XXX I18N?
        }

        private static final String SUFFIX = "-copy-";

        public Transferable paste() throws IOException {
            ObjectRef ref;
            try {
                ref = (ObjectRef) t.getTransferData(f);
            } catch (UnsupportedFlavorException e) {
                throw (IOException) new IOException(e.toString()).initCause(e);
            }
            Object o =  cloneBinding(ref.getObject());
            if (o != null) {
                Collection existingNames = c.getBindingNames();
                int x = 0;
                while (existingNames.contains(ref.getBindingName() + SUFFIX + x)) {
                    x++;
                }
                c.putObject(ref.getBindingName() + SUFFIX + x, o);
            }
            return (o != null) ? t : null;
        }
    }
}
