/*
 * PovrayLogicalView.java
 *
 * Created on June 6, 2006, 7:02 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.netbeans.examples.modules.povproject;

import java.awt.Image;
import org.netbeans.spi.project.ui.LogicalViewProvider;
import org.openide.ErrorManager;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import org.openide.util.Utilities;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/**
 *
 * @author Tim Boudreau
 */
class PovrayLogicalView implements LogicalViewProvider {
    private final PovrayProject project;

    public PovrayLogicalView(PovrayProject project) {
        this.project = project;
    }

    public Node createLogicalView() {
        try {
            //Get the scenes directory, creating if deleted
            FileObject scenes = project.getScenesFolder(true);

            //Get the DataObject that represents it
            DataFolder scenesDataObject =
                    DataFolder.findFolder (scenes);

            //Get its default node - we'll wrap our node around it to change the
            //display name, icon, etc.
            Node scenesNode = scenesDataObject.getNodeDelegate();

            //This FilterNode will be our project node
            return new ScenesNode (scenesNode, project);
        } catch (DataObjectNotFoundException donfe) {
            ErrorManager.getDefault().notify(donfe);
            //Fallback - the directory couldn't be created -
            //read-only filesystem or something evil happened
            return new AbstractNode (Children.LEAF);
        }
    }

    /** This is the node you actually see in the project tab for the project */
    private static final class ScenesNode extends FilterNode {
        final PovrayProject project;
        public ScenesNode (Node node, PovrayProject project) throws DataObjectNotFoundException {
            super (node, new FilterNode.Children (node),
                    //The projects system wants the project in the Node's lookup.
                    //NewAction and friends want the original Node's lookup.
                    //Make a merge of both
                    new ProxyLookup (new Lookup[] { Lookups.singleton(project),
                        node.getLookup() }));
            this.project = project;
        }

        public Image getIcon(int type) {
            return Utilities.loadImage (
                "org/netbeans/examples/modules/povproject/resources/scenes.gif"); //NOI18N
        }

        public Image getOpenedIcon(int type) {
            return getIcon(type);
        }

        public String getDisplayName() {
            return project.getProjectDirectory().getName();
        }
    }

    public Node findPath(org.openide.nodes.Node root, Object target) {
        //XXX NOT IMPLEMENTED
        return null;
    }
}