/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.examplemodule01;

import org.openide.ErrorManager;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;

import java.awt.BorderLayout;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import javax.swing.JButton;

/**
 *
 * Example implementation of persistent TopComponent.
 *
 * @author  Marek Slama
 *
 */
class TestComponent01 extends TopComponent {

    private static final String TC_ID = "test01tc01";

    private JButton button;

    private boolean isGUICreated = false;

    /** Used to count number of created instances in current session */
    private static int instanceCounter = 0;

    private String baseName;

    public TestComponent01 () {
        baseName = NbBundle.getMessage(TestComponent01.class, "LBL_Tab_Title01");
        instanceCounter++;
        setName(baseName + " :" + instanceCounter);
    }

    private void initGUI () {
        setLayout(new BorderLayout());
        button = new JButton("<html><big>" + getName() + "</big></html>");
        add(button);
        isGUICreated = true;
    }

    protected void componentShowing () {
        if (!isGUICreated) {
            initGUI();
        }
    }

    protected String preferredID () {
        return TC_ID;
    }
        
    /** Overriden to explicitely set persistence type of TestComponent01
     * to PERSISTENCE_ONLY_OPENED */
    public int getPersistenceType() {
        return TopComponent.PERSISTENCE_ONLY_OPENED;
    }

    public void writeExternal(ObjectOutput out) throws IOException {
        super.writeExternal(out);
        //TODO serialize your instance data here
    }

    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
        super.readExternal(in);
        //TODO deserialize your instance data here
    }
    
    public void requestFocus () {
        super.requestFocus();
        button.requestFocus();
    }
    
    public boolean requestFocusInWindow () {
        super.requestFocusInWindow();
        return button.requestFocusInWindow();
    }
}
