/**
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 **/


package org.netbeans.pluginportal.ui;

import java.util.Comparator;
import org.netbeans.pluginportal.system.*;
import pluginportal.OrderBy;

/**
 * This class is a Comparator for comparing CategoryCount types.
 * @author David Botterill
 */
public class CategoryCountComparator implements Comparator {
    
    private OrderBy orderby;
    
    /**
     * This constructor takes the ordering in the form of an OrderBy type.
     * @param inOrderBy The OrderBy used to determine how to order the CategoryCount instances.
     */
    public CategoryCountComparator(OrderBy inOrderBy) {
        orderby=inOrderBy;
    }
    /**
     * This method is used to compare o1 and o2.
     * @param o1 The first object to compare to o2
     * @param o2 The second object to compare to o1
     * @return a negative integer, zero, or a positive integer if o1 
     * is less than, equal to, or greater than o2
     */
    public int compare(Object o1, Object o2) {
        if(orderby.equals(OrderBy.NAME)) {
            String o1categoryname = ((CategoryCount)o1).getCategoryName();
            String o2categoryname = ((CategoryCount)o2).getCategoryName();
            return o1categoryname.compareToIgnoreCase(o2categoryname);
        } else {
            int o1count = ((CategoryCount)o1).getCount();
            int o2count = ((CategoryCount)o2).getCount();
            return o1count-o2count;
        }
    }
    
    
}
