/**
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 **/

package org.netbeans.pluginportal.system;

import java.io.File;
import java.io.IOException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * This class represents a NetBeans plugin (NBM) deployment descriptor or the 
 * Info.xml file.
 * @author David Botterill
 */
public class NBMDeploymentDescriptor {
    
    
    
    /**
     * Creates a new instance of NBMDeploymentDescriptor
     * @param inInfoFile - the Info.xml from the NBM file.
     */
    public NBMDeploymentDescriptor(File inInfoFile) {
        try {
            parseDeploymentDescriptor(inInfoFile);
        } catch (IOException fnfe) {
            System.err.println("NBMFile: IOException:" + fnfe);
            fnfe.printStackTrace();
        }
    }
    
    
    private void parseDeploymentDescriptor(File inInfoFile) throws IOException {
        DefaultHandler handler = new NBMParser();
        // Use the default (non-validating) parser
        SAXParserFactory factory = SAXParserFactory.newInstance();
        factory.setValidating(false);
        factory.setSchema(null);
        factory.setNamespaceAware(false);
        SAXParser saxParser;
        try {
            saxParser = factory.newSAXParser();
            
            saxParser.parse(inInfoFile, handler);
            
        } catch (SAXException se) {
            System.err.println("NBMFile: SAXException:" + se);
            se.printStackTrace();
        } catch (ParserConfigurationException pce) {
            System.err.println("NBMFile: ParserConfigurationException:" + pce);
            pce.printStackTrace();
        }
        
    }
    
    /**
     * This is the actual parser that parses the Info.xml file.
     */
    public class NBMParser extends DefaultHandler {
        
        /**
         * Default constructor
         */
        public NBMParser() {
            
        }
        
        private StringBuffer textBuffer = null;
        
        
        public void startElement(String uri, String localName, String qName, Attributes attributes)
        throws SAXException {
            if(qName.equals("module")) {
                setCodenamebase(attributes.getValue("codenamebase"));
                setHomepage(attributes.getValue("homepage"));
                setDistribution(attributes.getValue("distribution"));
                setLicense(attributes.getValue("license"));
                if(null != attributes.getValue("downloadsize")) {
                    setDownloadsize(new Long(attributes.getValue("downloadsize")));
                }
                setNeedsrestart(attributes.getValue("needsrestart"));
                setModuleauthor(attributes.getValue("moduleauthor"));
                setReleasedate(attributes.getValue("releasedate"));
            } else if(qName.equals("manifest")) {
                setOpenide_module(attributes.getValue("OpenIDE-Module"));
                openide_display_category = attributes.getValue("OpenIDE-Module-Display-Category");
                setOpenide_impl_version(attributes.getValue("OpenIDE-Module-Implementation-Version"));
                setOpenide_java_dependencies(attributes.getValue("OpenIDE-Module-Java-Dependencies"));
                setOpenide_module_dependencies(attributes.getValue("OpenIDE-Module-Module-Dependencies"));
                setOpenide_long_desc(attributes.getValue("OpenIDE-Module-Long-Description"));
                setOpenide_name(attributes.getValue("OpenIDE-Module-Name"));
                setOpenide_module_provides(attributes.getValue("OpenIDE-Module-Provides"));
                setOpenide_requires(attributes.getValue("OpenIDE-Module-Requires"));
                setOpenide_short_desc(attributes.getValue("OpenIDE-Module-Short-Description"));
                setOpenide_module_spec_version(attributes.getValue("OpenIDE-Module-Specification-Version"));
            } else if(qName.equals("license")) {
                setLicense_name(attributes.getValue("name"));
            }
            
        }
        
        public void endElement(String uri, String localName, String qName)
        throws SAXException {
            if(qName.equals("license")) {
                setLicense_text(textBuffer.toString());
            }
        }
        
        public void characters(char buf[], int offset, int len)
        throws SAXException {
            String s = new String(buf, offset, len);
            if (textBuffer == null) {
                textBuffer = new StringBuffer(s);
            } else {
                textBuffer.append(s);
            }
        }
        
    }
    
    /**
     * Holds value of property codenamebase.
     */
    private String codenamebase;
    
    /**
     * Getter for property codenamebase.
     * @return Value of property codenamebase.
     */
    public String getCodenamebase() {
        return this.codenamebase;
    }
    
    /**
     * Holds value of property homepage.
     */
    private String homepage;
    
    /**
     * Getter for property homepage.
     * @return Value of property homepage.
     */
    public String getHomepage() {
        return this.homepage;
    }
    
    /**
     * Holds value of property distribution.
     */
    private String distribution;
    
    /**
     * Getter for property distribution.
     * @return Value of property distribution.
     */
    public String getDistribution() {
        return this.distribution;
    }
    
    
    /**
     * Holds value of property license.
     */
    private String license;
    
    /**
     * Getter for property license.
     * @return Value of property license.
     */
    public String getLicense() {
        return this.license;
    }
    
    
    /**
     * Holds value of property downloadsize.
     */
    private Long downloadsize;
    
    /**
     * Getter for property downloadsize.
     * @return Value of property downloadsize.
     */
    public Long getDownloadsize() {
        return this.downloadsize;
    }
    
    /**
     * Holds value of property needsrestart.
     */
    private String needsrestart;
    
    /**
     * Getter for property needsrestart.
     * @return Value of property needsrestart.
     */
    public String getNeedsrestart() {
        return this.needsrestart;
    }
    
    /**
     * Holds value of property moduleauthor.
     */
    private String moduleauthor;
    
    /**
     * Getter for property moduleauthor.
     * @return Value of property moduleauthor.
     */
    public String getModuleauthor() {
        return this.moduleauthor;
    }
    
    /**
     * Holds value of property releasedate.
     */
    private String releasedate;
    
    /**
     * Getter for property releasedate.
     * @return Value of property releasedate.
     */
    public String getReleasedate() {
        return this.releasedate;
    }
    
    /**
     * Holds value of property openide_module.
     */
    private String openide_module;
    
    /**
     * Getter for property openide_module.
     * @return Value of property openide_module.
     */
    public String getOpenide_module() {
        return this.openide_module;
    }
    
    /**
     * Holds value of property openide_display_category.
     */
    private String openide_display_category;
    
    /**
     * Getter for property openide_display_category.
     * @return Value of property openide_display_category.
     */
    public String getOpenide_display_category() {
        return this.openide_display_category;
    }
    
    /**
     * Holds value of property openide_impl_version.
     */
    private String openide_impl_version;
    
    /**
     * Getter for property openide_impl_version.
     * @return Value of property openide_impl_version.
     */
    public String getOpenide_impl_version() {
        return this.openide_impl_version;
    }
    
    /**
     * Holds value of property openide_java_dependencies.
     */
    private String openide_java_dependencies;
    
    /**
     * Getter for property openide_java_dependencies.
     * @return Value of property openide_java_dependencies.
     */
    public String getOpenide_java_dependencies() {
        return this.openide_java_dependencies;
    }
    
    /**
     * Holds value of property openide_long_desc.
     */
    private String openide_long_desc;
    
    /**
     * Getter for property openide_long_desc.
     * @return Value of property openide_long_desc.
     */
    public String getOpenide_long_desc() {
        return this.openide_long_desc;
    }
    
    /**
     * Holds value of property openide_name.
     */
    private String openide_name;
    
    /**
     * Getter for property openide_name.
     * @return Value of property openide_name.
     */
    public String getOpenide_name() {
        return this.openide_name;
    }
    
    /**
     * Holds value of property openide_module_provides.
     */
    private String openide_module_provides;
    
    /**
     * Getter for property openide_module_provides.
     * @return Value of property openide_module_provides.
     */
    public String getOpenide_module_provides() {
        return this.openide_module_provides;
    }
    
    /**
     * Holds value of property openide_requires.
     */
    private String openide_requires;
    
    /**
     * Getter for property openide_requires.
     * @return Value of property openide_requires.
     */
    public String getOpenide_requires() {
        return this.openide_requires;
    }
    
    /**
     * Holds value of property openide_short_desc.
     */
    private String openide_short_desc;
    
    /**
     * Getter for property openide_short_desc.
     * @return Value of property openide_short_desc.
     */
    public String getOpenide_short_desc() {
        return this.openide_short_desc;
    }
    
    /**
     * Holds value of property openide_module_spec_version.
     */
    private String openide_module_spec_version;
    
    /**
     * Getter for property openide_module_spec_version.
     * @return Value of property openide_module_spec_version.
     */
    public String getOpenide_module_spec_version() {
        return this.openide_module_spec_version;
    }
    
    /**
     * Codenamebase property
     * @param codenamebase - the codename base
     */
    public void setCodenamebase(String codenamebase) {
        this.codenamebase = codenamebase;
    }
    
    public void setHomepage(String homepage) {
        this.homepage = homepage;
    }
    
    public void setDistribution(String distribution) {
        this.distribution = distribution;
    }
    
    public void setLicense(String license) {
        this.license = license;
    }
    
    public void setDownloadsize(Long downloadsize) {
        this.downloadsize = downloadsize;
    }
    
    public void setNeedsrestart(String needsrestart) {
        this.needsrestart = needsrestart;
    }
    
    public void setModuleauthor(String moduleauthor) {
        this.moduleauthor = moduleauthor;
    }
    
    public void setReleasedate(String releasedate) {
        this.releasedate = releasedate;
    }
    
    public void setOpenide_module(String openide_module) {
        this.openide_module = openide_module;
    }
    
    public void setOpenide_impl_version(String openide_impl_version) {
        this.openide_impl_version = openide_impl_version;
    }
    
    public void setOpenide_java_dependencies(String openide_java_dependencies) {
        this.openide_java_dependencies = openide_java_dependencies;
    }
    
    public void setOpenide_long_desc(String openide_long_desc) {
        this.openide_long_desc = openide_long_desc;
    }
    
    public void setOpenide_name(String openide_name) {
        this.openide_name = openide_name;
    }
    
    public void setOpenide_module_provides(String openide_module_provides) {
        this.openide_module_provides = openide_module_provides;
    }
    
    public void setOpenide_requires(String openide_requires) {
        this.openide_requires = openide_requires;
    }
    
    public void setOpenide_short_desc(String openide_short_desc) {
        this.openide_short_desc = openide_short_desc;
    }
    
    public void setOpenide_module_spec_version(String openide_module_spec_version) {
        this.openide_module_spec_version = openide_module_spec_version;
    }
    
    /**
     * Holds value of property openide_module_dependencies.
     */
    private String openide_module_dependencies;
    
    /**
     * Getter for property openide_module_dependencies.
     * @return Value of property openide_module_dependencies.
     */
    public String getOpenide_module_dependencies() {
        return this.openide_module_dependencies;
    }
    
    /**
     * Setter for property openide_module_dependencies.
     * @param openide_module_dependencies New value of property openide_module_dependencies.
     */
    public void setOpenide_module_dependencies(String openide_module_dependencies) {
        this.openide_module_dependencies = openide_module_dependencies;
    }

    /**
     * Holds value of property license_name.
     */
    private String license_name;

    /**
     * Getter for property license_name.
     * @return Value of property license_name.
     */
    public String getLicense_name() {
        return this.license_name;
    }

    /**
     * Setter for property license_name.
     * @param license_name New value of property license_name.
     */
    public void setLicense_name(String license_name) {
        this.license_name = license_name;
    }

    /**
     * Holds value of property license_text.
     */
    private String license_text;

    /**
     * Getter for property license_text.
     * @return Value of property license_text.
     */
    public String getLicense_text() {
        return this.license_text;
    }

    /**
     * Setter for property license_text.
     * @param license_text New value of property license_text.
     */
    public void setLicense_text(String license_text) {
        this.license_text = license_text;
    }
}
