/**
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 **/

package org.netbeans.pluginportal.system.impl;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Query;
import org.netbeans.pluginportal.data.OtherCategoryImpl;
import org.netbeans.pluginportal.data.PluginImpl;
import org.netbeans.pluginportal.system.NBMDeploymentDescriptor;
import org.netbeans.pluginportal.system.SystemFileUtils;

/**
 *
 * @author David Botterill
 */
public class UpdateCenterCache extends AbstractCache {
    
    
    private EntityManagerFactory emf;
    private UpdateCache cacheUpdater;
    private boolean cacheDirty;
    
    /** Creates a new instance of UpdateCenterCache */
    public UpdateCenterCache(EntityManagerFactory inEmf,File inUpdateCenterFile) {
        
        emf=inEmf;
        cacheDirty=true;
        loadList();
        cacheUpdater = new UpdateCache(emf,inUpdateCenterFile);
        /**
         * Set up a Timer to check if the cache is dirty.
         */
        Timer cacheTimer = new Timer();
        cacheTimer.scheduleAtFixedRate(cacheUpdater,0,60000);
    }
    
    protected void loadList() {
        cacheDirty=true;
    }
    
    
    class UpdateCache extends TimerTask {
        private File updateCenterFile;
        private EntityManagerFactory emf;
        
        public UpdateCache(EntityManagerFactory inEMF,File inUpdateCenterFile) {
            updateCenterFile=inUpdateCenterFile;
            emf = inEMF;
        }
        
        public void run() {
            /**
             * Only rebuild the cache if it's dirty
             */
            if(cacheDirty) {
                /**
                 * reset the cache.
                 */
                cacheDirty=false;
                /**
                 *  Clear out the license hashmap.
                 */
                getLicenses().clear();
                EntityManager em = emf.createEntityManager();
                String categeorySelect = "select o from OtherCategoryImpl o order by o.categoryname";
                
                Query categoryQuery = em.createQuery(categeorySelect);
                List<OtherCategoryImpl> categoryList = categoryQuery.getResultList();
                Iterator<OtherCategoryImpl> catIterator = categoryList.iterator();
                
                try {
                    updateCenterFile.delete();
                    FileWriter writer = new FileWriter(updateCenterFile);
                    
//            writer.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
                    writer.write("<?xml version=\"1.0\"?>\n");
                    writer.write("<!DOCTYPE module_updates PUBLIC \"-//NetBeans//DTD Autoupdate Catalog 2.3//EN\" \"http://www.netbeans.org/dtds/autoupdate-catalog-2_3.dtd\">\n");
                    Calendar date = GregorianCalendar.getInstance();
                    writer.write("<module_updates timestamp=\"" +
                            date.get(Calendar.SECOND) + "/" +
                            date.get(Calendar.MINUTE) + "/" +
                            date.get(Calendar.HOUR_OF_DAY) + "/" +
                            date.get(Calendar.DAY_OF_MONTH) + "/" +
                            (date.get((Calendar.MONTH))+1) + "/" +
                            date.get(Calendar.YEAR) + "\">\n");
                    
                    /**
                     * create module-groups based on the category name.
                     */
                    String currentCategoryGroup = "";
                    OtherCategoryImpl currentCategory = null;
                    String select = null;
                    Query pluginQuery = null;
                    PluginImpl currentPlugin = null;
                    while(null != catIterator && catIterator.hasNext()) {
                        currentCategory = catIterator.next();
                        
                        /**
                         * Is this a new category? If so create a new module group.
                         */
                        if(null != currentCategory && null != currentCategory.getCategoryname() &&
                                !currentCategory.getCategoryname().equals(currentCategoryGroup)) {
                            /**
                             * If this isn't the first category then we need to end the last module-group
                             */
                            if(!currentCategoryGroup.equals("")) {
                                writer.write("</module_group>\n");
                            }
                            
                            /**
                             * Now write out the new group.
                             */
                            writer.write("<module_group name=\"" + currentCategory.getCategoryname() + "\">\n");
                            currentCategoryGroup = currentCategory.getCategoryname();
                        }
                        
                        /**
                         * Write out the plugin info under a "module" element.
                         */
                        
                        /**
                         * Get the plugin for this entry in the other categories.
                         */
                        select = "select p from PluginImpl p where p.pluginid=" + currentCategory.getPlugin().getPluginid();
                        pluginQuery = em.createQuery(select);
                        currentPlugin = (PluginImpl)pluginQuery.getSingleResult();
                        /**
                         * If the NBM filename is blank or null skip this plugin.
                         */
                        String currentNBMFilename = currentPlugin.getNbm_filename();
                        if(null ==  currentNBMFilename || currentNBMFilename.equals("")) {
                            continue;
                        }
                        
                        /**
                         * If this NBM file is a zip file, we need to extract the files first.
                         *
                         */
                        
                        String fileType = currentNBMFilename.substring(currentNBMFilename.lastIndexOf(".")+1);
                        if(fileType.equalsIgnoreCase("ZIP")) {
                            File [] nbmFiles = null;
                            try {
                                String tmpFilename = SystemFileUtils.getNbmLocation() + File.separator + currentNBMFilename;
                                nbmFiles = SystemFileUtils.extractNBMs(tmpFilename,currentPlugin.getPluginid() + "_");
                            } catch (Exception ex) {
                                System.err.println("Error extracting NBM files from ZIP:" +
                                        currentNBMFilename + "\nException=" + ex);
                                continue;
                            }
                            for(int ii=0; null != nbmFiles && ii < nbmFiles.length; ii++) {
                                String pathName = nbmFiles[ii].getAbsolutePath();
                                String fileName = pathName.substring(pathName.lastIndexOf("/")+1);
                                writeModuleElement(writer, fileName,currentPlugin.getPluginid().toString());
                            }
                            
                        } else {
                            writeModuleElement(writer,currentNBMFilename,currentPlugin.getPluginid().toString());
                        }
                    }
                    
                    writer.write("</module_group>\n");
                    
                    /**
                     * We are done with the module-group element so write the licenses.
                     */
                    Set<String> keys = getLicenses().keySet();
                    Iterator<String> keyIterator = keys.iterator();
                    String currentKey = "";
                    while(null != keyIterator && keyIterator.hasNext()) {
                        currentKey = keyIterator.next();
                        writer.write("<license name=\"" + currentKey + "\">\n");
                        writer.write(getLicenses().get(currentKey) + "\n");
                        writer.write("</license>\n");
                    }
                    
                    /**
                     * Close the update center element
                     */
                    writer.write("</module_updates>\n");
                    writer.close();
                    
                } catch(IOException ioe) {
                    System.err.println("IOException processing update center file:" + updateCenterFile + "\nException=" + ioe);
                    ioe.printStackTrace();
                }
            }
            
        }
        
        private void writeModuleElement(FileWriter inWriter, String inNBMFileName, String inPluginid) throws IOException {
            String currentNBMFilename = SystemFileUtils.getNbmLocation() + File.separator + inNBMFileName;
            File inNBMFile = new File(currentNBMFilename);
            File infoFile = null;
            try {
                infoFile = SystemFileUtils.extractInfoFile(inNBMFile.getAbsolutePath());
            } catch (Exception ex) {
                System.err.println("Error extracting Info.xml from NBM file:" +
                        inNBMFile + "\nException=" + ex);
                if(null != infoFile) {
                    infoFile.delete();
                }
                return;
            }
            NBMDeploymentDescriptor dd = new NBMDeploymentDescriptor(infoFile);
            if(null != infoFile) {
                infoFile.delete();
            }
            if(null == dd) {
                return;
            }
            
            inWriter.write("<module codenamebase=\"" + (null == dd.getCodenamebase() ? "" : dd.getCodenamebase()) + "\"");
            inWriter.write("  distribution=\"" + SystemFileUtils.getNBMURLString() + "/" + inNBMFileName + "\"");
            inWriter.write("  downloadsize=\"" + inNBMFile.length() + "\"");
            inWriter.write("  homepage=\"" + (null == dd.getHomepage() ? "" : dd.getHomepage()) + "\"");
            /**
             * Use the pluginid to make the license unique
             */
            String tempString = null;
            inWriter.write("  license=\"" + inPluginid + "\"");
            tempString = cleanString(dd.getModuleauthor());
            inWriter.write("  moduleauthor=\"" + (null == tempString  ? "" : tempString) + "\"");
            inWriter.write("  needsrestart=\"" +
                    (null == dd.getNeedsrestart() ? "" : dd.getNeedsrestart()) + "\"");
            inWriter.write("  releasedate=\"" +
                    (null == dd.getReleasedate() ? "" : dd.getReleasedate()) + "\">");
            inWriter.write("\n<manifest ");
            if(null != dd.getOpenide_module()) {
                inWriter.write("  OpenIDE-Module=\"" + dd.getOpenide_module()  + "\"");
            }
            if(null != dd.getOpenide_display_category()) {
                inWriter.write("  OpenIDE-Module-Display-Category=\"" +
                        dd.getOpenide_display_category() + "\"");
            }
            if(null != dd.getOpenide_impl_version()) {
                inWriter.write("  OpenIDE-Module-Implementation-Version=\"" +
                        dd.getOpenide_impl_version() + "\"");
            }
            if(null != dd.getOpenide_java_dependencies()) {
                inWriter.write("  OpenIDE-Module-Java-Dependencies=\"" +
                        dd.getOpenide_java_dependencies() + "\"");
            }
            if(null != dd.getOpenide_long_desc()) {
                inWriter.write("  OpenIDE-Module-Long-Description=\"" +
                        cleanString(dd.getOpenide_long_desc()) + "\"");
            }
            if(null != dd.getOpenide_module_dependencies()) {
                inWriter.write("  OpenIDE-Module-Module-Dependencies=\"" +
                        dd.getOpenide_module_dependencies() + "\"");
            }
            if (null != dd.getOpenide_name()) {
                inWriter.write("  OpenIDE-Module-Name=\"" + dd.getOpenide_name() + "\"");
            }
            if(null != dd.getOpenide_requires()) {
                inWriter.write("  OpenIDE-Module-Requires=\"" + dd.getOpenide_requires() + "\"");
            }
            if(null != dd.getOpenide_short_desc()) {
                inWriter.write("  OpenIDE-Module-Short-Description=\"" +
                        cleanString(dd.getOpenide_short_desc())  + "\"");
            }
            if(null != dd.getOpenide_module_spec_version()) {
                inWriter.write("  OpenIDE-Module-Specification-Version=\"" +
                        dd.getOpenide_module_spec_version() + "\"/>\n");
            }
            inWriter.write("</module>\n");
            
            /**
             * Save the license text to be added to the end of the update center file.
             */
            
            getLicenses().put(inPluginid,cleanString(dd.getLicense_text()));
            
        }
        
        private String cleanString(String inString) {
            String tempString = inString;
            if(null != tempString) {
                tempString = tempString.replaceAll("\\\"","&quot;");
                tempString = tempString.replaceAll("<","&lt;");
                tempString = tempString.replaceAll(">","&gt;");
                tempString = tempString.replaceAll("&","&amp;");
                tempString = tempString.replaceAll("[[^\\p{Alnum}]&&[^\\p{Punct}]&&[^\\p{Space}]]","");
                /**
                 * Now get rid of invalid UTF8 characters.
                 */
                StringBuffer out = new StringBuffer(); // Used to hold the output.
                char current; // Used to reference the current character.
                
                for (int ii = 0; ii < tempString.length(); ii++) {
                    current = tempString.charAt(ii); // NOTE: No IndexOutOfBoundsException caught here; it should not happen.
                    if ((current == 0x9) ||
                            (current == 0xA) ||
                            (current == 0xD) ||
                            ((current >= 0x20) && (current <= 0xD7FF)) ||
                            ((current >= 0xE000) && (current <= 0xFFFD)) ||
                            ((current >= 0x10000) && (current <= 0x10FFFF))) {
                        out.append(current);
                    }
                }
                tempString = out.toString();
            }
            return tempString;
        }
        
        /**
         * Holds value of property licenses.
         */
        private HashMap<String,String> licenses = new HashMap();
        
        /**
         * Getter for property licenses.
         * @return Value of property licenses.
         */
        public HashMap<String,String> getLicenses() {
            return this.licenses;
        }
        
        /**
         * Setter for property licenses.
         * @param licenses New value of property licenses.
         */
        public void setLicenses(HashMap<String,String> licenses) {
            this.licenses = licenses;
        }
        
    }
}
