/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.iz;

// import Issue;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.net.*;
import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author  jchalupa
 */
public class Query {
    
    private static final String QUERY_URL_PREFIX = "http://www.netbeans.org/issues/buglist.cgi?";
    private static final String SHOW_BUG_URL_PREFIX = "<A HREF=\"show_bug.cgi?id=";
    public static final String ITEM_SEPARATOR = "|";
    
    private static final String ISSUE_TYPE = "issue_type";
    private static final String COMPONENT = "component";
    private static final String SUBCOMPONENT = "subcomponent";
    private static final String ISSUE_STATUS = "issue_status";
    private static final String RESOLUTION = "resolution";
    private static final String PRIORITY = "priority";
    private static final String PLATFORM = "rep_platform";
    private static final String OP_SYS = "op_sys";
    private static final String VERSION = "version";
    private static final String TARGET_MILESTONE = "target_milestone";

    private static final String EMAIL = "email";

    private static final String REGEXP = "regexp";
    private static final String NOTREGEXP = "notregexp";
    private static final String SUBSTRING = "substring";
    private static final String EXACT = "exact";
    private static final String CASESUBSTRING = "casesubstring";
    private static final String ANYWORDS = "anywords";
    private static final String ALLWORDS = "allwords";
    private static final String NOWORDS = "nowords";
    
    private static final String ISSUE_ID_TYPE = "issueidtype";
    private static final String INCLUDE = "include";
    private static final String EXCLUDE = "exclude";
    private static final String ISSUE_ID = "issue_id";
    
    private static final String CHANGEDIN = "changedin";
    private static final String VOTES = "votes";
    
    private static final String CHFIELD = "chfield";
    private static final String CHFIELDFROM = "chfieldfrom";
    private static final String CHFIELDTO = "chfieldto";
    private static final String CHFIELDVALUE = "chfieldvalue";

    private static final String ISSUE_CREATION = "issue_creation";
    private static final String ISSUE_CREATION_ENC = "creation_ts";
    
    private static final String SHORT_DESC = "short_desc";
    private static final String SHORT_DESC_TYPE = "short_desc_type";
    private static final String LONG_DESC = "long_desc";
    private static final String LONG_DESC_TYPE = "long_desc_type";
    private static final String ISSUE_FILE_LOC = "issue_file_loc";
    private static final String ISSUE_FILE_LOC_TYPE = "issue_file_loc_type";
    private static final String STATUS_WHITEBOARD = "status_whiteboard";
    private static final String STATUS_WHITEBOARD_TYPE = "status_whiteboard_type";
    private static final String KEYWORDS = "keywords";
    private static final String KEYWORDS_TYPE = "keywords_type";
    private static final String ORDER = "order";
    
    private String name;
    private Hashtable query_struct;

    private class EMailType {
        
        public static final String ASSIGNED_TO = "assigned-to";
        public static final String REPORTER = "reporter";
        public static final String QA_CONTACT = "qa-contact";
        public static final String CC = "cc";
        public static final String COMMENT = "comment";
        
        private static final String EMAIL = "email";
        private static final String EMAILTYPE = "emailtype";
        private static final String EMAIL_ASSIGNED_TO = "emailassigned_to";
        private static final String EMAIL_REPORTER = "emailreporter";
        private static final String EMAIL_QA_CONTACT = "emailqa_contact";
        private static final String EMAIL_CC = "emailcc";
        private static final String EMAIL_LONGDESC = "emaillongdesc";
        
        private static final String REGEXP = "regexp";
        private static final String NOTREGEXP = "notregexp";
        private static final String SUBSTRING = "substring";
        private static final String EXACT = "exact";
        
        private int index;
        private String text;
        private String type;
        private String who;
        
        public EMailType(int index) {
            this.index = index;
            text = "";
            type = SUBSTRING;
            if (index == 1) {
                who = ASSIGNED_TO;
            }
            else {
                who = REPORTER;
            }
        }
        
        public String getText() {
            return text;
        }
        
        public void setText(String value) {
            text = value;
        }
        
        public String getType() {
            return type;
        }
        
        public void setType(String value) {
            type = value;
        }
        
        public String getWho() {
            return who;
        }
        
        public void setWho(String value) {
            who = value;
        }
        
        public String toString() {
            
            StringBuffer sbuf = new StringBuffer();
            
            sbuf.append(EMAIL + index + "=" + getText() + "&");
            sbuf.append(EMAILTYPE + index + "=" + getType() + "&");
            
            StringTokenizer stok = new StringTokenizer(getWho(), "|");
            while (stok.hasMoreTokens()) {
                String token = stok.nextToken();
                if (token.equals(ASSIGNED_TO))
                    sbuf.append(EMAIL_ASSIGNED_TO + index + "=1&");
                if (token.equals(REPORTER))
                    sbuf.append(EMAIL_REPORTER + index + "=1&");
                if (token.equals(QA_CONTACT))
                    sbuf.append(EMAIL_QA_CONTACT + index + "=1&");
                if (token.equals(CC))
                    sbuf.append(EMAIL_CC + index + "=1&");
                if (token.equals(COMMENT))
                    sbuf.append(EMAIL_LONGDESC + index + "=1&");
            }
            
            return sbuf.toString();
        }
       
    };
    
    /** Creates a new instance of Query */
    public Query() {
        query_struct = new Hashtable();
    }
    
    public void setName(String name) {
        this.name = name;
    }
    
    public String getName() {
        return this.name;
    }

    public String getIssueType() {
        return (String)query_struct.get(ISSUE_TYPE);
    }
    
    public void setIssueType(String value) {
        query_struct.put(ISSUE_TYPE, value);
    }
    
    public String getComponent() {
        return (String)query_struct.get(COMPONENT);
    }
    
    public void setComponent(String value) {
        query_struct.put(COMPONENT, value);
    }

    public String getSubcomponent() {
        return (String)query_struct.get(SUBCOMPONENT);
    }
    
    public void setSubcomponent(String value) {
        query_struct.put(SUBCOMPONENT, value);
    }
    
    public String getIssueStatus() {
        return (String)query_struct.get(ISSUE_STATUS);
    }
    
    public void setIssueStatus(String value) {
        query_struct.put(ISSUE_STATUS, value);
    }
    
    public String getResolution() {
        return (String)query_struct.get(RESOLUTION);
    }
    
    public void setResolution(String value) {
        query_struct.put(RESOLUTION, value);
    }

    public String getPriority() {
        return (String)query_struct.get(PRIORITY);
    }
    
    public void setPriority(String value) {
        query_struct.put(PRIORITY, value);
    }

    public String getPlatform() {
        return (String)query_struct.get(PLATFORM);
    }
    
    public void setPlatform(String value) {
        query_struct.put(PLATFORM, value);
    }
    public String getOpSys() {
        return (String)query_struct.get(OP_SYS);
    }
    
    public void setOpSys(String value) {
        query_struct.put(OP_SYS, value);
    }
    
    public String getVersion() {
        return (String)query_struct.get(VERSION);
    }
    
    public void setVersion(String value) {
        query_struct.put(VERSION, value);
    }
    
    public String getTargetMilestone() {
        return (String)query_struct.get(TARGET_MILESTONE);
    }
    
    public void setTargetMilestone(String value) {
        query_struct.put(TARGET_MILESTONE, value);
    }
    
    public String getEMail1() {
        return getEMail(1);
    }
    
    public void setEMail1(String value) {
        setEMail(1, value);
    }
    
    public String getEMail1Type() {
        return getEMailType(1);
    }
    
    public void setEMail1Type(String value) {
        setEMailType(1, value);
    }

    public String getEMail1Who() {
        return getEMailWho(1);
    }
    
    public void setEMail1Who(String value) {
        setEMailWho(1, value);
    }

    public String getEMail2() {
        return getEMail(2);
    }
    
    public void setEMail2(String value) {
        setEMail(2, value);
    }

    public String getEMail2Type() {
        return getEMailType(2);
    }
    
    public void setEMail2Type(String value) {
        setEMailType(2, value);
    }

    public String getEMail2Who() {
        return getEMailWho(2);
    }
    
    public void setEMail2Who(String value) {
        setEMailWho(2, value);
    }

    private String getEMail(int index) {
        EMailType emt;
        String retval = null;
        
        if ((emt = (EMailType)query_struct.get(EMAIL + index)) != null) {
            retval = emt.getText();
        }
        return retval;
    }
    
    private void setEMail(int index, String value) {
        
        EMailType emt;
        
        if ((emt = (EMailType)query_struct.get(EMAIL + index)) == null) {
            emt = new EMailType(index);
            query_struct.put(EMAIL + index, emt);
        }
        emt.setText(value);
    }

    
    private String getEMailType(int index ) {
        EMailType emt;
        String retval = null;
        
        if ((emt = (EMailType)query_struct.get(EMAIL + index)) != null) {
            retval = emt.getType();
        }
        return retval;
    }
            
    private void setEMailType(int index, String value) {
        
        EMailType emt;
        
        if ((emt = (EMailType)query_struct.get(EMAIL + index)) == null) {
            emt = new EMailType(index);
            query_struct.put(EMAIL + index, emt);
        }
        emt.setType(value);
    }

    private String getEMailWho(int index) {
        EMailType emt;
        String retval = null;
        
        if ((emt = (EMailType)query_struct.get(EMAIL + index)) != null) {
            retval = emt.getWho();
        }
        return retval;
    }
            
    private void setEMailWho(int index, String value) {
        
        EMailType emt;
        
        if ((emt = (EMailType)query_struct.get(EMAIL + index)) == null) {
            emt = new EMailType(index);
            query_struct.put(EMAIL + index, emt);
        }
        emt.setWho(value);
    }

    public String getIssueIdType() {
        return (String)query_struct.get(ISSUE_ID_TYPE);
    }
    
    public void setIssueIdType(String value) {
        query_struct.put(ISSUE_ID_TYPE, value);
    }
    
    public String getIssueIds() {
        return (String)query_struct.get(ISSUE_ID);
    }
    
    public void setIssueIds(String value) {
        query_struct.put(ISSUE_ID, value);
    }
        
    public String getChField() {
        return (String)query_struct.get(CHFIELD);
    }
    
    public void setChField(String value) {
        if (value.equals(ISSUE_CREATION))
            query_struct.put(CHFIELD, ISSUE_CREATION_ENC);
        else
            query_struct.put(CHFIELD, value);
    }
          
    public String getChangedIn() {
        return (String)query_struct.get(CHANGEDIN);
    }
    
    public void setChangedIn(String value) {
        query_struct.put(CHANGEDIN, value);
    }
    
    public String getVotes() {
        return (String)query_struct.get(VOTES);
    }
    
    public void setVotes(String value) {
        query_struct.put(VOTES, value);
    }
    
    public String getChFieldFrom() {
        return (String)query_struct.get(CHFIELDFROM);
    }
    
    public void setChFieldFrom(String value) {
        query_struct.put(CHFIELDFROM, value);
    }
    
    public String getChFieldTo() {
        return (String)query_struct.get(CHFIELDTO);
    }
    
    public void setChFieldTo(String value) {
        query_struct.put(CHFIELDTO, value);
    }
    
    public String getChFieldValue() {
        return (String)query_struct.get(CHFIELDVALUE);
    }
    
    public void setChFieldValue(String value) {
        query_struct.put(CHFIELDVALUE, value);
    }

    public String getShortDesc() {
        return (String)query_struct.get(SHORT_DESC);
    }
    
    public void setShortDesc(String value) {
        query_struct.put(SHORT_DESC, value);
    }
    
    public String getShortDescType() {
        return (String)query_struct.get(SHORT_DESC_TYPE);
    }
    
    public void setShortDescType(String value) {
        query_struct.put(SHORT_DESC_TYPE, value);
    }
    
    public String getLongDesc() {
        return (String)query_struct.get(LONG_DESC);
    }
    
    public void setLongDesc(String value) {
        query_struct.put(LONG_DESC, value);
    }

    public String getLongDescType() {
        return (String)query_struct.get(LONG_DESC_TYPE);
    }
    
    public void setLongDescType(String value) {
        query_struct.put(LONG_DESC_TYPE, value);
    }

    public String getIssueFileLoc() {
        return (String)query_struct.get(ISSUE_FILE_LOC);
    }
    
    public void setIssueFileLoc(String value) {
        query_struct.put(ISSUE_FILE_LOC, value);
    }

    public String getIssueFileLocType() {
        return (String)query_struct.get(ISSUE_FILE_LOC_TYPE);
    }
    
    public void setIssueFileLocType(String value) {
        query_struct.put(ISSUE_FILE_LOC_TYPE, value);
    }
    
    public String getStatusWhiteboard() {
        return (String)query_struct.get(STATUS_WHITEBOARD);
    }
    
    public void setStatusWhiteboard(String value) {
        query_struct.put(STATUS_WHITEBOARD, value);
    }
    
    public String getStatusWhiteboardType() {
        return (String)query_struct.get(STATUS_WHITEBOARD_TYPE);
    }
    
    public void setStatusWhiteboardType(String value) {
        query_struct.put(STATUS_WHITEBOARD_TYPE, value);
    }

    public String getKeywords() {
        return (String)query_struct.get(KEYWORDS);
    }
    
    public void setKeywords(String value) {
        query_struct.put(KEYWORDS, value);
    }
    
    public String getKeywordsType() {
        return (String)query_struct.get(KEYWORDS_TYPE);
    }
    
    public void setKeywordsType(String value) {
        query_struct.put(KEYWORDS_TYPE, value);
    }
    
    private void appendParam(StringBuffer sb, String name, String value) {
        sb.append(name).append('=').append(value).append('&');
    }
    
    private void appendMultiParam(StringBuffer sb, String name, String value) {
        StringTokenizer stok = new StringTokenizer(value, ITEM_SEPARATOR);
        while (stok.hasMoreElements()) {
            appendParam(sb, name, stok.nextToken());
        }
    }
    
    public String toString() {
        
        String param;
        String param2;
        StringBuffer sb = new StringBuffer(QUERY_URL_PREFIX);

        if ((param = getIssueType()) != null) {
            appendMultiParam(sb, ISSUE_TYPE, param);
        }
        
        if ((param = getComponent()) != null) {
            appendMultiParam(sb, COMPONENT, param);
        }
        
        if ((param = getSubcomponent()) != null) {
            appendMultiParam(sb, SUBCOMPONENT, param);
        }
        
        if ((param = getIssueStatus()) != null) {
            appendMultiParam(sb, ISSUE_STATUS, param);
        }

        if ((param = getResolution()) != null) {
            appendMultiParam(sb, RESOLUTION, param);
        }
        
        if ((param = getPriority()) != null) {
            appendMultiParam(sb, PRIORITY, param);
        }
        
        if ((param = getPlatform()) != null) {
            appendMultiParam(sb, PLATFORM, param);
        }
        
        if ((param = getOpSys()) != null) {
            appendMultiParam(sb, OP_SYS, param);
        }
        
        if ((param = getVersion()) != null) {
            appendMultiParam(sb, VERSION, param);
        }

        if ((param = getTargetMilestone()) != null) {
            appendMultiParam(sb, TARGET_MILESTONE, param);
        }
        
        EMailType emt;
        if ((emt = (EMailType)query_struct.get(EMAIL + 1)) != null) {
            sb.append(emt.toString());
        }
        if ((emt = (EMailType)query_struct.get(EMAIL + 2)) != null) {
            sb.append(emt.toString());
        }

        if ((param = getIssueIds()) != null) {
            if ((param2 = getIssueIdType()) != null)
                appendParam(sb, ISSUE_ID_TYPE, param2);
            else
                appendParam(sb, ISSUE_ID_TYPE, INCLUDE);
            StringTokenizer stok = new StringTokenizer(param, ",");
            sb.append(ISSUE_ID + "=");
            while (stok.hasMoreTokens()) {
                sb.append(stok.nextToken() + "%2C");
            }
            sb.append('&');
        }
        
        if ((param = getChangedIn()) != null) {
            appendParam(sb, CHANGEDIN, param);
        }
        
        if ((param = getVotes()) != null) {
            appendParam(sb, VOTES, param);
        }
        
        if ((param = getChField()) != null) {
            appendParam(sb, CHFIELD, param);
        }
        if ((param = getChFieldFrom()) != null) {
            appendParam(sb, CHFIELDFROM, param);
        }
        if ((param = getChFieldTo()) != null) {
            appendParam(sb, CHFIELDTO, param);
        }
        if ((param = getChFieldValue()) != null) {
            appendParam(sb, CHFIELDVALUE, param);
        }

        if ((param = getShortDesc()) != null) {
            appendParam(sb, SHORT_DESC, param);
            if ((param2 = getShortDescType()) != null)
                appendParam(sb, SHORT_DESC_TYPE, param2);
            else
                appendParam(sb, SHORT_DESC_TYPE, SUBSTRING);
        }
               
        if ((param = getLongDesc()) != null) {
            appendParam(sb, LONG_DESC, param);
            if ((param2 = getLongDescType()) != null)
                appendParam(sb, LONG_DESC_TYPE, param2);
            else
                appendParam(sb, LONG_DESC_TYPE, SUBSTRING);
        }

        if ((param = getIssueFileLoc()) != null) {
            appendParam(sb, ISSUE_FILE_LOC, param);
            if ((param2 = getIssueFileLocType()) != null)
                appendParam(sb, ISSUE_FILE_LOC_TYPE, param2);
            else
                appendParam(sb, ISSUE_FILE_LOC_TYPE, SUBSTRING);
        }
        
        if ((param = getStatusWhiteboard()) != null) {
            appendParam(sb, STATUS_WHITEBOARD, param);
            if ((param2 = getStatusWhiteboardType()) != null)
                appendParam(sb, STATUS_WHITEBOARD_TYPE, param2);
            else
                appendParam(sb, STATUS_WHITEBOARD_TYPE, SUBSTRING);
        }

        if ((param = getKeywords()) != null) {
            StringTokenizer stok = new StringTokenizer(param, ",");
            sb.append(KEYWORDS + "=");
            while (stok.hasMoreTokens()) {
                sb.append(stok.nextToken() + "%2C");
            }
            sb.append('&');

            if ((param2 = getKeywordsType()) != null)
                appendParam(sb, KEYWORDS_TYPE, param2);
            else
                appendParam(sb, KEYWORDS_TYPE, ANYWORDS);
        }
        
        // always sort by issue number
        sb.append(ORDER + "=" + "Issue+Number");
        
        return sb.toString();
    }

    public List execute() throws IOException {
        
        List list = new ArrayList();
        
        String url_string = toString();
        System.out.println(url_string);
        URL url = new URL(url_string);
        HttpURLConnection con = (HttpURLConnection)url.openConnection();
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(con.getInputStream()));
            try {
                String s;
                while ((s = br.readLine()) != null) {
                    //System.out.println(s);
                    
                    int pos = s.indexOf(SHOW_BUG_URL_PREFIX);
                    if (pos != -1) {
                        pos += SHOW_BUG_URL_PREFIX.length();
                        s = s.substring(pos);
                        s = s.substring(0, s.indexOf('"'));
                        list.add(new Integer(s));
                    }
                }
            } finally {
                br.close();
            }
        } catch(IOException ex) {
            ex.printStackTrace();
            throw ex;
        } finally {
            con.disconnect();
        }
        
        return list;
    }
}
