/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.iz;


import java.util.*;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.*;

import org.netbeans.iz.QuerySet;

/**
 *
 * @author  jchalupa
 */
public class XMLQueryHandler extends DefaultHandler {
    
    private static final String QUERY_SET = "query-set";
    private static final String QUERY = "query";
    private static final String NAME = "name";
    private static final String ISSUE_TYPE = "issue-type";
    private static final String COMPONENT = "component";
    private static final String SUBCOMPONENT = "subcomponent";
    private static final String ISSUE_STATUS = "issue-status";
    private static final String RESOLUTION = "resolution";
    private static final String PRIORITY = "priority";
    private static final String PLATFORM = "platform";
    private static final String OP_SYS = "op-sys";
    private static final String VERSION = "version";
    private static final String TARGET_MILESTONE = "target-milestone";

    private static final String EMAIL1 = "email1";
    private static final String EMAIL2 = "email2";
    private static final String TYPE = "type";
    private static final String WHO = "who";
    
    private static final String ISSUE_ID = "issue-id";
    private static final String INCLUDE = "include";
    private static final String EXCLUDE = "exclude";

    private static final String CHANGEDIN = "changed-in";
    private static final String VOTES = "minimum-votes";
    
    private static final String CHANGEDTO = "changed-to";
    private static final String CHFIELD = "field";
    private static final String CHFROM = "from";
    private static final String CHTO = "to";
    private static final String CHVALUE = "value";
    
    private static final String SUMMARY = "summary";
    private static final String COMMENT = "comment";
    private static final String URL = "url";
    private static final String STATUS_WHITEBOARD = "status-whiteboard";
    private static final String KEYWORDS = "keywords";
    
    
    private QuerySet querySet;
    private Query query;
    
    private StringBuffer chars;

    
    public XMLQueryHandler() throws SAXException {
    }
    
    
    /**
     * Handles start of the document.     
     */
    public void startDocument() throws SAXException {
        querySet = new QuerySet();
    }
    
    /**
     * Handles start of some element.
     */
    public void startElement(String namespaceURI, String sName, String qName, Attributes attrs) throws SAXException {        
        chars = new StringBuffer();
        
        if (qName.equals(QUERY_SET)) {
            for (int i = 0; i < attrs.getLength(); i++) {
                String key = attrs.getQName(i);
                String value = attrs.getValue(i);
                if (key.equals(NAME))
                    querySet.setName(value);
                else
                    querySet.setParam(key, value);
            }
            
        }
        if (qName.equals(QUERY)) {
            query = new Query();
            for (int i = 0; i < attrs.getLength(); i++) {
                String key = attrs.getQName(i);
                String value = attrs.getValue(i);
                if (key.equals(NAME))
                    query.setName(value);
            }
        }
        if (qName.equals(EMAIL1)) {
            for (int i = 0; i < attrs.getLength(); i++) {
                String key = attrs.getQName(i);
                String value = attrs.getValue(i);
                if (key.equals(TYPE))
                    query.setEMail1Type(value);
                else if (key.equals(WHO))
                    query.setEMail1Who(value);
            }
        }
        if (qName.equals(EMAIL2)) {
            for (int i = 0; i < attrs.getLength(); i++) {
                String key = attrs.getQName(i);
                String value = attrs.getValue(i);
                if (key.equals(TYPE))
                    query.setEMail2Type(value);
                else if (key.equals(WHO))
                    query.setEMail2Who(value);
            }
        }
        if (qName.equals(ISSUE_ID)) {
            for (int i = 0; i < attrs.getLength(); i++) {
                String key = attrs.getQName(i);
                String value = attrs.getValue(i);
                if (key.equals(TYPE))
                    query.setIssueIdType(value);
            }
        }
        if (qName.equals(CHANGEDTO)) {
            for (int i = 0; i < attrs.getLength(); i++) {
                String key = attrs.getQName(i);
                String value = attrs.getValue(i);
                if (key.equals(CHFIELD))
                    query.setChField(value);
                else if(key.equals(CHFROM))
                    query.setChFieldFrom(value);
                else if(key.equals(CHTO))
                    query.setChFieldTo(value);
                // change value is supposed to be the text of the changed-to
                // tag, but it can also be passed via attribute for compatibility
                // reasons
                else if(key.equals(CHVALUE))
                    query.setChFieldValue(value);                
            }
        }
        if (qName.equals(SUMMARY)) {
            for (int i = 0; i < attrs.getLength(); i++) {
                String key = attrs.getQName(i);
                String value = attrs.getValue(i);
                if (key.equals(TYPE))
                    query.setShortDescType(value);
            }
        }
        if (qName.equals(COMMENT)) {
            for (int i = 0; i < attrs.getLength(); i++) {
                String key = attrs.getQName(i);
                String value = attrs.getValue(i);
                if (key.equals(TYPE))
                    query.setLongDescType(value);
            }
        }
        if (qName.equals(URL)) {
            for (int i = 0; i < attrs.getLength(); i++) {
                String key = attrs.getQName(i);
                String value = attrs.getValue(i);
                if (key.equals(TYPE))
                    query.setIssueFileLocType(value);
            }
        }
        if (qName.equals(STATUS_WHITEBOARD)) {
            for (int i = 0; i < attrs.getLength(); i++) {
                String key = attrs.getQName(i);
                String value = attrs.getValue(i);
                if (key.equals(TYPE))
                    query.setStatusWhiteboardType(value);
            }
        }
        if (qName.equals(KEYWORDS)) {
            for (int i = 0; i < attrs.getLength(); i++) {
                String key = attrs.getQName(i);
                String value = attrs.getValue(i);
                if (key.equals(TYPE))
                    query.setKeywordsType(value);
            }
        }
    }
    
    /**
     * Handles character data.     
     */
    public void characters(char buf[], int offset, int len) throws SAXException {        
        chars.append(buf, offset, len);
    }

    
    /**
     * Handles ends of some elements.
     */
    public void endElement(String namespaceURI, String sName, String qName) throws SAXException {        
        if (qName.equals(QUERY)) {
            querySet.addQuery(query);
            query = null;
        }
        if (qName.equals(ISSUE_TYPE)) {
            if (chars.length() > 0)
                query.setIssueType(chars.toString());
        }
        if (qName.equals(COMPONENT)) {
            if (chars.length() > 0)
                query.setComponent(chars.toString());
        }
        if (qName.equals(SUBCOMPONENT)) {
            if (chars.length() > 0 )
                query.setSubcomponent(chars.toString());
        }
        if (qName.equals(ISSUE_STATUS)) {
            if (chars.length() > 0)
                query.setIssueStatus(chars.toString());
        }
        if (qName.equals(RESOLUTION)) {
            if (chars.length() > 0)
                query.setResolution(chars.toString());
        }
        if (qName.equals(PRIORITY)) {
            if (chars.length() > 0)
                query.setPriority(chars.toString());
        }
        if (qName.equals(PLATFORM)) {
            if (chars.length() > 0)
                query.setPlatform(chars.toString());
        }
        if (qName.equals(OP_SYS)) {
            if (chars.length() > 0)
                query.setOpSys(chars.toString());
        }
        if (qName.equals(VERSION)) {
            if (chars.length() > 0)
                query.setVersion(chars.toString());
        }
        if (qName.equals(TARGET_MILESTONE)) {
            if (chars.length() > 0)
                query.setTargetMilestone(chars.toString());
        }
        if (qName.equals(EMAIL1)) {
            if (chars.length() > 0)
                query.setEMail1(chars.toString());
        }
        if (qName.equals(EMAIL2)) {
            if (chars.length() > 0)
                query.setEMail2(chars.toString());
        }
        if (qName.equals(ISSUE_ID)) {
            if (chars.length() > 0)
                query.setIssueIds(chars.toString());
        }
        if (qName.equals(CHANGEDIN)) {
            if (chars.length() > 0)
                query.setChangedIn(chars.toString());
        }
        if (qName.equals(VOTES)) {
            if (chars.length() > 0)
                query.setVotes(chars.toString());
        }
        if (qName.equals(CHANGEDTO)) {
            if (chars.length() > 0)
                query.setChFieldValue(chars.toString());
        }
        if (qName.equals(SUMMARY)) {
            if (chars.length() > 0)
                query.setShortDesc(chars.toString());
        }
        if (qName.equals(COMMENT)) {
            if (chars.length() > 0)
                query.setLongDesc(chars.toString());
        }
        if (qName.equals(URL)) {
            if (chars.length() > 0)
                query.setIssueFileLoc(chars.toString());
        }
        if (qName.equals(STATUS_WHITEBOARD)) {
            if (chars.length() > 0)
                query.setStatusWhiteboard(chars.toString());
        }
        if (qName.equals(KEYWORDS)) {
            if (chars.length() > 0)
                query.setKeywords(chars.toString());
        }
     
    }
    
    
    public QuerySet getQuerySet() {
        return querySet;
    }
}
