/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.qa.qreporter;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;

import java.util.Collection;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.TreeSet;
import java.util.Iterator;
import java.util.Vector;
import java.util.List;
import java.util.Collections;
import java.util.Map.*;

import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 *
 * @author  mk97936
 */
public class OracleReporter {
    
    private static final String user = "guest";
    private static final String password = "guest";
    private static final String driver = "oracle.jdbc.driver.OracleDriver";
    private static final String url = "jdbc:oracle:thin:@qa-w2k-s1.czech.sun.com:1521:test9";
    
    private static final int DEFECTS = 0;
    private static final int ENHANCEMENTS = 1;
    private static final int TASKS = 2;
    private static final int FEATURES = 3;
    private static final int PATCHES = 4;
    
    private static final int [] cqis = {100, 33, 10, 4, 2};
    
    private boolean listEnhancements = false;
    private boolean listTasks = false;
    private boolean listFeatures = false;
    private boolean listPatches = false;
    private boolean listAssignees = false;
    private boolean listHiprios = false;
    
    private String dateFromToPrint;
    private String dateTillToPrint;
    
    private String [] listOfKeywords;
    private String [] listOfSubcomponents;
    private String [] listOfAssignees;
    private String [] listOfComponents;
    private String [] listOfVersions;
    
    private HashMap [] issuesInfoFrom = new HashMap[5];
    private HashMap [] issuesInfoTill = new HashMap[5];
    private HashMap [] issuesInfoCreated = new HashMap[5];
   
    private Connection conn;
    private Statement stmt;
    private ResultSet rset;
    private PrintWriter output;
    private boolean isFile = false;
    
    private QueryCreator queryCreator;
    private IssueComparator issuesComparator;
    
    /** Creates a new instance of OracleReporter */
    public OracleReporter() {
        queryCreator = new QueryCreator();
        issuesComparator = new IssueComparator();
    }
    
    private void openConnection() throws SQLException {
        
        try {
            Class.forName(driver);
        } catch (Exception exc) {
            exc.printStackTrace();
        }
        
        conn = DriverManager.getConnection(url, user, password);
        stmt = conn.createStatement();
        stmt.execute("alter session set NLS_DATE_FORMAT='YYYY-mm-dd HH24:MI:ss'");
        
    }
    
    private void closeConnection() throws SQLException {
        stmt.close();
        conn.close();
    }
    
    // -------------------------------------------------------------------------
    
    private ResultSet getResultSet(String sqlCommand) throws SQLException {
        ResultSet resultSet;
        resultSet = stmt.executeQuery(sqlCommand);
        return resultSet;
    }
    
    private void getMaps(ResultSet rset, HashMap [] maps) throws SQLException {
        
        maps[DEFECTS] = new HashMap();
        maps[ENHANCEMENTS] = new HashMap();
        maps[TASKS] = new HashMap();
        maps[FEATURES] = new HashMap();
        maps[PATCHES] = new HashMap();
        
        while (rset.next()) {
            Integer issue_id = new Integer(rset.getInt(1));
            if ("DEFECT".equalsIgnoreCase(rset.getString("issue_type"))) {
                maps[DEFECTS].put(issue_id, new IssueInfo(rset.getString("issue_id"), rset.getString("priority"), 
                                               rset.getString("issue_type"),
                                               rset.getString("short_desc"), rset.getString("issue_status"), 
                                               rset.getString("component"), rset.getString("assigned_to"),
                                               rset.getString("subcomponent"), rset.getString("keyword"),
                                               rset.getString("resolution")));
            } else if ("ENHANCEMENT".equalsIgnoreCase(rset.getString("issue_type"))) {
                maps[ENHANCEMENTS].put(issue_id, new IssueInfo(rset.getString("issue_id"), rset.getString("priority"), 
                                               rset.getString("issue_type"),
                                               rset.getString("short_desc"), rset.getString("issue_status"),
                                               rset.getString("component"), rset.getString("assigned_to"),
                                               rset.getString("subcomponent"), rset.getString("keyword"),
                                               rset.getString("resolution")));
            } else if ("TASK".equalsIgnoreCase(rset.getString("issue_type"))) {
                maps[TASKS].put(issue_id, new IssueInfo(rset.getString("issue_id"), rset.getString("priority"), 
                                               rset.getString("issue_type"),
                                               rset.getString("short_desc"), rset.getString("issue_status"),
                                               rset.getString("component"), rset.getString("assigned_to"),
                                               rset.getString("subcomponent"), rset.getString("keyword"),
                                               rset.getString("resolution")));
            } else if ("FEATURE".equalsIgnoreCase(rset.getString("issue_type"))) {
                maps[FEATURES].put(issue_id, new IssueInfo(rset.getString("issue_id"), rset.getString("priority"), 
                                               rset.getString("issue_type"),
                                               rset.getString("short_desc"), rset.getString("issue_status"),
                                               rset.getString("component"), rset.getString("assigned_to"),
                                               rset.getString("subcomponent"), rset.getString("keyword"),
                                               rset.getString("resolution")));
            } else if ("PATCH".equalsIgnoreCase(rset.getString("issue_type"))) {
                maps[PATCHES].put(issue_id, new IssueInfo(rset.getString("issue_id"), rset.getString("priority"), 
                                               rset.getString("issue_type"),
                                               rset.getString("short_desc"), rset.getString("issue_status"),
                                               rset.getString("component"), rset.getString("assigned_to"),
                                               rset.getString("subcomponent"), rset.getString("keyword"),
                                               rset.getString("resolution")));
            }
        }
        
    }
    
    /**
     * Compares 2 HashMaps and returns HashMap with keys additional in mapTill
     */
    private HashMap getAdditions(HashMap mapFrom, HashMap mapTill) {
        
        HashMap addons = new HashMap();
        
        Iterator tillKeyIter = mapTill.keySet().iterator();
        while (tillKeyIter.hasNext()) {
            Integer issue_id = (Integer) tillKeyIter.next();
            if (!mapFrom.containsKey(issue_id)) {                
                addons.put(issue_id, new IssueInfo((IssueInfo) mapTill.get(issue_id)));
            }
        }
        return addons;
        
    }
    
    /**
     * Compares 2 HashMap and returns HashMap with keys missing in mapTill
     */
    private HashMap getMissing(HashMap mapFrom, HashMap mapTill) {
        
        HashMap missing = new HashMap();
        
        Iterator fromKeyIter = mapFrom.keySet().iterator();
        while (fromKeyIter.hasNext()) {
            Integer issue_id = (Integer) fromKeyIter.next();
            if (!mapTill.containsKey(issue_id)) {
                // need to get IssueInfo with all fields updated to state of Till date
                try {
                    missing.put(issue_id, getUpdatedIssue(issue_id.toString()));
                } catch (SQLException sqle) {sqle.printStackTrace(output);}
            }
        }
        return missing;
        
    }
    
    /** 
     * Returns HashMap of all Issues with field equals to at least one of values filtered from map
     * @param map
     * @param field
     * @param values
     */
    private Collection filterMapToCollection(HashMap map, int field, String [] values) {
        
        ArrayList filteredList = new ArrayList();
        Iterator iterKey = map.keySet().iterator();
        IssueInfo iss = null;
        
        while (iterKey.hasNext()) {
            Integer key = (Integer) iterKey.next();
            boolean equals = false;
            switch (field) {
                case IssueInfo.ASSIGNED:
                    iss = (IssueInfo) map.get(key);
                    for (int i = 0; i < values.length; i++) {
                        if (values[i].equals(iss.assigned_to)) {
                            equals = true;
                        }
                    }
                    if (equals) filteredList.add(new IssueInfo(iss));
                    break;
                case IssueInfo.COMPONENT:
                    iss = (IssueInfo) map.get(key);
                    for (int i = 0; i < values.length; i++) {
                        if (values[i].equals(iss.component)) {
                            equals = true;
                        }
                    }
                    if (equals) filteredList.add(new IssueInfo(iss));
                    break;
                case IssueInfo.DESCRIPTION:
                    iss = (IssueInfo) map.get(key);
                    for (int i = 0; i < values.length; i++) {
                        if (values[i].equals(iss.description)) {
                            equals = true;
                        }
                    }
                    if (equals) filteredList.add(new IssueInfo(iss));
                    break;
                case IssueInfo.KEYWORD:
                    iss = (IssueInfo) map.get(key);
                    for (int i = 0; i < values.length; i++) {
                        if (values[i].equals(iss.keyword)) {
                            equals = true;
                        }
                    }
                    if (equals) filteredList.add(new IssueInfo(iss));
                    break;
                case IssueInfo.PRIORITY:
                    iss = (IssueInfo) map.get(key);
                    for (int i = 0; i < values.length; i++) {
                        if (values[i].equals(iss.priority)) {
                            equals = true;
                        }
                    }
                    if (equals) filteredList.add(new IssueInfo(iss));
                    break;
                case IssueInfo.RESOLUTION:
                    iss = (IssueInfo) map.get(key);
                    for (int i = 0; i < values.length; i++) {
                        if (values[i].equals(iss.resolution)) {
                            equals = true;
                        }
                    }
                    if (equals) filteredList.add(new IssueInfo(iss));
                    break;
                case IssueInfo.STATUS:
                    iss = (IssueInfo) map.get(key);
                    for (int i = 0; i < values.length; i++) {
                        if (values[i].equals(iss.status)) {
                            equals = true;
                        }
                    }
                    if (equals) filteredList.add(new IssueInfo(iss));
                    break;
                case IssueInfo.SUBCOMPONENT:
                    iss = (IssueInfo) map.get(key);
                    for (int i = 0; i < values.length; i++) {
                        if (values[i].equals(iss.subcomponent)) {
                            equals = true;
                        }
                    }
                    if (equals) filteredList.add(new IssueInfo(iss));
                    break;
                case IssueInfo.TYPE:
                    iss = (IssueInfo) map.get(key);
                    for (int i = 0; i < values.length; i++) {
                        if (values[i].equals(iss.type)) {
                            equals = true;
                        }
                    }
                    if (equals) filteredList.add(new IssueInfo(iss));
                    break;
            }
        }
        
        return filteredList;
        
    }
    /**
     * Returns all values which are contained by assigned_to field
     */
    public String [] getValuesOfAssignedTo(HashMap map, boolean notassigned) {
        
        TreeSet set = new TreeSet();
        Iterator iterKey = map.keySet().iterator();
        IssueInfo iss = null;
        
        while (iterKey.hasNext()) {
            Integer key = (Integer) iterKey.next();
            String assigned = ((IssueInfo) map.get(key)).assigned_to;
            if (!set.contains(assigned)) {
                if (notassigned && (assigned.indexOf("issues@") != -1)) {
                    set.add(assigned);
                } else if (!notassigned && (assigned.indexOf("issues@") == -1)) {
                    set.add(assigned);
                }
            }
        }
        
        return ((String []) set.toArray(new String [] {}));
        
    }
    
    // -------------------------------------------------------------------------
    
    /**
     * Creates correct date Strings to pass to QueryCreator from servlet or ant task
     */
    public void setDates(String reqDateFrom, String reqDateTill) {
        
        String dTill;
        String dFrom;
        Date dTillDate = null;
        Date dTillDateCount = null;
        Date dFromDate = null;
        long aweek = 604800000L;
        long nineHours = 32400000L;
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf_time = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        
        // timezone shift is subtracted from the time
        if ("now".equals(reqDateTill.trim())) {
            dTillDate = new Date();
            dateTillToPrint = sdf_time.format(dTillDate);
            dTillDateCount = new Date(dTillDate.getTime()); // will be used to count From date
            dTillDate.setTime(dTillDate.getTime() - nineHours);
            dTill = sdf_time.format(dTillDate);
        } else {
            try {
                dTillDate = sdf_time.parse(reqDateTill.trim());
                dateTillToPrint = sdf_time.format(dTillDate);
                dTillDateCount = new Date(dTillDate.getTime()); // will be used to count From date
                dTillDate.setTime(dTillDate.getTime() - nineHours);
                dTill = sdf_time.format(dTillDate);
            } catch (ParseException pe) {
                try {
                    Date dTillDatePre = sdf.parse(reqDateTill.trim());
                    dTillDate = sdf_time.parse(reqDateTill + " 23:59:59");
                    dateTillToPrint = sdf_time.format(dTillDate);
                    dTillDateCount = new Date(dTillDate.getTime()); // will be used to count From date
                    dTillDate.setTime(dTillDate.getTime() - nineHours);
                    dTill = sdf_time.format(dTillDate);
                } catch (ParseException pe2) {
                    output.println("Wrong date format!");
                    return;
                }
            }
        }
        
        if ("week ago".equals(reqDateFrom.trim())) {
            dFromDate = new Date(dTillDateCount.getTime() - aweek); 
            dateFromToPrint = sdf_time.format(dFromDate);
            dFromDate.setTime(dFromDate.getTime() - nineHours);
            dFrom = sdf_time.format(dFromDate);
        } else {
            try {
                dFromDate = sdf_time.parse(reqDateFrom.trim());
                dateFromToPrint = sdf_time.format(dFromDate);
                dFromDate.setTime(dFromDate.getTime() - nineHours);
                dFrom = sdf_time.format(dFromDate);
            } catch (ParseException pe) {
                try {
                    Date dFromDatePre = sdf.parse(reqDateFrom.trim());
                    dFromDate = sdf_time.parse(reqDateFrom + " 23:59:59");
                    dateFromToPrint = sdf_time.format(dFromDate);
                    dFromDate.setTime(dFromDate.getTime() - nineHours);
                    dFrom = sdf_time.format(dFromDate);
                } catch (ParseException pe2) {
                    output.println("Wrong date format!");
                    return;
                }
            }
        }
        
        if (dFromDate.after(dTillDate)) {
            output.println("First date must be before second date!");
            return;
        }        
        
        // debug output
        //output.println("Date from: " + dFrom + ", to print: " + dateFromToPrint + "<br>");
        //output.println("Date till: " + dTill + ", to print: " + dateTillToPrint + "<br>");
        
        queryCreator.setDates(dFrom, dTill);
        
    }
    
    /**
     * Passes selected versions to QueryCreator from servlet or ant task
     */
    public void setVersions(String [] vers) {
        listOfVersions = vers;
        queryCreator.setVersions(vers);
    }
    
    /**
     * Passes selected components (modules) to QueryCreator from servlet or ant task
     */
    public void setComponents(String [] comps) {
        listOfComponents = comps;
        queryCreator.setComponents(comps);
    }
    
    // ------------------------------------------------------------------------- 
    
    public void setSubcomponents(String [] subcomps) {
        listOfSubcomponents = subcomps;
    }
    
    public void setKeywords(String [] keyws) {
        listOfKeywords = keyws;
    }
    
    public void listEnhancements(boolean b) {
        listEnhancements = b;
    }
    
    public void listTasks(boolean b) {
        listTasks = b;
    }
    
    public void listFeatures(boolean b) {
        listFeatures = b;
    }
    
    public void listPatches(boolean b) {
        listPatches = b;
    }
    
    public void listAssignees(boolean b) {
        listAssignees = b;
    }
    
    public void listHiprios(boolean b) {
        listHiprios = b;
    }
    
    public void setOutput(PrintWriter out, boolean isf) {
        output = out;
        isFile = isf;
    }
    
    // -------------------------------------------------------------------------
    
    public void getReport() {
        
        try {
            
            // debug output
            //System.out.println("<p>" + queryCreator.createQuery(QueryCreator.QTYPE_FROM));
            //System.out.println("<p>" + queryCreator.createQuery(QueryCreator.QTYPE_TILL));
            //System.out.println("<p>" + queryCreator.createQuery(QueryCreator.QTYPE_CREATED));
            
            openConnection();
            
            ResultSet rsFrom = getResultSet(queryCreator.createQuery(QueryCreator.QTYPE_FROM)); // query 1
            getMaps(rsFrom, issuesInfoFrom);
            
            ResultSet rsTill = getResultSet(queryCreator.createQuery(QueryCreator.QTYPE_TILL)); // query 2
            getMaps(rsTill, issuesInfoTill);

            ResultSet rsCreated = getResultSet(queryCreator.createQuery(QueryCreator.QTYPE_CREATED)); // query 3
            getMaps(rsCreated, issuesInfoCreated);
            
            // select non evaluated bugs and create an List of them
            ResultSet rsNonEval = getResultSet(queryCreator.getNonEvaluatedQuery());
            ArrayList nonEvalList = new ArrayList();
            while (rsNonEval.next()) {
                nonEvalList.add(new IssueInfo(rsNonEval.getString("issue_id"), rsNonEval.getString("priority"), 
                                              rsNonEval.getString("issue_type"), rsNonEval.getString("short_desc"), 
                                              rsNonEval.getString("issue_status"), rsNonEval.getString("component"), 
                                              rsNonEval.getString("assigned_to"), rsNonEval.getString("subcomponent"), 
                                              rsNonEval.getString("keyword"), rsNonEval.getString("resolution")));
            }
            
            HashMap addons = getAdditions(issuesInfoFrom[DEFECTS], issuesInfoTill[DEFECTS]);
            HashMap missing = getMissing(issuesInfoFrom[DEFECTS], issuesInfoTill[DEFECTS]);
            
            HashMap incomming = new HashMap(); // to be printed
            incomming.putAll(addons);
            incomming.keySet().removeAll(issuesInfoCreated[DEFECTS].keySet());
            
            HashMap newIssues = new HashMap(); // to be printed
            newIssues.putAll(addons);
            newIssues.putAll(issuesInfoCreated[DEFECTS]);
            newIssues.keySet().removeAll(incomming.keySet());
            
            HashMap createdResolvedAll = new HashMap();
            createdResolvedAll.putAll(issuesInfoCreated[DEFECTS]);
            createdResolvedAll.keySet().removeAll(addons.keySet());
            createdResolvedAll.putAll(missing);
                        
            printHeader(output);
            int section = 1;
            
            // Bug counts table ------------------------------------------------
            output.println("<h3>" + section++ + ". Bug counts</h3>");
            printIssueCounts(output, issuesInfoFrom[DEFECTS].values(), issuesInfoTill[DEFECTS].values(), listOfComponents);
            output.println("<p>\n<hr align=\"left\" width=\"90%\">");
            
            // new bugs --------------------------------------------------------
            output.println("<h3>" + section++ + ". New bugs (" + newIssues.values().size() + ")</h3>");
            ArrayList nb = new ArrayList(newIssues.values());
            Collections.sort(nb, issuesComparator);
            printCollection(output, nb, false);
            
            if (incomming.values().size() > 0) {
                output.println("<p>");
                output.println("<b>Incomming bugs (" + incomming.values().size() + ")</b> <i><font size=\"-1\"> "+
                               "... changed component or issue type</font></i>");
                ArrayList ib = new ArrayList(incomming.values());
                Collections.sort(ib, issuesComparator);
                printCollection(output, ib, false);
            }
            output.println("<hr align=\"left\" width=\"90%\">");
            
            // non-evaluated bugs ----------------------------------------------
            if (nonEvalList.size() > 0) {
                Collections.sort(nonEvalList, issuesComparator);
                output.println("<h3>" + section++ + ". Non evaluated (" + nonEvalList.size() + ")</h3>");
                printCollection(output, nonEvalList, false);
                output.println("<hr align=\"left\" width=\"90%\">");
            }
            
            // resolved bugs ---------------------------------------------------
            ArrayList rb = new ArrayList(createdResolvedAll.values());
            Collections.sort(rb, issuesComparator);
            ArrayList outgoing = removeOutgoing("DEFECT", listOfComponents, rb);
            output.println("<h3>" + section++ + ". Resolved bugs (" + rb.size() + ")</h3>");
            printCollection(output, rb, true);
            if (outgoing.size() > 0) {
                output.println("<p>");
                output.println("<b>Outgoing (" + outgoing.size() + ")</b> <i><font size=\"-1\"> "+
                                       "... changed component or issue type</font></i>");
                printCollection(output, outgoing, false);
            }
            output.println("<hr align=\"left\" width=\"90%\">");
            
            // tasks, enhancements, features, patches --------------------------
            int [] issTypes = { ENHANCEMENTS, TASKS, FEATURES, PATCHES };
            boolean [] showIssTypes = { listEnhancements, listTasks, listFeatures, listPatches };
            String [] sectionNames = { "Enhancements", "Tasks", "Features", "Patches" };
            String [] issTypeNames = { "ENHANCEMENT", "TASK", "FEATURE", "PATCH" };
            for (int k = 0; k < issTypes.length; k++) {
                if (showIssTypes[k]) {
                    
                    HashMap additionalIss = getAdditions(issuesInfoFrom[issTypes[k]], issuesInfoTill[issTypes[k]]);
                    HashMap missingIss = getMissing(issuesInfoFrom[issTypes[k]], issuesInfoTill[issTypes[k]]);
                    
                    HashMap incommingIss = new HashMap(); // to be printed
                    incommingIss.putAll(additionalIss);
                    incommingIss.keySet().removeAll(issuesInfoCreated[issTypes[k]].keySet());
                    
                    HashMap newIss = new HashMap(); // to be printed
                    newIss.putAll(additionalIss);
                    newIss.putAll(issuesInfoCreated[issTypes[k]]);
                    newIss.keySet().removeAll(incommingIss.keySet());
                    
                    output.println("<h3>" + section++ + ". " + sectionNames[k] + "</h3>");
                    if (newIss.size() > 0) {
                        output.println("<b>New (" + newIss.size() + ")</b>");
                        ArrayList issList = new ArrayList(newIss.values());
                        Collections.sort(issList, issuesComparator);
                        printCollection(output, issList, false);
                    }
                    
                    if (incommingIss.size() > 0) {
                        output.println("<p>");
                        output.println("<b>Incomming (" + incommingIss.size() + ")</b> <i><font size=\"-1\"> "+
                                       "... changed component or issue type</font></i>");
                        ArrayList issList = new ArrayList(incommingIss.values());
                        Collections.sort(issList, issuesComparator);
                        printCollection(output, issList, false);
                    }
                    
                    // Resolved issues section
                    HashMap resolvedIss = new HashMap();
                    resolvedIss.putAll(issuesInfoCreated[issTypes[k]]);
                    resolvedIss.keySet().removeAll(additionalIss.keySet());
                    resolvedIss.putAll(missingIss);
                    
                    if (resolvedIss.size() > 0) {
                        ArrayList issList = new ArrayList(resolvedIss.values());
                        Collections.sort(issList, issuesComparator);
                        ArrayList outIssList = removeOutgoing(issTypeNames[k], listOfComponents, issList);
                        if (issList.size() > 0) {
                            output.println("<p>");
                            output.println("<b>Resolved (" + issList.size() + ")</b>");
                            printCollection(output, issList, true);
                        }
                        if (outIssList.size() > 0) {
                            output.println("<p>");
                            output.println("<b>Outgoing (" + outIssList.size() + ")</b> <i><font size=\"-1\"> "+
                                       "... changed component or issue type</font></i>");
                            printCollection(output, outIssList, false);
                        }
                    }
                    
                    output.println("<hr align=\"left\" width=\"90%\">");
                    
                }
                
            }
            
            // hi prios section ------------------------------------------------
            if (listHiprios) {
                ArrayList list = (ArrayList) filterMapToCollection(issuesInfoTill[DEFECTS], IssueInfo.PRIORITY, new String [] {"P1", "P2"});
                Collections.sort(list, issuesComparator);
                output.println("<h3>" + section++ + ". Hi Priority bugs (" + list.size() + ")</h3>");
                printCollection(output, list, false);
                output.println("<hr align=\"left\" width=\"90%\">");
            }
            
            // subcomponents section -------------------------------------------
            if ((listOfSubcomponents != null) && (listOfSubcomponents.length > 0)) {
                for (int j = 0; j < listOfSubcomponents.length; j++) {
                    ArrayList list = (ArrayList) filterMapToCollection(issuesInfoTill[DEFECTS], IssueInfo.SUBCOMPONENT, new String [] {listOfSubcomponents[j]}); 
                    output.println("<h3>" + section++ + ". Bugs assigned to subcomponent - " + listOfSubcomponents[j] + " (" + list.size() + ")</h3>");
                    if (list.size() > 0) {
                        Collections.sort(list, issuesComparator);
                        printCollection(output, list, false);
                    } else {
                        output.println("<i>There are no bugs assigned to subcomponent " + listOfSubcomponents[j] + "</i>");
                    }
                    output.println("<hr align=\"left\" width=\"90%\">");
                }
            }
            
            // keywords sections -----------------------------------------------
            if ((listOfKeywords != null) && (listOfKeywords.length > 0)) {
                for (int i = 0; i < listOfKeywords.length; i++) {                    
                    ArrayList list = (ArrayList) filterMapToCollection(issuesInfoTill[DEFECTS], IssueInfo.KEYWORD, new String [] {listOfKeywords[i]}); 
                    output.println("<h3>" + section++ + ". Bugs with assigned keyword - " + listOfKeywords[i] + " (" + list.size() + ")</h3>");
                    if (list.size() > 0) {
                        Collections.sort(list, issuesComparator);
                        printCollection(output, list, false);
                    } else {
                        output.println("<i>There are no bugs with keyword " + listOfKeywords[i] + "</i>");
                    }
                    output.println("<hr align=\"left\" width=\"90%\">");
                }
            }
            
            // assignees section -----------------------------------------------
            if (listAssignees) {
                String [] assignees = getValuesOfAssignedTo(issuesInfoTill[DEFECTS], false);
                output.println("<h3>" + section++ + ". Bugs by assignee</h3>");
                for (int i = 0; i < assignees.length; i++) {
                    ArrayList list = (ArrayList) filterMapToCollection(issuesInfoTill[DEFECTS], IssueInfo.ASSIGNED, new String [] {assignees[i]}); 
                    output.println("<b>Bugs assigned to " + assignees[i] + " (" + list.size() + ")</b>");
                    Collections.sort(list, issuesComparator);
                    printCollection(output, list, false);
                    output.println("<p>");
                }
                String [] notAssignees = getValuesOfAssignedTo(issuesInfoTill[DEFECTS], true);
                if (notAssignees.length > 0) {
                    ArrayList list = (ArrayList) filterMapToCollection(issuesInfoTill[DEFECTS], IssueInfo.ASSIGNED, notAssignees); 
                    output.println("<b>Not assigned bugs (" + list.size() + ")</b>");
                    Collections.sort(list, issuesComparator);
                    printCollection(output, list, false);
                }
                output.println("<hr align=\"left\" width=\"90%\">");
            }

            printFooter(output);
            
            rsFrom.close();
            rsTill.close();
            rsCreated.close();
            closeConnection();

            
        } catch (SQLException sqle) {
            sqle.printStackTrace();
        }
        
    }
    
    // -------------------------------------------------------------------------
    
    private void printHeader(PrintWriter out) {
        out.println("<html>");
        out.println("<head>");
        out.println("<meta http-equiv=\"Expires\" content=\"0\">");
        if (!isFile) {
            out.println("<link rel=\"stylesheet\" href=\"netbeans.css\" type=\"text/css\">");
        } else {
            out.println("<link rel=\"stylesheet\" href=\"http://www.netbeans.org/netbeans.css\" type=\"text/css\">");
        }
        out.println("<title>Bug summary</title>");
        out.println("</head>");
        out.println("<body>");
        out.println("<hr>");
        out.print("<h1 align=\"center\">Bug summary for modules: ");
        for (int i = 0; i < listOfComponents.length; i++) {
            out.print(listOfComponents[i]);
            if (i < listOfComponents.length - 1) out.print(", ");
        }
        out.println("</h1>");
        
        if (listOfVersions != null) {
            out.print("<h2 align=\"center\">Versions: ");
            for (int j = 0; j < listOfVersions.length; j++) {
                out.print(listOfVersions[j]);
                if (j < listOfVersions.length - 1) out.print(", ");
            }
            out.println("</h2>");
        }
        out.println("<h2 align=\"center\">" + dateFromToPrint + "&nbsp;&nbsp;--&nbsp;&nbsp;" + dateTillToPrint + "</h2>");
        out.println("<hr>");
    }
    
    private void printFooter(PrintWriter out) {
        out.println("</body>");
        out.println("</html>");
    }

    // -------------------------------------------------------------------------
    
    private void printIssueCounts(PrintWriter out, Collection listFrom, Collection listTill, String mods[]) {
        
        String [] prios = {"P1", "P2", "P3", "P4", "P5"};
        int issueNumbersFrom [][] = new int [mods.length][prios.length];
        int issueNumbersTill [][] = new int [mods.length][prios.length];
        
        // creates 2 two-dimensional arrays of all bug counts from - till
        for (int i = 0; i < mods.length; i++) { // through all modules
            for (int j = 0; j < prios.length; j++) { // through all priorities
                
                int countFrom = 0;
                int countTill = 0;
                
                Iterator fromIter = listFrom.iterator();
                while (fromIter.hasNext()) {
                    IssueInfo issue = (IssueInfo) fromIter.next();
                    if (prios[j].equals(issue.priority) && // bugs with given prio
                        mods[i].equals(issue.component)) { // on given module
                        countFrom++;
                    }
                }
                issueNumbersFrom[i][j] = countFrom;
                
                Iterator tillIter = listTill.iterator();
                while (tillIter.hasNext()) {
                    IssueInfo issue = (IssueInfo) tillIter.next();
                    if (prios[j].equals(issue.priority) && // bugs with given prio
                        mods[i].equals(issue.component)) { // on given module
                        countTill++;
                    }
                }
                issueNumbersTill[i][j] = countTill;
                
            }
            
        }
        
        // table header
        out.println("<table width=\"85%\" border=\"1\" cellspacing=\"0\" cellpadding=\"3\">");
        // first row 
        out.println("<tr>\n    <td width=\"23%\">Module/Priority</td><td width=\"11%\" align=\"center\"><b>P1</b></td><td align=\"center\" width=\"11%\"><b>P2</b></td>" +
                    "<td align=\"center\" width=\"11%\"><b>P3</b></td><td align=\"center\" width=\"11%\"><b>P4</b></td><td align=\"center\" width=\"11%\"><b>P5</b></td>" +
                    "<td align=\"center\" width=\"11%\"><b>All</b></td><td align=\"center\" width=\"11%\"><b>CQI</b></td>\n</tr>");
        
        int allAllCQI = 0;
        int allAllCQIPrev = 0;
        int allAllIssues = 0;
        int allAllIssuesPrev = 0;
        int [] allPrios = new int [prios.length];
        int [] allPriosPrev = new int [prios.length];
        
        for (int k = 0; k < mods.length; k++) {
            
            int allModIssues = 0;
            int allModIssuesPrev = 0;
            int allModCQI = 0;
            int allModCQIPrev = 0;
            
            // count all priorities together
            for (int u = 0; u < prios.length; u++) {
                allPrios[u] += issueNumbersTill[k][u];
                allPriosPrev[u] += issueNumbersFrom[k][u];
            }
                        
            out.println("<tr>");
            // name of module
            out.print("    <td><b>" + mods[k] + "</b></td>");
            
            for (int m = 0; m < prios.length; m++) {
                
                allModIssues += issueNumbersTill[k][m];
                allModCQI += (cqis[m] * issueNumbersTill[k][m]);
                allModIssuesPrev += issueNumbersFrom[k][m];
                allModCQIPrev += (cqis[m] * issueNumbersFrom[k][m]);
                
                out.print("<td align=\"center\">");
                // number of bugs for given prio
                out.print(issueNumbersTill[k][m]);
                if (issueNumbersTill[k][m] > issueNumbersFrom[k][m]) {
                    // more
                    out.print(" <font color=\"Red\">(+" + (issueNumbersTill[k][m] - issueNumbersFrom[k][m]) + ")</font>");
                } else if (issueNumbersTill[k][m] < issueNumbersFrom[k][m]) {
                    // less
                    out.print(" <font color=\"Blue\">(-" + (issueNumbersFrom[k][m] - issueNumbersTill[k][m]) + ")</font>");
                }
                
                out.println("</td>");
            }
            
            // summary of all bugs for one module
            out.print("<td align=\"center\">");
            out.print(allModIssues);
            if (allModIssues > allModIssuesPrev) {
                out.print(" <font color=\"Red\">(+" + (allModIssues - allModIssuesPrev) + ")</font>");
            } else if (allModIssues < allModIssuesPrev) {
                out.print(" <font color=\"Blue\">(-" + (allModIssuesPrev - allModIssues) + ")</font>");
            }
            out.println("</td>");
            
            // summary of all CQIs for one module
            out.print("<td align=\"center\">");
            out.print(allModCQI);
            if (allModCQI > allModCQIPrev) {
                out.print(" <font color=\"Red\">(+" + (allModCQI - allModCQIPrev) + ")</font>");
            } else if (allModCQI < allModCQIPrev) {
                out.print(" <font color=\"Blue\">(-" + (allModCQIPrev - allModCQI) + ")</font>");
            }
            out.println("</td>");
            
            out.println("</tr>");
            
            allAllCQI += allModCQI;
            allAllCQIPrev += allModCQIPrev;
            allAllIssues += allModIssues;
            allAllIssuesPrev += allModIssuesPrev;
                        
        }

        // last row of table with summaries across priorities
        out.println("<tr>");
        out.println("    <td><b>all modules</b></td>");
        for (int t = 0; t < prios.length; t++) {    
            out.print("<td align=\"center\">");
            out.print(allPrios[t]);
            if (allPrios[t] > allPriosPrev[t]) {
                out.print(" <font color=\"Red\">(+" + (allPrios[t] - allPriosPrev[t]) + ")</font>");
            } else if (allPrios[t] < allPriosPrev[t]) {
                out.print(" <font color=\"Blue\">(-" + (allPriosPrev[t] - allPrios[t]) + ")</font>");
            }
            out.print("</td>");
        }

        // cell with number of ALL issues of ALL modules
        out.print("<td align=\"center\">");
        out.print(allAllIssues);
        if (allAllIssues > allAllIssuesPrev) {
            out.print(" <font color=\"Red\">(+" + (allAllIssues - allAllIssuesPrev) + ")</font>");
        } else if (allAllIssues < allAllIssuesPrev) {
            out.print(" <font color=\"Blue\">(-" + (allAllIssuesPrev - allAllIssues) + ")</font>");
        }
        out.print("</td>");
        
        // cell with CQI of ALL issues of ALL modules
        out.print("<td align=\"center\">");
        out.print(allAllCQI);
        if (allAllCQI > allAllCQIPrev) {
            out.print(" <font color=\"Red\">(+" + (allAllCQI - allAllCQIPrev) + ")</font>");
        } else if (allAllCQI < allAllCQIPrev) {
            out.print(" <font color=\"Blue\">(-" + (allAllCQIPrev - allAllCQI) + ")</font>");
        }
        out.println("</td>");

        out.println("</tr>");
        out.println("</table>");
        
    }
    
    /**
     * Prints table with all items in collection, if resolution is true table
     * contains also resolution column
     */
    private void printCollection(PrintWriter pw, Collection coll, boolean resolution) {
        
        pw.println("<table cellpadding=\"2\">");
        Iterator iterList = coll.iterator();
        while (iterList.hasNext()) {
            IssueInfo issue = (IssueInfo) iterList.next();
            pw.println("<tr>");
            pw.print("    <td><a href=\"http://www.netbeans.org/issues/show_bug.cgi?id=" + issue.issue_id + "\">" + issue.issue_id + "</a></td>" +
                       "<td><b>" + issue.priority + "</b></td><td align=\"center\">" + issue.component + "</td>");
            if (resolution) {
                pw.print("<td align=\"center\">[" + issue.resolution + "]</td>");
            }
            pw.println("<td>" + issue.description + "</td>");
            pw.println("</tr>");
        }
        pw.println("</table>");
        
    }
    
    /**
     * Returns updated issue to till date
     */
    private IssueInfo getUpdatedIssue(String issue_id) throws SQLException {
        ResultSet rs = getResultSet(queryCreator.getSingleIssueQuery(issue_id));
        rs.next();
        IssueInfo iss = new IssueInfo(rs.getString("issue_id"), rs.getString("priority"), 
                              rs.getString("issue_type"), rs.getString("short_desc"), 
                              rs.getString("issue_status"), rs.getString("component"), 
                              rs.getString("assigned_to"), rs.getString("subcomponent"), 
                              rs.getString("keyword"), rs.getString("resolution"));
        return iss;
    }
    
    private ArrayList removeOutgoing(String issType, String [] comps, ArrayList list) {
        
        ArrayList outIss = new ArrayList();
        
        Iterator listIter = list.iterator();
        while (listIter.hasNext()) {
            IssueInfo iss = (IssueInfo) listIter.next();
            if (!issType.equals(iss.type)) {
                outIss.add(iss);
            } else {
                boolean found = false;
                for (int i = 0; i < comps.length; i++) {
                    if (comps[i].equals(iss.component) && !found) {
                        found = true;
                    }
                }
                if (!found) {
                    outIss.add(iss);
                }
            }
        }
        
        Iterator iterOut = outIss.iterator();
        while (iterOut.hasNext()) {
            list.remove(list.indexOf(iterOut.next()));
        }
        
        return outIss;
        
    }
    
    // -------------------------------------------------------------------------
    
    private class IssueComparator implements java.util.Comparator {
        
        public int compare(Object obj1, Object obj2) {
            int comp = (((IssueInfo) obj1).component).compareTo(((IssueInfo) obj2).component);
            if (comp == 0) { // if components are equal, sort acc. priorities
                return (((IssueInfo) obj1).priority).compareTo(((IssueInfo) obj2).priority);
            } else return comp;
        }
        
    }
    
    // -------------------------------------------------------------------------
    /*
    private void dumpCollection(Collection ctd, String title) {
        Iterator iter = ctd.iterator();
            int count = 0;
            while (iter.hasNext()) {
                count++;
                IssueInfo issue = (IssueInfo) iter.next();
                System.out.println(title + " " + issue.issue_id + " " + issue.component + " " + issue.priority);
            }
            System.out.println(title + " count: " + count);
    }
    
    private void dumpMap(HashMap mtd, String title, PrintWriter out) {
        Iterator iter = mtd.keySet().iterator();
            int count = 0;
            while (iter.hasNext()) {
                count++;
                Integer key = (Integer) iter.next();
                out.println(title + " " + key + " " + ((IssueInfo) mtd.get(key)).component + " " + ((IssueInfo) mtd.get(key)).priority + "<br>");
            }
            out.println(title + " count: " + count + "<br>");
    }
    */
    // -------------------------------------------------------------------------
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
    }
    
}
