/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.beans;

import java.util.ArrayList;
import java.util.Date;
import org.netbeans.testspeceditor.Application;
import org.netbeans.testspeceditor.TextData;
import org.netbeans.testspeceditor.events.Bean;
import org.netbeans.testspeceditor.events.BeanChangeSupport;
import org.netbeans.testspeceditor.events.Persistent;
import org.netbeans.testspeceditor.undo.UndoAnnotation;
import org.netbeans.testspeceditor.undo.UndoMethods;
import org.netbeans.testspeceditor.utils.ParsingHelper;
import org.netbeans.testspeceditor.utils.Utils;
import org.netbeans.testspeceditor.xmlizable.ObjectSource;
import org.netbeans.testspeceditor.xmlizable.XMLizableException;
import org.netbeans.testspeceditor.xmlparser.Tag;
/**
 *
 * @author ehucka
 */
public class TestSpec implements Bean, Persistent, TextData {
    
    private String module;
    private String release;
    private String author;
    private String email;
    private String version;
    private String changed;
    private Introduction introduction;
    private Comments comments;
    private ArrayList<TestSuite> suites;
    private ArrayList<TestCase> cases;
    
    transient BeanChangeSupport support = new BeanChangeSupport(this);
    transient int caseid = 0;
    transient int suiteid = 0;
    
    
    public TestSpec() {
        suites = new ArrayList(32);
        cases = new ArrayList(64);
        version = "$"+"Revision: 1.1 "+"$";
        changed = "$"+"Date: "+Application.format.format(new Date())+" "+"$";
        module = "?module?";
        comments = new Comments(this);
        introduction = new Introduction(this);
    }
    
    public String getModule() {
        return module;
    }
    
    public String getAuthor() {
        return author;
    }
    
    public String getEmail() {
        return email;
    }
    
    public String getVersion() {
        return version;
    }
    
    public String getChanged() {
        return changed;
    }
    
    public String getIntroduction() {
        return introduction.getContent();
    }
    
    public Introduction getIntroductionBean() {
        return introduction;
    }
    
    public String getComments() {
        return comments.getContent();
    }
    
    public Comments getCommentsBean() {
        return comments;
    }
    
    public TestSuite[] getTestSuites() {
        return suites.toArray(new TestSuite[suites.size()]);
    }
    
    public TestCase[] getTestCases() {
        return cases.toArray(new TestCase[cases.size()]);
    }
    
    public TestCase getTestCase(int index) {
        return cases.get(index);
    }
    
    public int getTestCasesSize() {
        return cases.size();
    }
    
    public void setModule(String module) {
        String old = getModule();
        this.module = module;
        support.propertyChanged("module", old, getModule());
    }
    
    public void setAuthor(String author) {
        String old = getAuthor();
        this.author = author;
        support.propertyChanged("author", old, getAuthor());
    }
    
    public void setEmail(String email) {
        String old = getEmail();
        this.email = email;
        support.propertyChanged("email", old, getEmail());
    }
    
    public void setVersion(String version) {
        String old = getVersion();
        this.version = version;
        support.propertyChanged("version", old, getVersion());
    }
    
    public void setChanged(String changed) {
        String old = getChanged();
        this.changed = changed;
        support.propertyChanged("changed", old, getChanged());
    }
    
    public void setIntroduction(String introduction) {
        this.introduction.setContent(introduction);
    }
    
    public void setComments(String comments) {
        this.comments.setContent(comments);
    }
    
    public void addTestSuite(TestSuite testSuite) {
        suites.add(testSuite);
        support.childrenChanged(testSuite, -1, suites.size()-1);
    }
    
    public void addTestCase(TestCase testCase) {
        cases.add(testCase);
        testCase.id=caseid++;
        support.childrenChanged(testCase, -1, cases.size()-1);
    }
    
    public void removeTestSuite(TestSuite testSuite) {
        int old=suites.indexOf(testSuite);
        if (old >= 0) {
            suites.remove(testSuite);
            support.childrenChanged(testSuite, old, -1);
        }
    }
    
    public void removeTestCase(TestCase testCase) {
        int old=cases.indexOf(testCase);
        //remove from all suites
        for (TestSuite suite:suites) {
            suite.removeCase(testCase);
        }
        //remove from cases
        if (old >= 0) {
            cases.remove(testCase);
            support.childrenChanged(testCase, old, -1);
        }
    }
    
    public TestCase findTestCase(String case_id) {
        for (TestCase tc: cases) {
            if (tc.getCase_id().equals(case_id)) {
                return tc;
            }
        }
        return null;
    }
    
    public void moveUp(TestCase cas) {
        int index=cases.indexOf(cas);
        cases.remove(index);
        index--;
        cases.add(index, cas);
        support.childrenChanged(cas, index+1, index);
    }
    
    public void moveDown(TestCase cas) {
        int index=cases.indexOf(cas);
        cases.remove(index);
        index++;
        cases.add(index, cas);
        support.childrenChanged(cas, index-1, index);
    }
    
    public void moveUp(TestSuite suite) {
        int index=suites.indexOf(suite);
        suites.remove(index);
        index--;
        suites.add(index, suite);
        support.childrenChanged(suite, index+1, index);
    }
    
    public void moveDown(TestSuite suite) {
        int index=suites.indexOf(suite);
        suites.remove(index);
        index++;
        suites.add(index, suite);
        support.childrenChanged(suite, index-1, index);
    }
    
    @UndoAnnotation(method=UndoMethods.ADD)
    public void addCase(TestCase cs, int index) {
        cases.add(index, cs);
        support.childrenChanged(cs, -1, index);
    }
    
    @UndoAnnotation(method=UndoMethods.GET)
    public TestCase getCase(int index) {
        return cases.get(index);
    }
    
    @UndoAnnotation(method=UndoMethods.REMOVE)
    public TestCase removeCase(int index) {
        TestCase cs=cases.get(index);
        removeTestCase(cs);
        return cs;
    }
    
    @UndoAnnotation(method=UndoMethods.ADD)
    public void addSuite(TestSuite cs, int index) {
        suites.add(index, cs);
        support.childrenChanged(cs, -1, index);
    }
    
    @UndoAnnotation(method=UndoMethods.GET)
    public TestSuite getSuite(int index) {
        return suites.get(index);
    }
    
    @UndoAnnotation(method=UndoMethods.REMOVE)
    public TestSuite removeSuite(int index) {
        TestSuite cs=suites.get(index);
        removeTestSuite(cs);
        return cs;
    }
    
    public BeanChangeSupport getBeanSupport() {
        return support;
    }
    
    public void setObject(Tag tag) {
        module=tag.getAttribute("module");
        release=tag.getAttribute("release");
        author=tag.getAttribute("author");
        email=tag.getAttribute("email");
        version=tag.getAttribute("version");
        changed=tag.getAttribute("changed");
        comments.setContent(Utils.getValueNotNull(tag, "comments"));
        introduction.setContent(Utils.getValueNotNull(tag, "introduction"));
        
        Tag tcases=tag.getChild("cases");
        ParsingHelper.getInstance().getCases().clear();
        if (tcases != null) {
            for (int i=0;i < tcases.getChildrenSize();i++) {
                Tag tcs=tcases.getChild(i);
                ObjectSource osrc=new ObjectSource(tcs);
                try {
                    TestCase cs=(TestCase)(osrc.getObject());
                    if (cs != null) {
                        cases.add(cs);
                        ParsingHelper.getInstance().getCases().put(cs.getCase_id(), cs);
                    } else {
                        throw new XMLizableException("File contains wrong test case ");
                    }
                } catch (XMLizableException ex) {
                    ex.printStackTrace();
                }
            }
        }
        Tag tsuites=tag.getChild("suites");
        if (tsuites != null) {
            for (int i = 0; i < tsuites.getChildrenSize(); i++) {
                Tag ts=tsuites.getChild(i);
                ObjectSource osrc=new ObjectSource(ts);
                try {
                    TestSuite scr=(TestSuite)(osrc.getObject());
                    if (scr != null) {
                        suites.add(scr);
                    } else {
                        throw new XMLizableException("File contains wrong test case ");
                    }
                } catch (XMLizableException ex) {
                    ex.printStackTrace();
                }
            }
        }
    }
    
    public void initObject() {
        comments.setParent(this);
        introduction.setParent(this);
        caseid=0;
        suiteid=0;
        for (TestCase cs:cases) {
            int in=cs.getCase_id().lastIndexOf('_');
            if (in >= 0) {
                try {
                    String n=cs.getCase_id().substring(in+1);
                    int num=Integer.parseInt(n);
                    cs.id=num;
                    if (caseid < num+1) {
                        caseid=num+1;
                    }
                } catch (Exception ex) {
                }
            }
            if (cs.id < 0) {
                cs.id=caseid++;
            }
        }
        for (TestSuite ts:suites) {
            int in=ts.getSuite_id().lastIndexOf('_');
            if (in >= 0) {
                try {
                    String n=ts.getSuite_id().substring(in+1);
                    int num=Integer.parseInt(n);
                    ts.id=num;
                    if (suiteid < num+1) {
                        suiteid=num+1;
                    }
                } catch (Exception ex) {
                }
            }
            if (ts.id < 0) {
                ts.id=caseid++;
            }
        }
    }
    
    public Bean copy() {
        return this;
    }
    
    public String[] getContents() {
        String[] ret=new String[2];
        ret[0]=introduction.getContent();
        ret[1]=comments.getContent();
        return ret;
    }
    
    public String[] getWholeContents() {
        ArrayList<String> ret=new ArrayList(suites.size()*4+cases.size()*12);
        ret.add(introduction.getContent());
        ret.add(comments.getContent());
        for (TestSuite suit:suites) {
            String[] cnt=suit.getContents();
            for (String s:cnt) {
                ret.add(s);
            }
        }
        for (TestCase cas:cases) {
            String[] cnt=cas.getContents();
            for (String cn:cnt) {
                ret.add(cn);
            }
        }
        return ret.toArray(new String[ret.size()]);
    }
    
    public String getRelease() {
        return release;
    }
    
    public void setRelease(String release) {
        String old = getRelease();
        this.release = release;
        support.propertyChanged("release", old, getRelease());
    }
}
