/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.find;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *
 * @author ehucka
 */
public class ReplacePattern extends FindPattern {
    
    Pattern findPattern;
    String replaceText="";
    
    
    public ReplacePattern(String value, boolean matchCase, boolean wholeWord, boolean regExp,
            boolean selectedNodes, String replaceText) {
        super(value, matchCase, wholeWord, regExp, selectedNodes);
        this.replaceText = replaceText;
        if (replaceText == null) {
            this.replaceText="";
        }
        if (regExp) {
            findPattern = Pattern.compile(value, Pattern.DOTALL|((matchCase)?0:Pattern.CASE_INSENSITIVE));
            findGroups();
        }
    }
    
    public String replace(FoundOccurance occurance) {
        String replace=new String(replaceText);
        if (isRegExp()) {
            Matcher m=findPattern.matcher(occurance.getPattern());
            m.find();
            if (m.groupCount() > 0) {
                for (int i = 0; i < m.groupCount(); i++) {
                    String t=m.group(i);
                    String v="$"+String.valueOf(i)+"$";
                    replace=replace.replace(v, t);
                }
            }
        }
        return replace;
    }
    
    protected void findGroups() {
        StringBuilder sb=new StringBuilder();
        
        int in=replaceText.indexOf('$');
        int pre=0;
        while (in > -1) {
            int x=in+1;
            while (x < replaceText.length() && Character.isDigit(replaceText.charAt(x))) {
                x++;
            }
            sb.append(replaceText.substring(pre, x));
            if (x > in+1) {
                sb.append('$');
            }
            pre=x;
            in=replaceText.indexOf('$', in+1);
        }
        if (pre < replaceText.length()) {
            sb.append(replaceText.substring(pre));
        }
        replaceText=sb.toString();
    }
    
    protected FoundOccurance createFoundOccurance(int start, int end, String text) {
        return new ReplaceOccurance(start, end, text, replaceText);
    }
}
