/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui;


import java.lang.reflect.Method;
import org.netbeans.testspeceditor.ui.actions.CopyAction;
import org.netbeans.testspeceditor.ui.actions.CutAction;
import org.netbeans.testspeceditor.ui.tree.Node;
import org.netbeans.testspeceditor.undo.UndoManager;
import org.netbeans.testspeceditor.utils.Utils;

/**
 *
 * @author  eh103527
 */
public class NodesClipboard {
    
    static NodesClipboard instance = null;
    
    Node[] content;
    boolean copy;
    /**
     * Creates a new instance of NodesClipboard
     */
    public NodesClipboard() {
    }
    
    public static NodesClipboard getInstance() {
        if (instance == null) {
            instance=new NodesClipboard();
        }
        return instance;
    }
    
    public boolean isEmpty() {
        return (content == null || content.length == 0);
    }
    
    public Node[] content() {
        return content;
    }
    
    public void copy(Node[] selected) {
        copy=true;
        content=selected;
    }
    
    public void cut(Node[] selected) {
        copy=false;
        content=selected;
    }
    
    public void paste(Node target) {
        UndoManager.getInstance().startAtomic();
        UndoManager.getInstance().setLogProperties(false);
        try {
            if (copy) {
                //make copy of content
                Class cl=content[0].getClass();
                Method[] ms=Utils.findActionAnnotatedMethods(cl, CopyAction.class);
                for (Node n:content) {
                    try {
                        ms[0].invoke(n, target);
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                }
            } else {
                //remove all from parents
                Class cl=content[0].getClass();
                Method[] ms=Utils.findActionAnnotatedMethods(cl, CutAction.class);
                for (Node n:content) {
                    try {
                        ms[0].invoke(n, target);
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                }
            }
        } finally {
            UndoManager.getInstance().endAtomic();
            UndoManager.getInstance().setLogProperties(true);
        }
    }
}
