/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui;

import java.awt.CardLayout;
import java.util.HashMap;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionListener;
import org.netbeans.testspeceditor.DocumentListener;
import org.netbeans.testspeceditor.beans.TestAction;
import org.netbeans.testspeceditor.beans.TestCase;
import org.netbeans.testspeceditor.beans.TestSpec;
import org.netbeans.testspeceditor.beans.TestSuite;
import org.netbeans.testspeceditor.events.Bean;
import org.netbeans.testspeceditor.ui.panels.AbstractPanel;
import org.netbeans.testspeceditor.ui.panels.ActionPanel;
import org.netbeans.testspeceditor.ui.panels.CasePanel;
import org.netbeans.testspeceditor.ui.panels.SuitePanel;
import org.netbeans.testspeceditor.ui.panels.TestSpecPanel;
import org.netbeans.testspeceditor.ui.tree.DataNode;
import org.netbeans.testspeceditor.ui.tree.Node;

/**
 *
 * @author ehucka
 */
public class PanelCard implements TreeSelectionListener, DocumentListener {
    
    private static final String CARD_EMPTY="empty";
    
    JPanel root;
    HashMap<String, AbstractPanel> panels;
    CardLayout layout;
    String selected=null;
    Bean selectedObject = null;
    
     
    public PanelCard(JPanel root) {
        this.root=root;
        panels=new HashMap(8);
        layout=(CardLayout)(root.getLayout());
        
        panels.put(TestSpec.class.getSimpleName(), new TestSpecPanel());
        panels.put(TestSuite.class.getSimpleName(), new SuitePanel());
        panels.put(TestCase.class.getSimpleName(), new CasePanel());
        panels.put(TestAction.class.getSimpleName(), new ActionPanel());
        panels.put(CARD_EMPTY, new AbstractPanel() {
            protected void initAdapters() {}
        });
        for (String key : panels.keySet()) {
            root.add(panels.get(key), key);
        }
        selectPanel(CARD_EMPTY);
    }
    
    public void setSelectedNode(Node selectedNode) {
        if (!(selectedNode instanceof DataNode)) {
            if (selected != CARD_EMPTY)
                selectPanel(CARD_EMPTY);
            selectedObject=null;
            return;
        }
        Bean bean=((DataNode)selectedNode).getData();
        Bean parent=((selectedNode.getParent() instanceof DataNode) ?
            ((DataNode)selectedNode.getParent()).getData():null);
        Class cl=bean.getClass();
        String sel=cl.getSimpleName();
        
        while (panels.get(sel) == null && cl.getSuperclass() != null) {
            cl=cl.getSuperclass();
            sel=cl.getSimpleName();
        }
        if (selectedObject != bean) {
            if (!selected.equals(sel))
                selectPanel(sel);
            panels.get(sel).setBean(bean, parent);
            selectedObject=bean;
        }
    }
    
    private void selectPanel(String name) {
        layout.show(root, name);
        selected=name;
        AbstractPanel p=panels.get(name);
        for (AbstractPanel pan:panels.values()) {
            if (pan != p) {
                pan.setBean(null, null);
            }
        }
    }
    
    public void valueChanged(javax.swing.event.TreeSelectionEvent e) {
        Node selected=((Node)(e.getPath().getLastPathComponent()));
        setSelectedNode(selected);
    }
    
    public void invokeDefaultAction(JTree tree) {
        setSelectedNode(null);
        if (tree.getSelectionPath() != null) {
            Node selected=((Node)(tree.getSelectionPath().getLastPathComponent()));
            setSelectedNode(selected);
        }
    }
    
    public void documentChanged() {
        selectPanel(CARD_EMPTY);
    }
    
    public void finish() {
        for (AbstractPanel panel:panels.values()) {
            panel.finish();
        }
    }
}
