/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui;

import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import org.netbeans.testspeceditor.ui.actions.*;

/**
 *
 * @author ehucka
 */
public class UIInitializer {
    
    /**
     * Creates a new instance of UIInitializer
     */
    public UIInitializer() {
    }
    
    public static void initActions(MainFrame frame) {
        ActionsManager.getInstance().registerAction(frame, new CopyAction());
        ActionsManager.getInstance().registerAction(frame, new CutAction());
        ActionsManager.getInstance().registerAction(frame, new DeleteAction());
        ActionsManager.getInstance().registerAction(frame, new ExitApplicationAction());
        ActionsManager.getInstance().registerAction(frame, new NewDocumentAction());
        ActionsManager.getInstance().registerAction(frame, new OpenDocumentAction());
        ActionsManager.getInstance().registerAction(frame, new OptionsAction());
        ActionsManager.getInstance().registerAction(frame, new PasteAction());
        ActionsManager.getInstance().registerAction(frame, new RedoAction());
        ActionsManager.getInstance().registerAction(frame, new SaveAsDocumentAction());
        ActionsManager.getInstance().registerAction(frame, new SaveDocumentAction());
        ActionsManager.getInstance().registerAction(frame, new UndoAction());
        ActionsManager.getInstance().registerAction(frame, new ErrorsAction(frame.getErrors()));
        ActionsManager.getInstance().registerAction(frame, new HelpAction());
        ActionsManager.getInstance().registerAction(frame, new NextMatchingWordAction());
        ActionsManager.getInstance().registerAction(frame, new CompletionAction());
        ActionsManager.getInstance().registerAction(frame, new MoveUpAction());
        ActionsManager.getInstance().registerAction(frame, new MoveDownAction());
        ActionsManager.getInstance().registerAction(frame, new FindAction());
        ActionsManager.getInstance().registerAction(frame, new ReplaceAction());
    }
    
    public static void addTreeUpdateListener(JTree tree) {
        tree.getModel().addTreeModelListener(new TreeUpdater(tree));
    }
    
    static class TreeUpdater implements TreeModelListener {
        
        JTree tree;
        public TreeUpdater(JTree tree) {
            this.tree=tree;
        }
        
        public void treeNodesChanged(final TreeModelEvent e) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    tree.setRowHeight(((tree.getRowHeight() == 0)?-1:0));//hack
                    tree.invalidate();
                    tree.treeDidChange();
                }
            });
        }
        public void treeNodesInserted(TreeModelEvent e) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    tree.treeDidChange();
                }
            });
        }
        public void treeNodesRemoved(TreeModelEvent e) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    tree.treeDidChange();
                }
            });
        }
        public void treeStructureChanged(TreeModelEvent e) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    tree.treeDidChange();
                }
            });
        }
    }
}
