/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.dialogs;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import org.netbeans.testspeceditor.Application;
import org.netbeans.testspeceditor.utils.FileUtil;
import org.netbeans.testspeceditor.utils.JSPWikiFilter;
import org.netbeans.testspeceditor.utils.PostTransformationFilter;
import org.netbeans.testspeceditor.utils.Timer;

/**
 *
 * @author  ehucka
 */
public class TransformDialog extends javax.swing.JDialog implements DocumentListener, Runnable {
    
    public static String[] XSL_NAMES = new String[] {"testspec2html.xsl", "testspec2html2.xsl", "testspec2jspwiki.xsl"};
    public static String[] xsls = null;
    static final int XSL_DEFAULT = 0;
    static final int XSL_CUSTOM = 1;
    static String customF;
    static String outputF;
    static int selectedType=-1;
    
    protected static void initXSLs() {
        ArrayList<String> list = new ArrayList<String>(4);
        for (String s : XSL_NAMES) {
            URL url = TransformDialog.class.getResource("/"+s);
            if (url != null) {
                list.add(s);
            }
        }
        xsls = list.toArray(new String[list.size()]);
        
    }
    
    int xsltype=XSL_DEFAULT;
    File xsl, html;
    
    public TransformDialog() {
        super(Application.getInstance().getUi(), false);
        if (xsls == null) {
            initXSLs();
        }
        initComponents();
        if (xsls.length == 0) {
            defaultR.setEnabled(false);
        }
        if (selectedType > -1) {
            selectType(selectedType);
            customFileF.setText(customF);
            htmlF.setText(outputF);
        } else {
            selectType(XSL_DEFAULT);
        }
        htmlF.getDocument().addDocumentListener(this);
        getRootPane().setDefaultButton(transformB);
        setLocation(getParent().getX()+(getParent().getWidth()-getWidth())/2,
                getParent().getY()+(getParent().getHeight()-getHeight())/2);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonGroup1 = new javax.swing.ButtonGroup();
        jPanel2 = new javax.swing.JPanel();
        defaultR = new javax.swing.JRadioButton();
        customR = new javax.swing.JRadioButton();
        customFileF = new javax.swing.JTextField();
        selectFileB = new javax.swing.JButton();
        defaultNameL = new javax.swing.JLabel();
        xslList = new JComboBox(xsls);
        jPanel3 = new javax.swing.JPanel();
        htmlF = new javax.swing.JTextField();
        selectHtmlB = new javax.swing.JButton();
        jPanel4 = new javax.swing.JPanel();
        transformB = new javax.swing.JButton();
        stornoB = new javax.swing.JButton();
        previewB = new javax.swing.JButton();

        FormListener formListener = new FormListener();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Transform to HTML");
        addWindowListener(formListener);

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Select XSL:"));

        buttonGroup1.add(defaultR);
        defaultR.setSelected(true);
        defaultR.setText("Default");
        defaultR.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        defaultR.setMargin(new java.awt.Insets(0, 0, 0, 0));
        defaultR.addActionListener(formListener);

        buttonGroup1.add(customR);
        customR.setText("Custom:");
        customR.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        customR.setMargin(new java.awt.Insets(0, 0, 0, 0));
        customR.addActionListener(formListener);

        selectFileB.setText("...");
        selectFileB.setMargin(new java.awt.Insets(2, 2, 2, 2));
        selectFileB.setPreferredSize(new java.awt.Dimension(21, 22));
        selectFileB.addActionListener(formListener);

        defaultNameL.setFont(new java.awt.Font("Dialog", 0, 12));

        xslList.setModel(xslList.getModel());

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel2Layout.createSequentialGroup()
                        .add(defaultR, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 69, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(defaultNameL)
                            .add(xslList, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 202, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel2Layout.createSequentialGroup()
                        .add(customFileF, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 326, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(selectFileB, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(customR))
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(defaultR)
                        .add(xslList, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(defaultNameL))
                .add(14, 14, 14)
                .add(customR)
                .add(6, 6, 6)
                .add(customFileF, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 22, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(13, 13, 13))
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel2Layout.createSequentialGroup()
                .add(51, 51, 51)
                .add(selectFileB, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder("Select Output:"));

        selectHtmlB.setText("...");
        selectHtmlB.setMargin(new java.awt.Insets(2, 2, 2, 2));
        selectHtmlB.setPreferredSize(new java.awt.Dimension(21, 22));
        selectHtmlB.addActionListener(formListener);

        org.jdesktop.layout.GroupLayout jPanel3Layout = new org.jdesktop.layout.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .add(htmlF, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 326, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(selectHtmlB, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel3Layout.createSequentialGroup()
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(htmlF, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 22, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(selectHtmlB, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        jPanel4.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 10, 10));

        transformB.setText("Transform");
        transformB.setMargin(new java.awt.Insets(2, 5, 2, 5));
        transformB.setPreferredSize(new java.awt.Dimension(85, 22));
        transformB.addActionListener(formListener);
        jPanel4.add(transformB);

        stornoB.setText("Close");
        stornoB.setMargin(new java.awt.Insets(2, 5, 2, 5));
        stornoB.setPreferredSize(new java.awt.Dimension(85, 22));
        stornoB.addActionListener(formListener);
        jPanel4.add(stornoB);

        previewB.setText("Preview");
        previewB.setEnabled(false);
        previewB.setMargin(new java.awt.Insets(2, 5, 2, 5));
        previewB.setPreferredSize(new java.awt.Dimension(85, 22));
        previewB.addActionListener(formListener);
        jPanel4.add(previewB);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel4, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 387, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel4, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 52, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }

    // Code for dispatching events from components to event handlers.

    private class FormListener implements java.awt.event.ActionListener, java.awt.event.WindowListener {
        FormListener() {}
        public void actionPerformed(java.awt.event.ActionEvent evt) {
            if (evt.getSource() == defaultR) {
                TransformDialog.this.defaultRActionPerformed(evt);
            }
            else if (evt.getSource() == customR) {
                TransformDialog.this.customRActionPerformed(evt);
            }
            else if (evt.getSource() == selectFileB) {
                TransformDialog.this.selectFileBActionPerformed(evt);
            }
            else if (evt.getSource() == selectHtmlB) {
                TransformDialog.this.selectHtmlBActionPerformed(evt);
            }
            else if (evt.getSource() == transformB) {
                TransformDialog.this.transformBActionPerformed(evt);
            }
            else if (evt.getSource() == stornoB) {
                TransformDialog.this.stornoBActionPerformed(evt);
            }
            else if (evt.getSource() == previewB) {
                TransformDialog.this.previewBActionPerformed(evt);
            }
        }

        public void windowActivated(java.awt.event.WindowEvent evt) {
        }

        public void windowClosed(java.awt.event.WindowEvent evt) {
        }

        public void windowClosing(java.awt.event.WindowEvent evt) {
            if (evt.getSource() == TransformDialog.this) {
                TransformDialog.this.closeDialog(evt);
            }
        }

        public void windowDeactivated(java.awt.event.WindowEvent evt) {
        }

        public void windowDeiconified(java.awt.event.WindowEvent evt) {
        }

        public void windowIconified(java.awt.event.WindowEvent evt) {
        }

        public void windowOpened(java.awt.event.WindowEvent evt) {
        }
    }// </editor-fold>//GEN-END:initComponents
    
    private void previewBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_previewBActionPerformed
        PreviewWindow dlg=new PreviewWindow(getHtmlFile());
        dlg.setVisible(true);
    }//GEN-LAST:event_previewBActionPerformed
    
    private void customRActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_customRActionPerformed
        selectType(XSL_CUSTOM);
    }//GEN-LAST:event_customRActionPerformed
    
    private void defaultRActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_defaultRActionPerformed
        selectType(XSL_DEFAULT);
    }//GEN-LAST:event_defaultRActionPerformed
    
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
    
    private void stornoBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stornoBActionPerformed
        closeDialog(null);
    }//GEN-LAST:event_stornoBActionPerformed
    
    private void transformBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_transformBActionPerformed
        String s=null;
        s=htmlF.getText();
        if (s == null || s.length() == 0) {
            JOptionPane.showMessageDialog(getParent(),
                    "You have not specified output html file.", "Warrning!", JOptionPane.WARNING_MESSAGE);
            return;
        }
        html=new File(s);
        if (defaultR.isSelected()) {
            xsltype=XSL_DEFAULT;
        } else {
            xsltype=XSL_CUSTOM;
            s=customFileF.getText();
            if (s == null || s.length() == 0 || !(xsl=new File(s)).exists()) {
                JOptionPane.showMessageDialog(getParent(),
                        "You have not specified XSL file.", "Warrning!", JOptionPane.WARNING_MESSAGE);
                return;
            }
        }
        customF=customFileF.getText();
        outputF=htmlF.getText();
        selectedType=(defaultR.isSelected())?XSL_DEFAULT:XSL_CUSTOM;
        Timer.getInstance().delayOnce(new Runnable() {
            public void run() {
                try {
                    enableComponents(false);
                    InputStream xsl = getTransformationFile();
                    Application.getInstance().getDocument().transform(getHtmlFile(), xsl, getFilter());
                    xsl.close();
                } catch (Exception ex) {
                    ex.printStackTrace();
                } finally {
                    enableComponents(true);
                }
                if (getHtmlFile().exists() && getHtmlFile().length() > 0) {
                    previewEnable(true);
                }
            }
        }, 10);
    }//GEN-LAST:event_transformBActionPerformed
    
    private void selectHtmlBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_selectHtmlBActionPerformed
        String s=FileUtil.openFileDialog(true, "Open", false, new String[] {"html"}, "HTML");
        if (s != null) {
            if (!s.toLowerCase().endsWith(".html")) {
                s=s+".html";
            }
            htmlF.setText(s);
        }
    }//GEN-LAST:event_selectHtmlBActionPerformed
    
    private void selectFileBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_selectFileBActionPerformed
        String s=FileUtil.openFileDialog(true, "Open", false, new String[] {"xsl"}, "XSL");
        if (s != null) {
            customFileF.setText(s);
        }
    }//GEN-LAST:event_selectFileBActionPerformed
    
    protected void previewEnable(final boolean val) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                previewB.setEnabled(val);
            }
        });
    }
    
    protected void enableComponents(final boolean val) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                previewB.setEnabled(val);
                transformB.setEnabled(val);
                stornoB.setEnabled(val);
                htmlF.setEditable(val);
                selectHtmlB.setEnabled(val);
            }
        });
    }
    
    protected void selectType(int type) {
        if (type == XSL_DEFAULT) {
            if (!defaultR.isEnabled())
                selectType(XSL_CUSTOM);
            if (!defaultR.isSelected())
                defaultR.setSelected(true);
            customFileF.setEnabled(false);
            selectFileB.setEnabled(false);
            defaultNameL.setEnabled(true);
        } else {
            if (!customR.isSelected())
                customR.setSelected(true);
            customFileF.setEnabled(true);
            selectFileB.setEnabled(true);
            defaultNameL.setEnabled(false);
        }
    }
    
    /**
     * should be closed
     */
    public InputStream getTransformationFile() throws Exception {
        if (xsltype == XSL_DEFAULT) {
            String name = (String)xslList.getSelectedItem();
            return new BufferedInputStream(getClass().getResourceAsStream("/"+name));
        } else {
            return new BufferedInputStream(new FileInputStream(xsl));
        }
    }
    
    public File getHtmlFile() {
        return html;
    }
    
    public PostTransformationFilter getFilter() {
        PostTransformationFilter filter = null;
        if (xsltype == XSL_DEFAULT) {
            String name = (String)xslList.getSelectedItem();
            if (name.endsWith("jspwiki.xsl")) {
                filter = new JSPWikiFilter();
            }
        } else {
            if (xsl.getName().endsWith("jspwiki.xsl")) {
                filter = new JSPWikiFilter();
            }
        }
        return filter;
    }
    
    public void insertUpdate(DocumentEvent e) {
        Timer.getInstance().delayOnce(this, 200);
    }
    
    public void removeUpdate(DocumentEvent e) {
        Timer.getInstance().delayOnce(this, 200);
    }
    
    public void changedUpdate(DocumentEvent e) {
    }
    
    public void run() {
        try {
            html=new File(htmlF.getText());
            if (html != null && html.exists() && html.length() > 0) {
                previewB.setEnabled(true);
                return;
            }
        } catch (Exception ex) {
        }
        if (previewB.isEnabled()) {
            previewB.setEnabled(false);
        }
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    public javax.swing.ButtonGroup buttonGroup1;
    public javax.swing.JTextField customFileF;
    public javax.swing.JRadioButton customR;
    public javax.swing.JLabel defaultNameL;
    public javax.swing.JRadioButton defaultR;
    public javax.swing.JTextField htmlF;
    public javax.swing.JPanel jPanel2;
    public javax.swing.JPanel jPanel3;
    public javax.swing.JPanel jPanel4;
    public javax.swing.JButton previewB;
    public javax.swing.JButton selectFileB;
    public javax.swing.JButton selectHtmlB;
    public javax.swing.JButton stornoB;
    public javax.swing.JButton transformB;
    public javax.swing.JComboBox xslList;
    // End of variables declaration//GEN-END:variables
    
}
