/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.edits;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import org.netbeans.testspeceditor.Application;
import org.netbeans.testspeceditor.TextData;
import org.netbeans.testspeceditor.utils.Timer;

/**
 *
 * @author ehucka
 */
public class MatchingWords implements Runnable {
    
    private static MatchingWords instance = null;
    
    ArrayList<String> words;
    int[] indicies;
    HashSet<String> temporary;
    
    LinkedList<TextData> updates=new LinkedList();
    HashSet<String> tempwords=new HashSet(32);
    
    FullUpdater updater;
    
     
    protected MatchingWords() {
        words=new ArrayList(2048);
        temporary=new HashSet(2048);
        updater=new FullUpdater();
        indicies=new int[1+'z'-'0'];
        for (int i = 0; i < indicies.length; i++) {
            indicies[i]=-1;
        }
    }
    
    public static MatchingWords getInstance() {
        if (instance == null) {
            instance=new MatchingWords();
        }
        return instance;
    }
    
    protected void update(TextData data, boolean clear) {
        temporary.addAll(words);
        HashSet<String> wordsset=new HashSet(1024);
        getWords(data, wordsset);
        if (clear) {
            for (String s : tempwords) {
                words.remove(s);
            }
            tempwords.clear();
        } else {
            cleanExisting(wordsset);
            tempwords.addAll(wordsset);
        }
        update(wordsset);
    }
    
    protected void cleanExisting(HashSet<String> wordsset) {
        String[] ars=wordsset.toArray(new String[wordsset.size()]);
        for (int i = 0; i < ars.length; i++) {
            if (temporary.contains(ars[i])) {
                wordsset.remove(ars[i]);
            }
        }
    }
    
    protected void update(HashSet<String> wordsset) {
        temporary.clear();
        if (words.size() == 0) {
            words.addAll(wordsset);
            Collections.sort(words);
        } else {
            boolean added=false;
            for (String w:wordsset) {
                added=false;
                int start=0;
                if (w.charAt(0)-'0' < indicies.length && indicies[w.charAt(0)-'0'] >= 0) {
                    start=indicies[w.charAt(0)-'0'];
                }
                for (int i = start; i < words.size(); i++) {
                    if (words.get(i).compareTo(w) > 0) {
                        words.add(i, w);
                        added=true;
                        break;
                    }
                }
                if (!added) {
                    words.add(w);
                }
            }
        }
        //reindexing
        for (int i = 0; i < indicies.length; i++) {
            indicies[i]=-1;
        }
        int last=-1;
        for (int i = 0; i < words.size(); i++) {
            int first=words.get(i).charAt(0)-'0';
            if (last < first) {
                indicies[first]=i;
                last=first;
            }
        }
        
    }
    
    protected void getWords(TextData data, HashSet<String> wordset) {
        String[] conts=data.getContents();
        for (String cn:conts) {
            if (cn != null && cn.length() > 0) {
                getWords(cn, wordset);
            }
        }
    }
    
    protected void getWords(String text, HashSet<String> wordset) {
        FastTokenizer st=new FastTokenizer(text, " \n!\"#$%&'()*+,-./:;<=>?@[\\]^_`{|}~");
        while (st.hasMoreTokens()) {
            String t=st.nextToken().trim();//.toLowerCase();
            if (t.length() > 1)
                wordset.add(t);
        }
    }
    
    public String[] getMatchings(String prefix) {
        if (prefix != null && prefix.length() > 0) {
            String p=prefix;//.toLowerCase();
            int ind=p.charAt(0)-'0';
            int next=words.size();
            if (indicies[ind] > -1) {
                for (int i = ind+1; i < indicies.length; i++) {
                    if (indicies[i] > -1) {
                        next=indicies[i];
                        break;
                    }
                }
                ind=indicies[ind];
                ArrayList<String> matches=new ArrayList(32);
                
                for (int i = ind; i < next; i++) {
                    if (words.get(i).startsWith(p)) {
                        matches.add(words.get(i));
                    }
                }
                return matches.toArray(new String[matches.size()]);
            }
        }
        return new String[0];
    }
    
    /**
     * Incremental updating.
     */
    public void updateWords(TextData data) {
        synchronized (updates) {
            if (updates.size() == 0 || updates.getLast() != data)
                updates.add(data);
        }
        Timer.getInstance().delayOnce(this, 1000);
        Timer.getInstance().delayOnce(updater, 20000);
    }
    
    public void run() {
        update();
    }
    
    TextData editing = null;
    
    private synchronized void update() {
        LinkedList<TextData> tmp=new LinkedList();
        synchronized (updates) {
            tmp.addAll(updates);
            updates.clear();
        }
        for (TextData obj : tmp) {
            if (obj != editing && editing != null) {
                update(editing, true);
                editing=obj;
            }
            update(obj, false);
        }
    }
    
    class FullUpdater implements Runnable {
        public void run() {
            words.clear();
            synchronized (updates) {
                updates.clear();
                updates.add(Application.getInstance().getDocument());
            }
            update();
        }
    }
    
    public static void main(String[] args) {
        /*ArrayList<String> words=new ArrayList();
        words.add("01234");
        words.add("abcd");
        words.add("abdd");
        words.add("abee");
        words.add("bert");
        words.add("bomb");
        words.add("cert");
        //reindexing
        int[] indicies=new int['z'-'0'];
        for (int i = 0; i < indicies.length; i++) {
            indicies[i]=-1;
        }
        int last=0;
        boolean found=false;
        for (int i = 0; i < indicies.length; i++) {
            found=false;
            for (int j = last; j < words.size(); j++) {
                String w=words.get(j);
                if (w.charAt(0) == (char)('0'+i)) {
                    indicies[i]=j;
                    found=true;
                    break;
                } else if (w.charAt(0) > (char)('0'+i)) {
                    indicies[w.charAt(0)-'0']=j;
                    i=w.charAt(0)-'0';
                    found=true;
                    break;
                }
            }
            if (found)
                last=indicies[i]+1;
            else
                indicies[i]=-1;
        }
        for (int i = 0; i < indicies.length; i++) {
            if (indicies[i] > -1) {
                System.out.println("indicies "+i+" = "+indicies[i]);
            }
        }*/
        for (int i = 31; i < 127; i++) {
            System.out.print((char)i);
        }
    }
}
