/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.xmlizable;



import org.netbeans.testspeceditor.xmlparser.ParserException;
import org.netbeans.testspeceditor.xmlparser.Tag;
import org.netbeans.testspeceditor.xmlparser.XMLDocument;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.StringTokenizer;


/**
 *
 * @author ehucka
 */
public class ObjectSource {
    
    ArrayList waitings=new ArrayList(100);
    HashMap<Object, ArrayList> blinds=new HashMap(128);
    HashMap<String, Object> ids=new HashMap(128);
    HashMap objects = new HashMap(128);
    
    String[] filterNameSpace = null;
    
    Tag rootTag=null;
    
    XMLUtils utils;
    

    public ObjectSource(XMLDocument document) {
        this(document.getRoot());
    }
    
    public ObjectSource(XMLDocument document, XMLUtils utils) {
        this(document.getRoot(), utils);
    }
    
    public ObjectSource(Tag tag) {
        this(tag, XMLUtils.getUtils());
    }
    
    public ObjectSource(Tag tag, XMLUtils utils) {
        rootTag=tag;
        this.utils=utils;
    }
    
    public XMLizable getObject() throws XMLizableException {
        try {
            Object obj=getObject(rootTag);
            solveBlindLinks();
            waitings.clear();
            blinds.clear();
            callInitMethods();
            objects.clear();
            return (XMLizable)obj;
        } catch (XMLizableException ex) {
            throw ex;
        } catch (Exception ex) {
            throw new XMLizableException(ex);
        }
    }
    
    public XMLUtils getUtils() {
        return utils;
    }
    
    protected Object getObject(Tag tag) throws Exception {
        //read tag class attribute first
        Class cl=utils.getClassForTag(tag);
        if (cl == null) {
            throw new XMLizableException(tag.getName()+" cannot be dexmlizated.");
        }
        Object ret=null;
        //new instance
        try {
            ret=cl.newInstance();
        } catch (InstantiationException ex) {
            throw new XMLizableException(cl.getName()+" has not defined default constructor.");
        }
        //is there setobject method?
        if (utils.callSetObject(ret, tag)) {
            objects.put(getObjectKey((XMLizable)ret, tag) ,ret);
            return ret;
        }
        
        //read all attributes
        String[] attrs=tag.getAttributes();
        for (int i=0;i < attrs.length;i++) {
            //skip class name
            if (attrs[i].equals(XMLUtils.CLASS_ATTRIBUTE)) {
                continue;
            }
            if (attrs[i].equals(utils.ID_ATTRIBUTE)) {
                ids.put(tag.getAttribute(attrs[i]), ret);
                continue;
            }
            //find field with attribute name
            try {
                Field field=utils.getField(ret.getClass(), attrs[i]);
                if (field == null) {
                    System.err.println("[Dexmlization] Unexisting field '"+attrs[i]+"' in "+ret.getClass());
                    continue;
                }
                field.setAccessible(true);
                //single association
                if (utils.isSingleField(field, ret)) {
                    if (XMLizable.class.isAssignableFrom(field.getType())) {
                        String id=tag.getAttribute(attrs[i]);
                        waitings.add(new WaitingLink((XMLizable)ret, field, id));
                    } else { //primitive
                        String val=tag.getAttribute(attrs[i]);
                        utils.setField(field, ret, val);
                        objects.put(getObjectKey(val, tag), val);
                    }
                } else if (utils.isMultiField(field, ret)) {//list - multi association
                    String val=tag.getAttribute(attrs[i]);
                    //                    Collection list=(Collection)(field.get(ret));
                    
                    if (val != null && val.length() > 0) {
                        StringTokenizer st=new StringTokenizer(val, ",");
                        while (st.hasMoreTokens()) {
                            String tok=st.nextToken().trim();
                            waitings.add(new WaitingLink((XMLizable)ret, field, tok));
                        }
                    }
                }
            } catch (Exception ex) {  //the exception is not thrown outside, wrong field is probbaly not a big problem
                ex.printStackTrace();
            }
        }
        //read all one-level subtags
        
        for (int i=0;i < tag.getChildrenSize();i++) {
            Tag child=tag.getChild(i);
            if (!canParseTag(child)) continue;
            boolean fieldtag=true;
            try {
                //test for field tag - auxiliary tag named by field
                Field field=utils.getField(ret.getClass(), child.getName());
                if (field != null) {
                    if (utils.isSingleField(field, ret)) {
                        if (child.getChildrenSize() <= 1) { //there should be only one subtag
                            boolean set=false;
                            try {
                                Tag t=child.getChild(0);
                                Object o=getObject(t);
                                if (o != null && field.getType().isAssignableFrom(o.getClass())) {
                                    makeAssociation(ret, field, o);
                                    set=true;
                                }
                            } catch (Exception ex) {}
                            if (!set) {
                                fieldtag=false;
                            }
                        } else if (child.getChildrenSize() > 1) {
                            fieldtag=false;
                        }
                    } else if (utils.isMultiField(field, ret)) {
                        try {
                            Collection list=(Collection)(field.get(ret));
                            if (list == null) {
                                new ParserException("Collection "+field.getName()+" of "+ret.getClass().getName()+" is not initialized in default constructor.").printStackTrace();
                            }
                            for (int j=0;j < child.getChildrenSize();j++) {
                                Tag t=child.getChild(j);
                                Object o=getObject(t);
                                makeAssociation(list, ret, o);
                            }
                        } catch (Exception ex) {
                            ex.printStackTrace();
                        }
                    }
                } else {
                    fieldtag=false;
                }
            } catch (Exception ex) {
                fieldtag=false;
            }
            if (!fieldtag) { //direct tag - find field of the type
                Class cla=utils.getClassForTag(child);
                if (cla != null) {
                    Object o=getObject(child);
                    if (o == null) {
                        throw new IllegalStateException("Tag: "+child.getName()+" cannot be dexmlizated.");
                    }
                    //find single field - precise
                    ArrayList<Field> fields=utils.getAllSingleFields(ret);
                    boolean conflict=false;
                    boolean set=false;
                    for (Field fl:fields) {
                        if (fl.getType().isAssignableFrom(cla)) {
                            if (set) {
                                conflict=true;
                            }
                            makeAssociation(ret, fl, o);
                            set=true;
                        }
                    }
                    if (conflict) {
                        throw new XMLizableException("Non-simple association - tag = "+child.getName());
                    }
                    if (!set) {
                        //find the best collection wich can store such object
                        fields=utils.getAllCollectionFields(ret);
                        Collection bestcol=null;
                        double rate=0.0;
                        for (int k=0;k < fields.size();k++) {
                            Field fl=(Field)(fields.get(k));
                            Collection col=(Collection)(fl.get(ret));
                            if (fields.size() == 1) { //is the only collection
                                bestcol=col;
                                break;
                            }
                            double rt=0.0;
                            Collection cand=null;
                            //try to find collection with similar content
                            Class c1=(Class)(objects.get(col));
                            if (c1 != null && c1.isAssignableFrom(cla)) {
                                rt=0.6;
                                cand=col;
                            }
                            //find collection with similar name - heuristic
                            if (utils.isSimilar(fl.getName(), child.getName(), 0.7)) {
                                rt+=0.4;
                                cand=col;
                            }
                            if (cand != null) {
                                if (bestcol != null) {
                                    if (rate < rt) {
                                        bestcol=cand;
                                        rate=rt;
                                    }
                                } else {
                                    bestcol=cand;
                                    rate=rt;
                                }
                            }
                        }
                        if (bestcol != null)
                            makeAssociation(bestcol, ret, o);
                    }
                }
            }
        }
        
        if (tag != rootTag) {
            //solve waiting links
            String key=getObjectKey((XMLizable)ret, tag);
            for (int i=0;i < waitings.size();i++) {
                WaitingLink wl=(WaitingLink)(waitings.get(i));
                if (Collection.class.isAssignableFrom(wl.field.getType())) { //collection
                    Collection col=(Collection)(wl.field.get(wl.object));
                    if (wl.id.equals(key)) {
                        ArrayList list=(ArrayList)(blinds.get(col));
                        if (list == null) {
                            list=new ArrayList(32);
                            blinds.put(col, list);
                        }
                        list.add(new WaitingBlindLink(key, (Collection)(wl.field.get(wl.object)), (XMLizable)ret, wl.object));
                        waitings.remove(wl);
                        i--;
                    }
                } else { //XMLizable
                    if (wl.id.equals(key) && wl.field.getType().isAssignableFrom(ret.getClass()) &&
                            wl.field.get(wl.object) == null) { //possible link
                        makeAssociation(wl.object, wl.field, ret);
                        waitings.remove(wl);
                        i--;
                    }
                }
            }
        }
        objects.put(getObjectKey((XMLizable)ret, tag) ,ret);
        return ret;
    }
    
    protected boolean canParseTag(Tag tag) {
        if (filterNameSpace == null) return true;
        String ns=tag.getNameSpace();
        if (ns == null || ns.length() == 0) return true;
        for (String n:filterNameSpace) {
            if (n.equals(ns))
                return true;
        }
        return false;
    }
    
    protected void callInitMethods() throws Exception {
        Object[] objs=objects.values().toArray();
        for (int i=0;i < objs.length;i++) {
            try {
                if (objs[i] instanceof XMLizable) {
                    XMLizable obj=(XMLizable)objs[i];
                    utils.callInitMethod(obj);
                }
            } catch (ClassCastException ex) {
                throw new ClassCastException(ex.getMessage()+" does not implement interface XMLizable.");
            }
        }
    }
    
    protected void solveBlindLinks() throws Exception {
        //remove blind links
        Object[]/*<String>*/ keys=blinds.keySet().toArray();
        /*for (String key : keys) {*/
        for (int i=0;i < keys.length;i++) {
            Collection col=(Collection)(keys[i]);
            ArrayList/*<WaitingBlindLink>*/ bs=(ArrayList)(blinds.remove(col));
            ArrayList t=new ArrayList(32);
            if (bs != null) {
                /*for (WaitingBlindLink lnk : bs) {*/
                for (int j=0;j < bs.size();j++) {
                    WaitingBlindLink lnk=(WaitingBlindLink)(bs.get(j));
                    if (!col.contains(lnk.object)) {
                        if (t.contains(lnk.id)) {
                            throw new IllegalStateException("Trying to add different object with the same ID: "+lnk);
                        }
                        makeAssociation(col, lnk.source, lnk.object);
                        t.add(lnk.id);
                    }
                }
            }
        }
        //remove all remaining waiting links
        for (int i=0;i < waitings.size();i++) {
            WaitingLink wl=(WaitingLink)(waitings.get(i));
            Object obj=objects.get(wl.id);
            /*if (obj == null) {
                throw new IllegalStateException("Remains non-collection links: "+wl);
            }*/
            //all remaining links should be to collections - probabbly primitive types
            if (Collection.class.isAssignableFrom(wl.field.getType())) {
                try {
                    Collection col=(Collection)(wl.field.get(wl.object));
                    if (obj != null)
                        col.add(obj);
                    else
                        col.add(wl.id); //risky
                } catch (Exception ex) {
                    System.err.println("the collection "+wl.field.getName()+" of "+wl.object.getClass().getName()+" was not initialized (made it in default constructor)");
                    ex.printStackTrace();
                }
            } else {
                try {
                    wl.field.set(wl.object, obj);
                } catch (Exception ex) {
                    ex.printStackTrace();
                    throw new IllegalStateException("Cannot set remaining link: "+wl);
                }
            }
        }
    }
    
    protected void makeBackAssociation(Object object, Object associted) throws Exception {
        Field fld=utils.getBackComposition(associted.getClass(), object.getClass());
        if (fld != null) {
            fld.set(associted, object);
        }
    }
    
    protected void makeAssociation(Object source, Field sourcefield, Object target) throws Exception {
        if (!sourcefield.getType().isAssignableFrom(target.getClass())) {
            throw new IllegalStateException("source-"+source+" field-"+sourcefield.getName()+" field-class-"+sourcefield.getDeclaringClass()+" target-"+target);
        }
        sourcefield.set(source, target);
        makeBackAssociation(source, target);
    }
    
    protected void makeAssociation(Collection multi, Object source, Object target) throws Exception {
        multi.add(target);
        //store usual class stored into that collection
        Class cl=(Class)(objects.get(multi));
        if (cl != null) {
            if (target.getClass().isAssignableFrom(cl)) {
                cl=target.getClass();
            }
        } else {
            cl=target.getClass();
        }
        objects.put(multi, cl);
        makeBackAssociation(source, target);
    }
    
    protected String getObjectKey(Object obj, Tag tag) {
        return /*obj.getClass().getName()+*/getObjectID(obj, tag);
    }
    
    protected String getObjectID(Object obj, Tag tag) {
        String id=tag.getAttribute(XMLUtils.ID_ATTRIBUTE);
        if (id == null) {
            id=tag.getAttribute("name");
            if (id == null) {
                id=obj.toString();
            }
        }
        //ids.put(id, obj);
        return id;
    }
    
    public void setFilterNameSpace(String[] filterNameSpace) {
        this.filterNameSpace = filterNameSpace;
    }
    
    public static class WaitingLink {
        public String id;
        public Field field;
        public XMLizable object;
        
        public WaitingLink(XMLizable obj, Field f, String value) {
            field=f;
            object=obj;
            id=value;
        }
        
        public String getKey() {
            return field.getType().getName()+id;
        }
        
        public String toString() {
            return object.getClass().getName()+" field="+field.getName()+" id="+id;
        }
    }
    
    public static class WaitingBlindLink {
        public Collection addTo;
        public String id;
        public XMLizable object;
        public XMLizable source;
        
        public WaitingBlindLink(String id, Collection addTo, XMLizable object, XMLizable source) {
            this.id=id;
            this.addTo=addTo;
            this.object=object;
            this.source=source;
        }
    }
}
