/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.xmlparser;


import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;


/**
 *
 * @author  ehucka
 */
public class DefaultDocumentFactory extends DocumentFactory {
    
    public static final String XML_TAG="<?xml version=\"1.0\" encoding=\""+XMLDocument.DEFAULT_CHARSET+"\"?>";
    
    protected DefaultDocumentFactory() {
    }
    
    public XMLDocument makeDocument(String rootTagName) {
        Tag tag = TagFactory.createTag(rootTagName);
        return makeDocument(tag);
    }
    
    public XMLDocument makeDocument(String rootTagName, String rootNameSpace) {
        Tag tag = TagFactory.createTag(rootTagName, rootNameSpace);
        return makeDocument(tag);
    }
    
    public XMLDocument makeDocument(Tag rootTag) {
        XMLDocument doc=new XMLDocument();
        doc.addElement(new Special(XML_TAG));
        doc.addElement(rootTag);
        return doc;
    }
    
    public XMLDocument parseDocument(InputStream stream) throws ParserException {
        return Parser.parse(stream);
    }
    
    public XMLDocument parseDocument(File file) throws ParserException {
        try {
            FileInputStream fis = new FileInputStream(file);
            XMLDocument doc = parseDocument(fis);
            fis.close();
            
            return doc;
        } catch (FileNotFoundException ex) {
            throw new ParserException("The file "+file.getAbsolutePath()+" not found.");
        } catch (Exception ex) {
            throw new ParserException("Problem with parsing file "+file.getAbsolutePath()+". "+ex.getMessage());
        }
    }
    
    public XMLDocument parseDocument(String text) throws ParserException {
        ByteArrayInputStream bis=new ByteArrayInputStream(text.getBytes());
        return Parser.parse(bis);
    }
}
