/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.xmlparser;

import java.io.*;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.ArrayList;



/**
 *
 * @author  ehucka
 */
public class XMLDocument implements Element {
    
    public static String DEFAULT_CHARSET = "UTF-8";
    
    ArrayList<Element> elements;
    Tag root;
    boolean parsed=false;
    
    protected XMLDocument() {
        elements=new ArrayList(8);
    }
    
    public void addElement(Element elem) {
        if (elem instanceof Value && elements.size() > 0 && elements.get(elements.size()-1) instanceof Value) {
            ((Value)(elements.get(elements.size()-1))).addValue(((Value)elem).toString());
        } else {
            elements.add(elem);
        }
        if (elem instanceof Tag) {
            if (root == null) {
                root = (Tag) elem;
            } else {
                throw new IllegalArgumentException("Document cannot has 2 root tags: "+elem);
            }
        }
    }
    
    public int getElementSize() {
        return elements.size();
    }
    
    public Element getElement(int index) {
        return elements.get(index);
    }
    
    public Tag getRoot() {
        return root;
    }
    
    public void print(File file) {
        try {
            FileChannel fch = new FileOutputStream(file).getChannel();
            String content = write("");
            byte[] buff = content.getBytes(DEFAULT_CHARSET);
            content = null;
            ByteBuffer bb = ByteBuffer.wrap(buff);
            fch.write(bb);
            fch.close();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public Element copy() {
        XMLDocument doc=new XMLDocument();
        doc.setParsed(isParsed());
        for (Element el:elements) {
            doc.addElement(el.copy());
        }
        return doc;
    }
    
    public String toString() {
        return write("");
    }
    
    public void append(StringBuilder sb, String context) {
        for (Element el:elements) {
            if (!parsed && !(el instanceof Value) && !(el instanceof Tag) && context != null) {
                sb.append(context);
            }
            el.append(sb, context);
            if (!parsed)
                sb.append("\n");
        }
    }
    
    public String write(String context) {
        StringBuilder sb=new StringBuilder();
        append(sb, context);
        return sb.toString();
    }
    
    public boolean isParsed() {
        return parsed;
    }
    
    public void setParsed(boolean parsed) {
        this.parsed = parsed;
        if (!parsed) {
            for (int i = 0; i < elements.size(); i++) {
                if (elements.get(i) instanceof Value && ((Value)(elements.get(i))).trim()) {
                    elements.remove(i--);
                } else if (elements.get(i) instanceof Tag) {
                    ((Tag)(elements.get(i))).setParsed(false);
                }
            }
        }
    }
}
