<?xml version="1.0"?>
<!--
DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.

Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.


The contents of this file are subject to the terms of either the GNU
General Public License Version 2 only ("GPL") or the Common
Development and Distribution License("CDDL") (collectively, the
"License"). You may not use this file except in compliance with the
License. You can obtain a copy of the License at
http://www.netbeans.org/cddl-gplv2.html
or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
specific language governing permissions and limitations under the
License.  When distributing the software, include this License Header
Notice in each file and include the License file at
nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
particular file as subject to the "Classpath" exception as provided
by Sun in the GPL Version 2 section of the License file that
accompanied this code. If applicable, add the following below the
License Header, with the fields enclosed by brackets [] replaced by
your own identifying information:
"Portions Copyrighted [year] [name of copyright owner]"

Contributor(s):

The Original Software is NetBeans. The Initial Developer of the Original
Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
Microsystems, Inc. All Rights Reserved.

If you wish your version of this file to be governed by only the CDDL
or only the GPL Version 2, indicate your decision by adding
"[Contributor] elects to include this software in this distribution
under the [CDDL or GPL Version 2] license." If you do not indicate a
single choice of license, a recipient has the option to distribute
your version of this file under either the CDDL, the GPL Version 2 or
to extend the choice of license to its licensees as provided above.
However, if you add GPL Version 2 code and therefore, elected the GPL
Version 2 license, then the option applies only if the new code is
made subject to such option by the copyright holder.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
                xmlns:xslstring="xalan://java.util.HashMap"
                xmlns:df="xalan://java.text.SimpleDateFormat"
                xmlns:date="xalan://java.util.Date" >
    <!-- xmlns:html="http://www.w3.org/1999/xhtml" -->
    
    <xsl:variable name="lastVar" select="xslstring:new()"/>
    <xsl:template match="/">
        <html>
            <title><xsl:value-of select="test-spec/@module"/> Test Specification</title>
            <head>
                <link rel="stylesheet" type="text/css" href="http://wiki.netbeans.org/wiki/templates/default/jspwiki.css"/>
                <link rel="alternate stylesheet" type="text/css" href="/wiki/templates/default/jspwiki.css" title="Standard"/>
                <style type="text/css">
                    td {
                    padding-right: 4px;
                    font-size: 12px;
                    vertical-align: top;
                    text-align: left;
                    }
                </style>
                <xsl:apply-templates select="test-spec/header"/>
            </head>
            <body style="margin: 10px;">
                <h2><font class="para"><xsl:value-of select="test-spec/@module"/> Test Specification for <xsl:value-of select="test-spec/@release"/></font></h2>
                <b>Author: </b> 
                <a><xsl:attribute name="href">mailto:<xsl:value-of select="test-spec/@email"/></xsl:attribute>
                    <xsl:value-of select="test-spec/@author"/>
                </a><br/>
                <xsl:choose>
                    <xsl:when test="contains(test-spec/@version,'$Revision')"> <!-- $Revision 1.1 $-->
                        <b>Version: </b> <xsl:value-of select="substring-before(substring-after(test-spec/@version,': '),' $')"/><br/>
                    </xsl:when>
                    <xsl:otherwise>
                        <b>Version: </b> <xsl:value-of select="test-spec/@version"/><br/>
                    </xsl:otherwise>
                </xsl:choose>
                <xsl:choose>
                    <xsl:when test="contains(test-spec/@changed,'$Date')"> <!-- $Date 2003/07/10 17:21:36 $-->
                        <b>Last update: </b> <xsl:value-of select="substring-before(substring-before(substring-after(test-spec/@changed,': '),' $'),' ')"/><br/>
                    </xsl:when>
                    <xsl:otherwise>
                        <b>Last update: </b> <xsl:value-of select="test-spec/@changed"/><br/>
                    </xsl:otherwise>
                </xsl:choose>
                <table cellspacing="0">
                    <tr><td><b>Introduction:</b></td><td><xsl:apply-templates select="test-spec/introduction"/></td></tr>
                    <tr><td><b>Comments:</b></td><td><xsl:apply-templates select="test-spec/comments"/></td></tr></table>
                <h4>Table of Contents</h4>
                <ul>
                    <xsl:apply-templates select="//suite" mode="toc"/>
                </ul><br/>
                <xsl:apply-templates select="//suite" mode="full"/> 
                <hr/>
                <xsl:variable name="dformat" select="df:new('yyyy MM dd hh:mm')"/>
                <xsl:variable name="time" select="date:new()"/>
                <font class="smalltext">Generated: <xsl:value-of select="df:format($dformat, $time)"/></font>
            </body> 
        </html>
    </xsl:template>
    
    <xsl:template match="suite" mode="toc">
        <li style="list-style: none;"><a style="font-weight: bold; text-decoration: none;">
                <xsl:attribute name="href">#<xsl:value-of select="@suite_id"/></xsl:attribute>
                <xsl:value-of select="@title"/>
        </a></li>
    </xsl:template>	
    
    <xsl:template match="suite" mode="full">
        <div>
            <h4><a><xsl:attribute name="NAME"><xsl:value-of select="@suite_id"/></xsl:attribute></a>
                <font class="para">Test suite: <xsl:value-of select="@title"/></font>
            </h4>
            <div style="display: block; padding-left: 2em;">
                <table cellspacing="0">
                <xsl:if test="./purpose != ''">
                    <tr><td><b>Purpose:</b></td><td><xsl:apply-templates select="./purpose"/></td></tr>
                </xsl:if>
                <xsl:if test="./comments != ''">
                    <tr><td><b>Comment:</b></td><td><xsl:apply-templates select="./comments"/></td></tr>
                </xsl:if>
                <xsl:if test="./setup != ''">
                    <tr><td><b>Setup:</b></td><td><xsl:apply-templates select="./setup"/></td></tr>
                </xsl:if></table>
                <xsl:if test="count(./suite-case) != 0">
                    <ol>
                        <xsl:apply-templates select="./suite-case|./var"/>
                    </ol>
                </xsl:if>
                <xsl:if test="./teardown != ''">
                    <table cellspacing="0"><tr><td><b>Teardown:</b></td><td><xsl:apply-templates select="./teardown"/></td></tr></table>
                </xsl:if>
            </div>
        </div>
    </xsl:template>
    
    <xsl:template match="suite-case">
        <xsl:variable name="reference">
            <xsl:value-of select="@ref"/>
        </xsl:variable>
        <li>
            <xsl:apply-templates select="//case[@case_id=$reference]"/>
        </li>
    </xsl:template>
    
    <xsl:template match="case">
        <span style="font-weight: bold;"><xsl:value-of select="@title"/></span><br/>
        <table cellspacing="0">
        <xsl:if test="./comments != ''">
            <tr><td>Comments:</td><td><xsl:apply-templates select="comments"/></td></tr>
        </xsl:if>
        <xsl:if test="./setup != ''">
            <tr><td>Setup:</td><td><xsl:apply-templates select="setup"/></td></tr>
        </xsl:if></table>
        <ol>
            <xsl:apply-templates select="actions/action"/>
        </ol>
        <xsl:apply-templates select="actions/result"/>
        <xsl:if test="./teardown != ''">
            <table cellspacing="0"><tr><td>Teardown:</td><td><xsl:apply-templates select="teardown"/></td></tr></table>
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="actions/action">
        <li><xsl:apply-templates/></li>
    </xsl:template>
    
    <xsl:template match="result">
        <ul><li><span class="result">EXPECTED RESULT: <xsl:apply-templates/></span></li></ul>
    </xsl:template>
    
    <xsl:template match="properties"><!--[@style='extended']">-->
        <table cols="2" border="1"><tr>
            <td colspan="2"><b><xsl:value-of select="@title"></xsl:value-of></b></td></tr>
            <xsl:apply-templates select="item"/>
        </table>
    </xsl:template>
    
    <xsl:template match="item">
        <tr><td class="smalltext"><xsl:value-of select="@name"/></td>
        <td class="smalltext"><xsl:value-of select="@value"/></td></tr>
    </xsl:template>
    
    <xsl:template match="image[@title]">        
        <table cellpadding="0" cellspacing="0" rows="2" cols="1" border="1">
            <tr><td><img>
                        <xsl:attribute name="src"><xsl:value-of select="@url"/></xsl:attribute>
            </img></td></tr>
            <tr><td><xsl:value-of select="@title"/>
        </td></tr></table>
    </xsl:template>
    
    <xsl:template match="image">
        <xsl:apply-templates select="./@title"/>
        <img>
            <xsl:attribute name="src"><xsl:value-of select="@url"/></xsl:attribute>
        </img>
    </xsl:template>
    
    <xsl:template match="include">
        <xsl:value-of select="xslstring:get($lastVar, @name)"/>
    </xsl:template>
    
    <xsl:template match="variable">
        <xsl:value-of select="xslstring:put($lastVar, atr/@ref, atr/@value)"/>
    </xsl:template>
    
    <xsl:template match="code">
        <code><i><xsl:value-of select="."/></i></code>
    </xsl:template>
    
    <xsl:template match="component">
        <i><xsl:value-of select="."/></i>
    </xsl:template>
    
    <xsl:template match="data">
        <tt>"<i><xsl:value-of select="."/></i>"</tt>
    </xsl:template>
    
    <xsl:template match="menu">
        <b><xsl:value-of select="."/></b>
    </xsl:template>
    
    <xsl:template match="path">
        <tt><i><xsl:value-of select="translate(.,'\\','/')"/></i></tt>
    </xsl:template>
    
    <xsl:template match="tree">
        <b><xsl:value-of select="."/></b>
    </xsl:template>
    
    <xsl:template match=" a | applet | b | big | body | br | caption | cite |
                  code | col | colgroup | dd | div | dl | dt | em | font | form | frame |
                  frameset | head | h1 | h2 | h3 | h4 | h5 | h6 | hr | html | i | iframe | img
                  | link | li | map | meta | noframes | ol | p | param | pre | s | script |
                  small | span | strong | style | sub | sup | td | th | title | tr | tt | u | ul |
    var | table ">
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
    
</xsl:stylesheet>
