<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
            xmlns:xslstring="xalan://java.util.HashMap">
    <!--
DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.

Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.


The contents of this file are subject to the terms of either the GNU
General Public License Version 2 only ("GPL") or the Common
Development and Distribution License("CDDL") (collectively, the
"License"). You may not use this file except in compliance with the
License. You can obtain a copy of the License at
http://www.netbeans.org/cddl-gplv2.html
or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
specific language governing permissions and limitations under the
License.  When distributing the software, include this License Header
Notice in each file and include the License file at
nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
particular file as subject to the "Classpath" exception as provided
by Sun in the GPL Version 2 section of the License file that
accompanied this code. If applicable, add the following below the
License Header, with the fields enclosed by brackets [] replaced by
your own identifying information:
"Portions Copyrighted [year] [name of copyright owner]"

Contributor(s):

The Original Software is NetBeans. The Initial Developer of the Original
Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
Microsystems, Inc. All Rights Reserved.

If you wish your version of this file to be governed by only the CDDL
or only the GPL Version 2, indicate your decision by adding
"[Contributor] elects to include this software in this distribution
under the [CDDL or GPL Version 2] license." If you do not indicate a
single choice of license, a recipient has the option to distribute
your version of this file under either the CDDL, the GPL Version 2 or
to extend the choice of license to its licensees as provided above.
However, if you add GPL Version 2 code and therefore, elected the GPL
Version 2 license, then the option applies only if the new code is
made subject to such option by the copyright holder.
-->
<!-- xmlns:html="http://www.w3.org/1999/xhtml" -->

    <xsl:variable name="lastVar" select="xslstring:new()"/>
<xsl:output  indent="no"/>
<xsl:preserve-space elements="table, tr"/>

<xsl:template match="/">
%%testspec
!!! <xsl:value-of select="test-spec/@module"/> Test Specification for <xsl:value-of select="test-spec/@release"/>
__Author:__ [<xsl:value-of select="test-spec/@author"/>|mailto:<xsl:value-of select="test-spec/@email"/>] \\
<xsl:choose>
<xsl:when test="contains(test-spec/@version,'$Revision')">__Version:__ <xsl:value-of select="substring-before(substring-after(test-spec/@version,': '),' $')"/> \\</xsl:when>
<xsl:otherwise>__Version:__ <xsl:value-of select="test-spec/@version"/> \\</xsl:otherwise>
</xsl:choose>
<xsl:choose>
<xsl:when test="contains(test-spec/@changed,'$Date')">__Lastupdate:__ <xsl:value-of select="substring-before(substring-before(substring-after(test-spec/@changed,': '),' $'),' ')"/> \\</xsl:when>
<xsl:otherwise>__Lastupdate:__ <xsl:value-of select="test-spec/@changed"/> \\</xsl:otherwise>
</xsl:choose>
__Introduction:__ <xsl:apply-templates select="test-spec/introduction"/> \\
<xsl:if test="test-spec/comments != ''">
__Comments:__     <xsl:apply-templates select="test-spec/comments"/> \\
</xsl:if>

[{TableOfContents}]
<xsl:apply-templates select="//suite"/>
%%
</xsl:template>

<xsl:template match="suite">
%%collapsebox
! Test suite: <xsl:value-of select="@title"/>
Purpose:  <xsl:apply-templates select="./purpose"/> \\
<xsl:if test="./comments != ''">
Comments: <xsl:apply-templates select="./comments"/> \\
</xsl:if>
Setup:    <xsl:apply-templates select="./setup"/> \\
<xsl:apply-templates select="./suite-case|./var"/>
<xsl:if test="./teardown != ''">
Teardown: <xsl:apply-templates select="./teardown"/> \\
</xsl:if>
%% \\
</xsl:template>

    <xsl:template match="suite-case">
        <xsl:variable name="reference">
            <xsl:value-of select="@ref"/>
        </xsl:variable>
        <xsl:apply-templates select="//case[@case_id=$reference]"/>
    </xsl:template>

<xsl:template match="case">
# %%testcase <xsl:value-of select="@title"/> %%
<xsl:if test="comments != ''">
**Comments: <xsl:apply-templates select="comments"/>
</xsl:if>
<xsl:if test="setup != ''">
**Setup:    <xsl:apply-templates select="setup"/>
</xsl:if>
<xsl:apply-templates select="actions/action"/>
**%%result EXPECTED RESULT: <xsl:apply-templates select="actions/result"/>%%
<xsl:if test="teardown != ''">
**Teardown: <xsl:apply-templates select="teardown"/>
</xsl:if>
</xsl:template>

<xsl:template match="actions/action">
##<xsl:apply-templates/>
</xsl:template>

<xsl:template match="result">
<xsl:apply-templates/>
</xsl:template>

    <xsl:template match="include">
        <xsl:value-of select="xslstring:get($lastVar, @name)"/>
    </xsl:template>

    <xsl:template match="variable">
        <xsl:value-of select="xslstring:put($lastVar, atr/@ref, atr/@value)"/>
    </xsl:template>

    <xsl:template match="code">{{{<xsl:value-of select="."/>}}}</xsl:template>

    <xsl:template match="pre">{{{<xsl:value-of select="."/>}}}</xsl:template>

    <xsl:template match="component">
        ''<xsl:value-of select="."/>''
    </xsl:template>

    <xsl:template match="data">
        "''<xsl:value-of select="."/>''"
    </xsl:template>

    <xsl:template match="menu">
__<xsl:value-of select="."/>__
    </xsl:template>

    <xsl:template match="path">
        ''<xsl:value-of select="translate(.,'\\','/')"/>''
    </xsl:template>

    <xsl:template match="tree">
__<xsl:value-of select="."/>__
    </xsl:template>

    <xsl:template match="b">__<xsl:value-of select="."/>__</xsl:template>

    <xsl:template match="i">''<xsl:value-of select="."/>''</xsl:template>

    <xsl:template match="a">[<xsl:value-of select="."/>|<xsl:value-of select="@href"/>]</xsl:template>

    <xsl:template match="th">|| <xsl:value-of select="."/> </xsl:template>

    <xsl:template match="td">| <xsl:value-of select="."/> </xsl:template>

    <xsl:template match="br"> \\
    </xsl:template>

    <xsl:template match="tr">
<xsl:apply-templates select="./th"/>
<xsl:apply-templates select="./td"/>
</xsl:template>

    <xsl:template match="ul" name="ult">
        <xsl:param name="ulevel">***</xsl:param>
        <xsl:text>
</xsl:text>
        <xsl:for-each select = "./li">
<xsl:value-of select="$ulevel" /> <xsl:apply-templates select=".">
    <xsl:with-param name="ulevel" select="concat($ulevel, '*')"/>
        </xsl:apply-templates>
        <xsl:text>
</xsl:text>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="ol" name="olt">
        <xsl:param name="olevel">###</xsl:param>
        <xsl:text>
</xsl:text>
        <xsl:for-each select = "./li">
<xsl:value-of select="$olevel" /> <xsl:apply-templates select=".">
    <xsl:with-param name="ulevel" select="concat($olevel, '#')"/>
        </xsl:apply-templates>
        <xsl:text>
</xsl:text>
        </xsl:for-each>
    </xsl:template>

    
    <xsl:template match=" applet | big | body | caption | cite |
                    col | colgroup | dd | div | dl | dt | em | font | form | frame |
                    frameset | head | h1 | h2 | h3 | h4 | h5 | h6 | hr | html | iframe | img
                  | link | map | meta | noframes | p | param | pre | s | script |
                    small | span | strong | style | sub | sup | title | tt | u | var ">
        <xsl:copy>
            <xsl:copy-of select="@*"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

</xsl:stylesheet>
