/*
                    Sun Public License Notice
 
The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/
 
The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.
 
Contributor(s): Petr Slegr.
 */

package org.netbeans.modules.regsup.auted;

import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.*;
import java.beans.*;
import java.awt.Point;
import java.awt.Color;
import org.netbeans.modules.regsup.logic.*;
import org.netbeans.modules.regsup.auted.*;
import java.awt.RenderingHints;
import org.netbeans.modules.regsup.util.Util;

/**
 *
 * @author  Petr Slegr
 * @version 24.6.01
 */
public class VisualAutomatonEdge extends VisualAutomatonElement /*implements PropertyChangeListener*/ {
    
    protected VisualAutomatonNode from;
    
    protected VisualAutomatonNode to;
    
    protected LogicAutomatonEdge logicEdge;
    
    //protected RexElement rexElem;
    
    public static final String PROP_EDGE_VALUE = new String ("edgeValue");
    
    /** For subclassing.
     */
    protected VisualAutomatonEdge () {
    }
    
    /**
     * @param from
     * @param to
     * @param logicEdge May be null if and only if from and to params are equal (in this
     * case this VisualAutomatonEdge represents "self-edge".
     */
    VisualAutomatonEdge (VisualAutomatonNode from, VisualAutomatonNode to,
    LogicAutomatonEdge logicEdge) {
        //System.out.println("panelEdge: from: " + from + " to: " + to);
        if (null == logicEdge) {
            if (! from.equals (to)) {
                throw new IllegalArgumentException ("logicEdge cannot be null when no self-edge");
            }
        }
        this.from = from;
        this.to = to;
        this.logicEdge = logicEdge;
        // !!!! asi neni potreba
        //rexElem = logicEdge.getEdgeValue ();
        setHumanName ("Edge"); // !! bundle
        geometry = prepareGeometry ();
        //
        PCListenPoint listener = new PCListenPoint ();
        from.addPropertyChangeListener (VisualAutomatonNode.PROP_POSITION, listener);
        to.addPropertyChangeListener (VisualAutomatonNode.PROP_POSITION, listener);
        /*
        from.addOutputEdge (this);
        to.addInputEdge (this);
         */
        //
        /*
        ConnectionManager.ConnectConfig cfg = from.getConnectPoints (to);
        Point conSource = cfg.first;
        Point conTarget = cfg.second;
        Line2D line = new Line2D.Double (conSource.getX (), conSource.getY (),
        conTarget.getX (), conTarget.getY ());
        EdgeGeomData edgeGeom = new EdgeGeomData (conSource, conTarget, rex.toString (),
        cfg.arrowIndex);
        geom = edgeGeom;
         */
    }
    
    public boolean isDragable () {
        return false;
    }
    
    /**
     */
    public final VisualAutomatonNode getSourceNode () {
        return from;
    }
    
    /**
     */
    public final VisualAutomatonNode getTargetNode () {
        return to;
    }
    
    public final LogicAutomatonEdge getLogicEdge () {
        return logicEdge;
    }
    
    public boolean isOn (Point p) {
        return false;
    }
    
    public boolean isOnForSelection (Point position) {
        //        double dist = ((EdgeGeometry) geometry).getDistanceFrom (position);
        //System.out.println("dist: " + dist);
        //        return dist < 6 ? true : false; // !! const;
        return geometry.isOnForSelection (position);
    }
    
    public void handleDeleteOn (Point position) {
        container.removeElement (this);
    }
    
    /** Now returns position of from node
     */
    public Point getPosition () {
        return from.getPosition ();
    }
    
    public RexEdgeElement getEdgeValue () {
        return logicEdge.getEdgeValue ();
    }
    
    void setEdgeValueAsSystem (RexEdgeElement edgeValue) {
        RexEdgeElement oldValue = getEdgeValue ();
        logicEdge.setEdgeValue (edgeValue);
        PropertyChangeEvent pce = new PropertyChangeEvent (this, PROP_EDGE_VALUE,
        oldValue, edgeValue);
    }
    
    public void setEdgeValue (RexEdgeElement edgeValue) {
        RexEdgeElement oldValue = getEdgeValue ();
        if (edgeValue.equals (oldValue)) {
            return;
        }
        logicEdge.setEdgeValue (edgeValue);
        PropertyChangeEvent pce = new PropertyChangeEvent (this, PROP_EDGE_VALUE,
        oldValue, edgeValue);
        VisualAutomatonChange edit = new VisualAutomatonChange (
        container, pce);
        container.fireUndoableEdit (edit);
        container.fireContentChanged (edit);
        propChangeSup.firePropertyChange (pce);
    }
    
    protected ElementGeometry prepareGeometry () {
        ElementGeometry res;
        if (! from.equals (to)) {
            res = new EdgeGeometry (this);
        } else {
            res = new SelfEdgeGeometry (from);
        }
        return res;
    }
    
    public String toString () {
        // !! self-edge has logicEdge null
        if (null != logicEdge) {
            return "<vedge: " + from.toString () + "," + to.toString () + "," + logicEdge.getEdgeValue () +">";
        } else {
            return "<vedge: " + from.toString () + "," + from.toString () + "," + from.getLogicNode ().getSelfEdge () +">";
        }
    }
    
    ////////////////////////////////////////////////////////////////////////////
    protected static class SelfEdgeGeometry extends ElementGeometry {
        
        protected CubicCurve2D curve;
        
        protected GeneralPath arrow;
        
        protected VisualAutomatonNode from;
        
        // coordinates of curve control points. They are also used as coords of select points
        protected double cont1x;
        
        protected double cont1y;
        
        protected double cont2x;
        
        protected double cont2y;
        
        // coordinates of second connect point. Used also for drawing of arrow.
        // !!! neni nutne
        protected double node2x;
        
        protected double node2y;
        
        double baseAlfa = 0.52359877559829887307710723054658; // 30 degrees :o)))
        
        double alfa = Math.PI + baseAlfa;
        
        //
        double textX, textY;
        
        /**
         */
        SelfEdgeGeometry (VisualAutomatonNode from) {
            this.from = from;
            double cosAlfa = Math.cos (baseAlfa);
            double sinAlfa = Math.sin (baseAlfa);
            double radius = from.size / 2.0;
            double x = sinAlfa * radius;
            double y = cosAlfa * radius;
            double posX = from.getPosition ().getX ();
            double posY = from.getPosition ().getY ();
            double node1x = posX + radius - x;
            double node1y = posY + radius - y ;
            node2x = posX + radius + x;
            node2y = posY + radius - y ;
            double relContX = 1.5 * from.size;
            double relContY = 3.5 * radius;
            cont1x = posX + radius - relContX;
            cont1y = cont2y = posY + radius - relContY;
            cont2x = posX + radius + relContX;
            curve = new CubicCurve2D.Double (node1x, node1y, cont1x, cont1y, cont2x, cont2y, node2x, node2y);
            //
            arrow = prepareArrow (alfa, node2x, node2y);
            //
            textX = posX + radius;
            textY = cont1y;
        }
        
        /**
         */
        public boolean isOn (Point position) {
            return false;
        }
        
        public boolean isOnForSelection (Point position) {
            return curve.contains (position);
        }
        
        /**
         */
        public void paint (Graphics2D g) {
            g.draw (curve);
            g.fill (arrow);
            g.drawString (from.getLogicNode ().getSelfEdge ().getEdgeValue ().toString (),
            (float) textX, (float) textY);
        }
        
        void paintSelected (Graphics2D g) {
            paint (g);
            paintSelectSquare (g, cont1x, cont1y);
            paintSelectSquare (g, cont2x, cont2y);
        }
        
    }
    
    ////////////////////////////////////////////////////////////////////////////
    static class EdgeGeometry extends ElementGeometry {
        
        protected Line2D line;
        
        protected GeneralPath arrow;
        
        protected double textX, textY;
        
        /** From-point
         */
        protected Point first;
        
        /** To-point
         */
        protected Point second;
        
        /** Precalculated point for painting this edge
         */
        protected Point middleSelectPoint;
        
        private boolean centerFirst;
        
        private boolean centerSecond;
        
        private Point arrowX;
        
        private Point arrowY;
        
        protected VisualAutomatonNode from;
        
        protected VisualAutomatonNode to;
        
        protected RexEdgeElement rexValue;
        
        public static final int ARROW_B = 5;
        
        public static final int ARROW_A = 9;
        
        protected Point a1Point;
        
        protected Point a2Point;
        // debug
        private Line2D a1Line;
        
        private Line2D a2Line;
        
        //private Shape vCircle;
        
        //private Shape a1Circle;
        
        /*
        EdgeGeometry () {
            line = new Line2D.Double (from.getGeometry ().getCenter (),to.getGeometry ().getCenter ());
            Rectangle2D r1 = line.getBounds2D ();
            Rectangle2D r2 = from.getGeometry ().shape.getBounds2D ();
            rn = new Rectangle2D.Double ();
            r1.intersect (r1, r2, rn);
            System.out.println("intersect: " + rn);
        }
         */
        
        /** Finds point from points array that is the closest to <code>point</code> param.
         * @return index to points array specifying the closest point
         */
        private int findClosest (Point[] points, Point point) {
            double minDistance = Double.MAX_VALUE;
            int minIndex = 0;
            for (int i = 0; i < points.length; i++) {
                double distance = points[i].distanceSq (point);
                if (distance < minDistance) {
                    minDistance = distance;
                    minIndex = i;
                }
            }
            return minIndex;
        }
        
        /**
         * @param snaps1 snap point of from-node
         * @param snaps2 snap point of to-node
         */
        private void setSnapXsnap (Point[] snaps1, Point[] snaps2) {
            int minIndex1, minIndex2;
            minIndex1 = findClosest (snaps1, snaps2[0]);
            minIndex2 = findClosest (snaps2, snaps1[minIndex1]);
            first = snaps1[minIndex1];
            second = snaps2[minIndex2];
        }
        
        /* NOT FUNC
         */
        private void setSnapXcenter (Point[] snaps, Point center) {
            first  = snaps [findClosest (snaps, center)];
            second = center;
            centerSecond = true;
        }
        
        /**
         * @param centerFrom is center-connecting from-node (otherwise to-node)?
         */
        /* NOT FUNC
        private void setSnapXcenter (boolean centerFrom, Point[] snaps) {
            if (centerFrom) {
                second = snaps [findClosest (snaps, from.getGeometry().getCenter ())];
                first = ((VisualAutomatonNode.NodeGeometry) from.getGeometry ()).getConnectPoint (second);
            } else {
                first = snaps [findClosest (snaps, to.getGeometry().getCenter ())];
                second = ((VisualAutomatonNode.NodeGeometry) to.getGeometry ()).getConnectPoint (first);
            }
        }
         */
        
        /**
         */
        /* NOT FUNC
        private void setCenterXcenter () {
            first = ((VisualAutomatonNode.NodeGeometry) from.getGeometry ()).getConnectPoint (
            to.getGeometry ().getCenter ());
            second = ((VisualAutomatonNode.NodeGeometry) to.getGeometry ()).getConnectPoint (
            from.getGeometry ().getCenter ());
        }
         */
        
        /** Prepares arrow.
         * Also prepares middle select point.
         * Also prepares text placing point.
         * Params are not used now.
         */
        private void setArrow (boolean center1, boolean center2) {
            //System.out.println("first: " + first);
            //System.out.println("second: " + second);
            double a = second.getX () - first.getX ();
            double secondY = second.getY ();
            double firstY = first.getY ();
            double b;
            b = - (secondY - firstY); // change to Cartesian system
            //System.out.println("a: " + a);
            //System.out.println("b: " + b);
            double r = Math.sqrt (Math.pow (a, 2) + Math.pow (b, 2));
            double cosAlfa = a / r;
            double sinAlfa = b / r;
            //System.out.println("cosAlfa: " + cosAlfa);
            //System.out.println("sinAlfa: " + sinAlfa);
            //double ARROW_A = 10;
            double rs = r - ARROW_A;
            double vpx = rs * cosAlfa;
            double vpy = rs * sinAlfa;
            // change back to Java system
            PointPlus vRelPoint = new PointPlus (vpx, - vpy);
            //System.out.println("vRelPoint: " + vRelPoint);
            Point vPoint = vRelPoint.add (first);
            //System.out.println("vPoint: " + vPoint);
            //vCircle = new Ellipse2D.Double (vPoint.getX (), vPoint.getY (), 5, 5);
            //double arrB = 6;
            //a1Circle = new Ellipse2D.Double (a1Point.getX (), a1Point.getY (), 5, 5);
            double x = ARROW_B * sinAlfa;
            double y = ARROW_B * cosAlfa; // ! note: x ~ sin, y ~ cos
            PointPlus a1RelPoint = new PointPlus (x, y);
            a1Point = a1RelPoint.add (vPoint);
            x = -x;
            y = -y;
            PointPlus a2RelPoint = new PointPlus (x, y);
            a2Point = a2RelPoint.add (vPoint);
            arrow = new GeneralPath ();
            arrow.moveTo ((float) second.getX (), (float) second.getY ());
            arrow.lineTo ((float) a1Point.getX (), (float) a1Point.getY ());
            arrow.lineTo ((float) a2Point.getX (), (float) a2Point.getY ());
            arrow.closePath ();
            //
            double rhalf = r / 2.0;
            double rmsx = rhalf * cosAlfa;
            double rmsy = rhalf * sinAlfa;
            // change to Java system
            PointPlus relMiddleSelectPoint = new PointPlus (rmsx, - rmsy);
            middleSelectPoint = relMiddleSelectPoint.add (first);
            // text
            textX = middleSelectPoint.getX () + 0; // was 10
            textY = middleSelectPoint.getY () + 10;
            // debug
            /*
            a1Line = new Line2D.Double (a1Point, vPoint);
            a2Line = new Line2D.Double (a2Point, vPoint);
             */
        }
        
        protected EdgeGeometry (VisualAutomatonEdge edge) {
            this.from = edge.from;
            this.to = edge.to;
            this.rexValue  = edge.getEdgeValue ();
            //
            int fromPolicy = ((ConnectableGeometry) from.getGeometry ()).getConnectPolicy ();
            int toPolicy = ((ConnectableGeometry) to.getGeometry ()).getConnectPolicy ();
            //
            if (fromPolicy == ConnectableGeometry.CONNECT_POLICY_SNAPS) {
                Point[] snaps1 = ((ConnectableGeometry) from.getGeometry ()).getSnapPoints ();
                if (toPolicy == ConnectableGeometry.CONNECT_POLICY_SNAPS) {
                    Point[] snaps2 = ((ConnectableGeometry) to.getGeometry ()).getSnapPoints ();
                    setSnapXsnap (snaps1, snaps2);
                }
                else if (toPolicy == ConnectableGeometry.CONNECT_POLICY_CENTERS) {
                    /*
                    setSnapXcenter (false, snaps1);
                     */
                }
                /*
                else {
                    throw new IllegalStateException ("Unknown connect policy");
                }
                 */
            }
            else if (fromPolicy == ConnectableGeometry.CONNECT_POLICY_CENTERS) {
                if (toPolicy == ConnectableGeometry.CONNECT_POLICY_SNAPS) {
                    Point[] snaps2 = ((ConnectableGeometry) to.getGeometry ()).getSnapPoints ();
                    //setSnapXcenter (snaps2, from.getGeometry ().getCenter ());
                    /*
                    setSnapXcenter (true, snaps2);
                     */
                }
                else if (toPolicy == ConnectableGeometry.CONNECT_POLICY_CENTERS) {
                    /*
                    setCenterXcenter ();
                     */
                }
                /*
                else {
                    throw new IllegalStateException ("Unknown connect policy");
                }
                 */
            } else {
                throw new IllegalStateException ("Unknown connect policy");
            }
            line = new Line2D.Double (first, second);
            setArrow (false, false);
        }
        
        public void paint (Graphics2D g) {
            // !!!!!???? uz se dela globalne v panelu?
            //g.setRenderingHint (RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
            Color old = g.getColor ();
            g.setColor (Color.lightGray); // !! const
            g.draw (line);
            g.setColor (Color.black); // !! const
            g.fill (arrow);
            g.drawString (rexValue.toString (), (float) textX, (float) textY);
            g.setColor (old);
            // debug
            //g.draw (vCircle);
            //g.draw (a1Circle);
            //g.draw (a1Line);
            //g.draw (a2Line);
        }
        
        void paintSelected (Graphics2D g) {
            paint (g);
            paintSelectSquare (g, first.getX (), first.getY ());
            paintSelectSquare (g, middleSelectPoint.getX (), middleSelectPoint.getY ());
            paintSelectSquare (g, second.getX (), second.getY ());
        }
        
        public void paintActive (Graphics2D g) {
            Color old = g.getColor ();
            g.setColor (Color.green); // !! const
            g.draw (line);
            g.setColor (Color.black); // !! const
            g.fill (arrow);
            g.drawString (rexValue.toString (), (float) textX, (float) textY);
            g.setColor (old);
        }
        
        public boolean isOn (Point position) {
            throw new UnsupportedOperationException ();
        }
        
        public boolean isOnForSelection (Point position) {
            boolean b1 = line.ptSegDist (position) < 6;
            // ! do not use Line.contains (), returns false always
            // ! do not use Line.ptDistLine (), threats line as infinitively long
            boolean b2 = arrow.contains (position);
            return b1 || b2;
        }
    }
    
    ///////////////////////////////////////////////////////////////////////////
    /** Not used now
     */
    /*
    private class CurvedEdgeGeometry extends EdgeGeometry {
     
        void paint (Graphics2D g) {
        }
     
    }
     */
    
    ////////////////////////////////////////////////////////////////////////////
    private class PCListenPoint implements PropertyChangeListener {
        public void propertyChange (PropertyChangeEvent event) {
            geometry = prepareGeometry ();
        /*
        if (! propertyChangeEvent.getPropertyName ().equals (VisualAutomatonNode.PROP_POSITION))
            return;
        Point position = (Point) propertyChangeEvent.getNewValue ();
        if (propertyChangeEvent.getSource () == from || propertyChangeEvent.getSource () == to) {
         
        ConnectionManager.ConnectConfig cfg = from.getConnectPoints (to);
        Point conSource = cfg.first;
        Point conTarget = cfg.second;
         
            ((EdgeGeomData) geom).update (conSource, conTarget, rexElem.toString (),
            cfg.arrowIndex);
         
        }
        else
            throw new IllegalStateException ("event from strange source");
        AutomatonCanvasManager.getDefault ().addChangedElement (this);
         */
        }
    }
    
    
}
