/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.auted.actions;

import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CallableSystemAction;
import org.netbeans.modules.regsup.auted.*;
import org.netbeans.modules.regsup.logic.*;
import java.awt.Point;

/** Creates edge between two selected nodes.
 *
 * @author  Petr Slegr
 */
public class CreateEdgeAction extends ElementAction {

    public void performAction (VisualAutomatonElement[] selected) {
        VisualAutomatonNode from = (VisualAutomatonNode) selected[0];
        VisualAutomatonNode to = (VisualAutomatonNode) selected[1];
        VisualAutomatonContainer container = from.getContainer ();
        VisualAutomatonContainer cont2 = to.getContainer ();
        if (! container.equals (cont2)) {
            throw new IllegalStateException ("Nodes are not in the same container");
        }
        AutomatonEditor editor = MainManager.getDefault ().createEditor (container);
        Point current = editor.getCurrentPosition ();
        Point dest = new PointPlus (40, 40).add (current);
        // !!!! pozor kdyz vrati null
        RexEdgeElement rexEdgeValue = UserSupport.letUserEditEdgeValue (null, dest);
        container.createEdge (from, to, rexEdgeValue);
        // !!!!! delat automaticky a ne v kazde akci
        editor.resetSelectedElements ();
    } 

    public String getName () {
        //return NbBundle.getMessage (CreateEdgeAction.class, "CTL_CREATE_EDGE_ACTION");
        return "Create Edge";
    }

    protected String iconResource () {
        return "ActionIcon.gif";
    }

    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
        // If you will provide context help then use:
        // return new HelpCtx (AddNodeActionAction.class);
    }

    /** Test whether the action is currently enabled.
     * @return <code>true</code> if so
     */
    public boolean isEnabled (VisualAutomatonElement[] selected) {
        if (selected.length != 2) {
            return false;
        }
        if (! (selected[0] instanceof VisualAutomatonNode)) {
            return false;
        }
        if (! (selected[1] instanceof VisualAutomatonNode)) {
            return false;
        }
        return ! ((VisualAutomatonNode) selected[0]).hasOutputEdgeTo ((VisualAutomatonNode) selected[1]);
    }
    
    /** Perform extra initialization of this action's singleton.
     * PLEASE do not use constructors for this purpose!
    protected void initialize () {
	super.initialize ();
     * putProperty (Action.SHORT_DESCRIPTION, NbBundle.getMessage (AddNodeActionAction.class, "HINT_Action"));
    }
    */

}
