/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.vcs.cmdline;

import javax.swing.event.*;
import javax.swing.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import org.openide.util.*;
import org.openide.*;

import org.netbeans.modules.vcscore.util.*;

import org.netbeans.modules.vcs.cmdline.passwd.*;

/** Panel in the NewCvsCustomizer which takes care of CVS server settings.
 * After the user fills in all info needed and the type of server is pserver, looks in the .cvspass file for settings.
 * If not found, it attempts to connect to the pserver and prompts for password.
 * If successful, the current settings are written to the .cvspass file.
 * @author mkleint
 * @version 1.0
 */
public class Cust2ServerPanel extends javax.swing.JPanel implements CvsCustomPanel{

    private WizardDescriptor.Panel wdp;
    private NewCvsCustomizer customizer;
    private CvsFileSystem fs;
    private transient CVSPasswd passFile;
    boolean isLastPanel;
    //boolean loginCancelled = false;
    
    private ArrayList listeners = new ArrayList ();

    static final long serialVersionUID = 754889716014516133L;

    /** Creates new form Cust2Server */
    public Cust2ServerPanel (WizardDescriptor.Panel p) {
        this.wdp = p;
        initComponents ();
        javax.swing.ButtonGroup bg = new javax.swing.ButtonGroup ();
        bg.add (rbLocal);
        bg.add (rbServer);
        bg.add (rbPserver);
        bg.add (rbExtServer);
        /*
        passFile = new CVSPasswd();  // this way or has to be rewritten to suit all OS.
        passFile.loadPassFile();
         */
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        rbLocal = new javax.swing.JRadioButton();
        rbServer = new javax.swing.JRadioButton();
        rbPserver = new javax.swing.JRadioButton();
        rbExtServer = new javax.swing.JRadioButton();
        txServer = new javax.swing.JTextField();
        txUser = new javax.swing.JTextField();
        txRepository = new javax.swing.JTextField();
        btnRepository = new javax.swing.JButton();
        lbServer = new javax.swing.JLabel();
        lbUser = new javax.swing.JLabel();
        lbRepository = new javax.swing.JLabel();
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        setDebugGraphicsOptions(javax.swing.DebugGraphics.NONE_OPTION);
        
        rbLocal.setSelected(true);
        rbLocal.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("NewCvsCustomizer.rbLocal.text"));
        rbLocal.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                rbLocalActionPerformed(evt);
            }
        }
        );
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(rbLocal, gridBagConstraints1);
        
        
        rbServer.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("NewCvsCustomizer.rbServer.text"));
        rbServer.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                rbServerActionPerformed(evt);
            }
        }
        );
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 2;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.insets = new java.awt.Insets(5, 12, 0, 1);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(rbServer, gridBagConstraints1);
        
        
        rbPserver.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("NewCvsCustomizer.rbPserver.label"));
        rbPserver.setLabel(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("NewCvsCustomizer.rbPserver.label"));
        rbPserver.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                rbPserverActionPerformed(evt);
            }
        }
        );
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 3;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.insets = new java.awt.Insets(5, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(rbPserver, gridBagConstraints1);
        
        
        rbExtServer.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("Cust2ServerPanel.rbExtServer.text"));
        rbExtServer.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                rbExtServerActionPerformed(evt);
            }
        }
        );
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.insets = new java.awt.Insets(5, 12, 0, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(rbExtServer, gridBagConstraints1);
        
        
        txServer.setColumns(20);
        txServer.setDebugGraphicsOptions(javax.swing.DebugGraphics.NONE_OPTION);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(11, 0, 0, 11);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(txServer, gridBagConstraints1);
        
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 6;
        gridBagConstraints1.gridwidth = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(5, 0, 0, 11);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(txUser, gridBagConstraints1);
        
        
        txRepository.setColumns(20);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 7;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(5, 0, 11, 0);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints1.weightx = 1.0;
        add(txRepository, gridBagConstraints1);
        
        
        btnRepository.setLabel(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("NewCvsCustomizer.btnRepository.label"));
        btnRepository.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnRepositoryActionPerformed(evt);
            }
        }
        );
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 2;
        gridBagConstraints1.gridy = 7;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints1.insets = new java.awt.Insets(4, 5, 11, 11);
        add(btnRepository, gridBagConstraints1);
        
        
        lbServer.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("NewCvsCustomizer.lbServer.text"));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 5;
        gridBagConstraints1.insets = new java.awt.Insets(11, 18, 0, 8);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(lbServer, gridBagConstraints1);
        
        
        lbUser.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("NewCvsCustomizer.lbUser.text"));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 6;
        gridBagConstraints1.insets = new java.awt.Insets(5, 18, 0, 8);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(lbUser, gridBagConstraints1);
        
        
        lbRepository.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("NewCvsCustomizer.lbRepository.text"));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 7;
        gridBagConstraints1.insets = new java.awt.Insets(5, 18, 11, 8);
        gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
        add(lbRepository, gridBagConstraints1);
        
    }//GEN-END:initComponents

  private void rbExtServerActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_rbExtServerActionPerformed
// Add your handling code here:
      updateEnabled();
  }//GEN-LAST:event_rbExtServerActionPerformed

  private void btnRepositoryActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnRepositoryActionPerformed
// Add your handling code here
        ChooseDirDialog chooseDir=new ChooseDirDialog(new JFrame(), new File(txRepository.getText ()));
        VcsUtilities.centerWindow (chooseDir);
        HelpCtx.setHelpIDString (chooseDir.getRootPane (), NewCvsCustomizer.class.getName ());
        chooseDir.show();
        String selected=chooseDir.getSelectedDir();
        if( selected==null ){
            //D.deb("no directory selected"); // NOI18N
            return ;
        }
        File dir=new File(selected);
        if( !dir.isDirectory() ){
            return ;
        }
        txRepository.setText(selected);
  }//GEN-LAST:event_btnRepositoryActionPerformed

  private void rbPserverActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_rbPserverActionPerformed
// Add your handling code here:
      updateEnabled();
  }//GEN-LAST:event_rbPserverActionPerformed

  private void rbServerActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_rbServerActionPerformed
// Add your handling code here:
      updateEnabled();
  }//GEN-LAST:event_rbServerActionPerformed

  private void rbLocalActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_rbLocalActionPerformed
// Add your handling code here:
      updateEnabled();
  }//GEN-LAST:event_rbLocalActionPerformed


  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JRadioButton rbLocal;
  private javax.swing.JRadioButton rbServer;
  private javax.swing.JRadioButton rbPserver;
  private javax.swing.JRadioButton rbExtServer;
  private javax.swing.JTextField txServer;
  private javax.swing.JTextField txUser;
  private javax.swing.JTextField txRepository;
  private javax.swing.JButton btnRepository;
  private javax.swing.JLabel lbServer;
  private javax.swing.JLabel lbUser;
  private javax.swing.JLabel lbRepository;
  // End of variables declaration//GEN-END:variables

    /** name of panel
     * @return the name of the current panel
     */
    public String getName() {
      return org.openide.util.NbBundle.getBundle(NewCvsCustomizer.class).getString("NewCvsCustomizer.serverDialogLabel");        
    }
    
    
    public void setLast(boolean last) {
        isLastPanel = last;
    }

    /** last panel check.
     * @return true if last else false.
     */    
    public boolean isLast() {
        D("is last");
        return isLastPanel;
    }
    
/** If everything is ok, returns the number of the Cust3CheckoutPanel.
 * Otherwise returns it's onw number, so that the user has to stay here or cancel.
 * @return the number of next panel.
 */
    public int getNext() {
       boolean [] loginCancelled = new boolean[1];
       if (!isLoggedIn(loginCancelled)) {
         //return 2;
           if (loginCancelled[0]) return 2; // return me if the login was cancelled
           fs.setOffLine(true); // set offline mode
           return 4; // skip checkout when not logged in
       }    
       if (this.rbLocal.isSelected()) { // see BUG no 6990 - needs to check if the repository is initiated
         String repos = this.txRepository.getText();
         File repDir = new File(repos);
         if (repDir.exists() && repDir.isDirectory()) {
           String[] subdirs = repDir.list();
           if (subdirs.length == 0) return 4;
           for (int i = 0; i < subdirs.length; i++) {
             if (subdirs[i].equalsIgnoreCase("CVSROOT")) return 3; // is cvs repository
           }
           return 4; // skip checkout for non-cvs or empty local repository 
         }
         else {
             javax.swing.SwingUtilities.invokeLater(new Runnable () {
                 public void run () {
                     TopManager.getDefault ().notify (new NotifyDescriptor.Message(org.openide.util.NbBundle.getBundle(Cust2ServerPanel.class).getString("NewCvsCustomizer.repositoryNonExistent")));
                 }
             });
             return 2; // non existent repository path or is not directory
         }
       }
       return 3; // show checkout panel for everything else
    }


    /** Add a listener to changes of the panel's validity.
 * @param l the listener to add
 * @see #isValid
 */
    private synchronized void addChangeListener(ChangeListener l) {
        if (listeners == null) {
          listeners = new ArrayList();
        }
        listeners.add (l);        
    }
    /** Remove a listener to changes of the panel's validity.
 * @param l the listener to remove
 */
    private synchronized void removeChangeListener(ChangeListener l) {
        if (listeners != null) {
            listeners.remove (l);
        }
    }
    
    private void fireChangeState() {
        ArrayList list;
        synchronized (this) {
            if (this.listeners == null)
                return;
            list = (ArrayList) this.listeners.clone();
        }
        ChangeEvent event = new ChangeEvent (this);
        for (int i=0; i<list.size(); i++)
            ((ChangeListener)list.get (i)).stateChanged (event);
    /*    
      Object[] list = listeners.getListenerList();
      ChangeEvent event = null;
      event = new ChangeEvent(this);
      for (int i=0; i< list.length - 1; i++){
        Object listen = list[i];
        if (listen == null) continue;  
        if (listen.getClass() == ChangeListener.class) {
          ChangeListener listener = (ChangeListener) list[i];
          listener.stateChanged(event);
        }

      }
*/
    }        
    
        
    private void updateEnabled() {
        if(rbLocal.isSelected ()) {
            txUser.setEnabled (false);
            txServer.setEnabled (false);
            lbUser.setEnabled(false);
            lbServer.setEnabled(false);
            btnRepository.setEnabled (true);
        } else {
            txUser.setEnabled (true);
            txServer.setEnabled (true);
            lbUser.setEnabled(true);
            lbServer.setEnabled(true);
            btnRepository.setEnabled (false);
        }
    }    
    
    /**
     * Find out whether the user is logged in.
     * @param loginCancelled an one-element array. When true, the login was cancelled, when false offline mode should be used.
     * @return true if the user is logged in successfully,
     *         false if the user is not logged in and offline mode should be used.
     */
    public boolean isLoggedIn(boolean [] loginCancelled) {
        if (rbPserver.isSelected()) {
            String connectStr = ":pserver:" + txUser.getText().trim()
                                + "@" + txServer.getText().trim()
                                + ":" + txRepository.getText().trim();
            if (passFile == null) {
                passFile = new CVSPasswd(fs.getCygwinPath());  // this way or has to be rewritten to suit all OS.
            }
            wdp.storeSettings(customizer);
            customizer.setChanges();
            passFile.loadPassFile();
            LoginDialog login = LoginDialog.createDialog(fs);
            login.setPserverName(txServer.getText().trim());
            login.setConnectString(connectStr);
            PasswdEntry entry = passFile.find(connectStr);
            if (entry == null) {
                login.show();
            } else {
                login.showLogging();
            }
            loginCancelled[0] = !login.isOffline();
            boolean loggedIn = login.isLoggedIn();
            /*
            if (loggedIn) {
                entry = passFile.add(connectStr, login.getPassword());
                passFile.savePassFile();
            }
             */
            return loggedIn;
        }
        return true;  // is not pserver, -> always ok.
    }    

     private static void D(String debug) {
//        System.out.println("Cust2ServPanel(): "+debug);
    }


    public static class Panel implements WizardDescriptor.Panel {

        private Cust2ServerPanel panel;
        private String name;

        public Panel (String name) {
            this.name = name;
        }

        /** Get the component displayed in this panel.
         * @return the component
         */
        public synchronized java.awt.Component getComponent() {
            if (this.panel == null) {
                this.panel = new Cust2ServerPanel (this);
                if (this.name != null)
                    this.panel.setName (this.name);
            }
            return this.panel;
        }

        /** Help for this panel.
         * When the panel is active, this is used as the help for the wizard dialog.
         * @return the help or <code>null</code> if no help is supplied
         */
        public HelpCtx getHelp() {
            return HelpCtx.DEFAULT_HELP;
        }

        /** Provides the wizard panel with the current data--either
         * the default data or already-modified settings, if the user used the previous and/or next buttons.
         * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
         * @param settings the object representing wizard panel state, as originally supplied to {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
         */
        public void readSettings(Object settings) {
            D("ReadSettings");
            if (settings instanceof NewCvsCustomizer) {
                Cust2ServerPanel p = (Cust2ServerPanel) this.getComponent();
                p.customizer = (NewCvsCustomizer)settings;
                p.fs = p.customizer.getFileSystem();
                String st = p.customizer.serverType;
                if (st != null) {
                    if (st.equals (p.fs.CVS_SERVER_LOCAL)) p.rbLocal.setSelected (true);
                    else if (st.equals (p.fs.CVS_SERVER_SERVER)) p.rbServer.setSelected (true);
                    else if (st.equals (p.fs.CVS_SERVER_PSERVER)) p.rbPserver.setSelected (true);
                    else if (st.equals (p.fs.CVS_SERVER_EXT)) p.rbExtServer.setSelected (true);
                }
                p.txServer.setText(p.customizer.serverName);
                p.txUser.setText(p.customizer.userName);
                p.txRepository.setText(p.customizer.repository);
                p.updateEnabled();
            }
        }
        /** Provides the wizard panel with the opportunity to update the
         * settings with its current customized state.
         * Rather than updating its settings with every change in the GUI, it should collect them,
         * and then only save them when requested to by this method.
         * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
         * rather, the (copy) passed in here should be mutated according to the collected changes.
         * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
         * @param settings the object representing a settings of the wizard
         */
        public void storeSettings(Object settings) {
            if (settings instanceof NewCvsCustomizer) {
                Cust2ServerPanel p = (Cust2ServerPanel) this.getComponent();
                p.customizer = (NewCvsCustomizer)settings;
                p.fs = p.customizer.getFileSystem();
                if (p.rbLocal.isSelected()) { p.customizer.serverType = p.fs.CVS_SERVER_LOCAL; }
                if (p.rbServer.isSelected()) { p.customizer.serverType = p.fs.CVS_SERVER_SERVER; }
                if (p.rbPserver.isSelected()) { p.customizer.serverType = p.fs.CVS_SERVER_PSERVER; }
                if (p.rbExtServer.isSelected()) { p.customizer.serverType = p.fs.CVS_SERVER_EXT; }
                p.customizer.serverName = p.txServer.getText();
                p.customizer.userName = p.txUser.getText();
                p.customizer.repository = p.txRepository.getText();
            }
        }
        /** Test whether the panel is finished and it is safe to proceed to the next one.
         * If the panel is valid, the "Next" (or "Finish") button will be enabled.
         * @return <code>true</code> if the user has entered satisfactory information
         */
        public boolean isValid() {
            D("isValid");
            return true;
        }

        public void addChangeListener (ChangeListener l) {
            Cust2ServerPanel p = (Cust2ServerPanel) this.getComponent();
            p.addChangeListener (l);
        }

        public void removeChangeListener (ChangeListener l) {
            Cust2ServerPanel p = (Cust2ServerPanel) this.getComponent();
            p.removeChangeListener (l);
        }

    }

}
