/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.vcs.cmdline;

import org.openide.*;
import java.util.*;
import javax.swing.event.ChangeListener;
import java.awt.Panel;
import javax.swing.JPanel;
import javax.swing.event.*;
import java.awt.event.*;


/** The iterator (WizardDescriptor.Iterator) for the NewCvsCustomizer
 *
 * @author mkleint
 * @version 1.0
 */
public class CustomizerWizard implements WizardDescriptor.Iterator {
    private EventListenerList listeners;
    private LinkedList list;
    private WizardDescriptor.Panel current;

    private WizardDescriptor.Panel working;
    private WizardDescriptor.Panel server;
    private WizardDescriptor.Panel mount;
    private WizardDescriptor.Panel checkout;
    
    /** Creates new CustomizerWizard */
    public CustomizerWizard() {
      working = new Cust1WorkPanel.Panel (null);
      server = new Cust2ServerPanel.Panel (null);
      mount = new Cust4MountPanel.Panel (null);
      checkout = new Cust3CheckoutPanel.Panel (null);
      list = new LinkedList();
      current = working;
      list.addLast(current);
    }
    


    /** Move to the previous panel.
 * I.e. decrement its index, need not actually change any GUI itself
 *
 * Removes the current panel from the list and gets the last one displayed.
 *
 */
    public void previousPanel() {
        list.removeLast();
        current = (WizardDescriptor.Panel)list.getLast();
        ((Cust2ServerPanel)server.getComponent()).setLast(false);
    }
    /** Move to the next panel.
 * I.e. increment its index, need not actually change any GUI itself.
 * To find out which panel is the next, CvsCustomPanel.getNext() is called.
 * To the number the right panel is assigned then.
 * Then it's added to the list of panels which have been seen.
 */
    public void nextPanel() {
        WizardDescriptor.Panel newCurr = null;
        int next = ((CvsCustomPanel)current.getComponent()).getNext();
        if (next == 4) {
            int ind = list.indexOf(server);
            if (ind < 0 ) ((Cust4MountPanel)mount.getComponent()).setLast(false);
            else ((Cust4MountPanel)mount).setLast(true);
        }
        if (next == 2 && current == mount) { // we're before the second server panel
           ((Cust2ServerPanel)server).setLast(true); 
        } else {((Cust2ServerPanel)server).setLast(false); }
        if (next == 1) { newCurr = working;}
        if (next == 2) { newCurr = server;}
        if (next == 3) { newCurr = checkout;}
        if (next == 4) { newCurr = mount;}
        if (newCurr != current) {
          list.addLast(newCurr);
        }
        current = newCurr;
    }

/** Add a change listener to the iterator
 * @param l the listener to be added
 */
    public void addChangeListener(ChangeListener l) {
        if (listeners == null) {
          listeners = new EventListenerList ();
        }
        listeners.add (ChangeListener.class, l);        
    }

/** removes the listener from the list.
 * @param l the listener to be removed.
 */
    public void removeChangeListener(ChangeListener l) {
        if (listeners != null) {
            listeners.remove (ChangeListener.class, l);
        }
    }
    
   
    private void fireChangeState() {
      Object[] list = listeners.getListenerList();
      ChangeEvent event = null;
      event = new ChangeEvent(this);
      for (int i=0; i< list.length - 1; i++){
        Object listen = list[i];
        if (listen == null) continue;  
        if (listen.getClass() == ChangeListener.class) {
          ChangeListener listener = (ChangeListener) list[i];
          listener.stateChanged(event);
        }

      }
    }    
  
    
    
/** Indicates wheather a next panel exists or not.
 * @return false if it's the last one.
 */
    public boolean hasNext() {
       return (!((CvsCustomPanel)current.getComponent()).isLast());
    }


/** Get the current panel.
 * @return the panel
 */
    public WizardDescriptor.Panel current() {
        return (WizardDescriptor.Panel) current;
    }
    /** Test whether there is a previous panel.
 * @return <code>true</code> if so
 */
    public boolean hasPrevious() {
        return !current.equals(list.getFirst());
    }
    /** Get the name of the current panel.
 * @return the name
 */
    public String name() {
        return org.openide.util.NbBundle.getBundle(NewCvsCustomizer.class).getString("NewCvsCustomizer.wizardName");
    }


}